@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Algebraic_kernel_d/include/CGAL/Algebraic_kernel_d/Xy_coordinate_2.h $
+// $Id: Xy_coordinate_2.h 8e4a74d 2019-01-31T13:55:37+01:00 Andreas Fabri
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Eric Berberich <eric@mpi-inf.mpg.de>
@@ -25,7 +26,6 @@
 #define CGAL_ALGEBRAIC_CURVE_KERNEL_XY_COORDINATE_2_H
 
 #include <CGAL/basic.h>
-#include <boost/pool/pool_alloc.hpp>
 #include <boost/numeric/interval.hpp>
 
 #include <CGAL/Bbox_2.h>
@@ -219,9 +219,11 @@ public:
     /*!\brief
      * copy constructor
      */
+#ifdef DOXYGEN_RUNNING  
     Xy_coordinate_2(const Self& p) : 
         Base(static_cast<const Base&>(p)) {  
     }
+#endif  
 
     /*!\brief
      * Point at \c x, on \c curve with \c arcno. Finite points on vertical arcs
