@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Apollonius_graph_2/include/CGAL/Apollonius_graph_2.h $
+// $Id: Apollonius_graph_2.h a6c528c 2019-06-18T13:18:25+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Menelaos Karavelas <mkaravel@iacm.forth.gr>
@@ -23,12 +24,17 @@
 #ifndef CGAL_APOLLONIUS_GRAPH_2_H
 #define CGAL_APOLLONIUS_GRAPH_2_H
 
+#include <CGAL/license/Apollonius_graph_2.h>
+
+#include <CGAL/disable_warnings.h>
+
 #define CGAL_APOLLONIUS_GRAPH_PSEUDO_CIRCLE_DESIGN 1
 
 
 #include <iostream>
-#include <vector>
 #include <map>
+#include <vector>
+#include <stack>
 
 #include <boost/tuple/tuple.hpp>
 
@@ -40,7 +46,7 @@
 #include <CGAL/Apollonius_graph_vertex_base_2.h>
 
 #include <CGAL/in_place_edge_list.h>
-#include <CGAL/Segment_Delaunay_graph_2/edge_list.h>
+#include <CGAL/internal/TDS_2/edge_list.h>
 #include <CGAL/Apollonius_graph_2/Traits_wrapper_2.h>
 
 #include <CGAL/Apollonius_graph_2/Constructions_C2.h>
@@ -536,7 +542,7 @@ public:
     if ( assign(s, o) ) {
       return s;
     } else {
-      bool the_assign_statement_must_always_work(false);
+      CGAL_assertion_code( bool the_assign_statement_must_always_work(false); )
       CGAL_assertion( the_assign_statement_must_always_work );
     }
     return s;
@@ -1341,5 +1347,6 @@ std::istream& operator>>(std::istream& is,
 
 #include <CGAL/Apollonius_graph_2/Apollonius_graph_2_impl.h>
 
+#include <CGAL/enable_warnings.h>
 
 #endif // CGAL_APOLLONIUS_GRAPH_2_H
