@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Bounding_volumes/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation_impl.h $
+// $Id: Khachiyan_approximation_impl.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Kaspar Fischer <fischerk@inf.ethz.ch>
@@ -28,6 +29,7 @@
 #include <numeric>
 
 #include <CGAL/tags.h>
+#include <CGAL/use.h>
 #include <CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h>
 
 namespace CGAL {
@@ -366,7 +368,7 @@ namespace CGAL {
     CGAL_APPEL_ASSERT(!check_tag(Exact_flag()) || tau == eps/((1+eps)*d-1));
     CGAL_APPEL_ASSERT(!check_tag(Exact_flag()) || 
 		      excess<ET>(tco.cartesian_begin(*P[k])) == (1+eps)*d);
-    
+    CGAL_USE(tau);
     const FT mu = eps / ((d-1)*(1+eps));
     const FT alpha = 1 + mu;
     const FT beta = mu / (1+eps);
