@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Arrangement_on_surface_2/include/CGAL/Arr_face_index_map.h $
+// $Id: Arr_face_index_map.h 18acb2e 2018-01-18T15:05:34+00:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Ron Wein          <wein@post.tau.ac.il>
@@ -21,18 +22,18 @@
 #ifndef CGAL_ARR_FACE_INDEX_MAP_H
 #define CGAL_ARR_FACE_INDEX_MAP_H
 
+#include <CGAL/license/Arrangement_on_surface_2.h>
+
+#include <CGAL/disable_warnings.h>
+
 /*! \file
  * Definition of the Arr_face_index_map<Arrangement> class.
  */
 
 #include <CGAL/Arr_observer.h>
 #include <CGAL/Unique_hash_map.h>
-#include <boost/version.hpp>
-#if BOOST_VERSION >= 104000
-#include <boost/property_map/property_map.hpp>
-#else
-#include <boost/property_map.hpp>
-#endif
+#include <CGAL/property_map.h>
+
 #include <boost/graph/properties.hpp>
 
 namespace CGAL {
@@ -278,4 +279,6 @@ unsigned int get (const CGAL::Arr_face_index_map<Arrangement>& index_map,
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif
