@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Arrangement_on_surface_2/include/CGAL/Arr_overlay.h $
+// $Id: Arr_overlay.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Eric Berberich <eric@mpi-inf.mpg.de>
@@ -21,16 +22,18 @@
 #ifndef CGAL_ARR_OVERLAY_H
 #define CGAL_ARR_OVERLAY_H
 
+#include <CGAL/license/Arrangement_on_surface_2.h>
+
+#define CGAL_DEPRECATED_HEADER "<CGAL/Arr_overlay.h>"
+#define CGAL_REPLACEMENT_HEADER "<CGAL/Arr_overlay_2.h>"
+#include <CGAL/internal/deprecation_warning.h>
+
+#include <CGAL/Arr_overlay_2.h>
+
 /*! \file
  * Helping file to include Arr_overlay_2 for backward compatibility.
  */
 
-#if (defined __GNUC__)
-  #warning Arr_overlay.h is DEPRECATED, please include Arr_overlay_2.h instead
-#elif (defined _MSC_VER)
-  #pragma message("Arr_overlay.h is DEPRECATED, please include Arr_overlay_2.h instead")
-#endif
-
 #include <CGAL/Arr_overlay_2.h>
 
 #endif
