@@ -1,9 +1,10 @@
-// Copyright (c) 2005  Tel-Aviv University (Israel).
+// Copyright (c) 2006,2007,2009,2010,2011 Tel-Aviv University (Israel).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_walk_along_line_pl_impl.h $
-// $Id: Arr_walk_along_line_pl_impl.h 56667 2010-06-09 07:37:13Z sloriot $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Arrangement_on_surface_2/include/CGAL/Arr_point_location/Arr_walk_along_line_pl_impl.h $
+// $Id: Arr_walk_along_line_pl_impl.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Ron Wein   <wein@post.tau.ac.il>
@@ -22,6 +24,9 @@
 #ifndef CGAL_ARR_WALK_ALONG_LINE_POINT_LOCATION_IMPL_H
 #define CGAL_ARR_WALK_ALONG_LINE_POINT_LOCATION_IMPL_H
 
+#include <CGAL/license/Arrangement_on_surface_2.h>
+
+
 /*! \file
  * Member-function definitions for the
  * Arr_walk_along_line_point_location<Arrangement> class.
@@ -33,8 +38,8 @@ namespace CGAL {
 // Locate the arrangement feature containing the given point.
 //
 template <class Arrangement>
-Object Arr_walk_along_line_point_location<Arrangement>::locate
-    (const Point_2& p) const
+typename Arr_walk_along_line_point_location<Arrangement>::result_type
+Arr_walk_along_line_point_location<Arrangement>::locate(const Point_2& p) const
 {
   // Start from the initial face (as determined by the topology-traits class),
   // and an invalid halfedge representing the closest edge to p from above it
@@ -49,8 +54,7 @@ Object Arr_walk_along_line_point_location<Arrangement>::locate
   bool                                found_containing_hole;
 
   face = Face_const_handle (top_traits->initial_face());
-  do
-  {
+  do {
     // Go over the holes in the current face.
     found_containing_hole = false;
     for (ic_it = face->inner_ccbs_begin();
@@ -66,33 +70,30 @@ Object Arr_walk_along_line_point_location<Arrangement>::locate
                                                closest_to_target);
 
       // Check if the query point is located on the returned edge.
-      if (is_on_edge)
-      {
+      if (is_on_edge) {
         // Check if the point is located on one of the edge endpoints.
         if (! closest_he->source()->is_at_open_boundary() &&
             equal (p, closest_he->source()->point()))
         {
           // The query point is located on the source vertex:
           Vertex_const_handle  vh = closest_he->source();
-          return (CGAL::make_object (vh));
+          return make_result(vh);
         }
         else if (! closest_he->target()->is_at_open_boundary() &&
                  equal (p, closest_he->target()->point()))
         {
           // The query point is located on the target vertex:
           Vertex_const_handle  vh = closest_he->target();
-          return (CGAL::make_object (vh));
+          return make_result(vh);
         }
 
         // The query point is located in the edge interior:
-        return (CGAL::make_object (closest_he));
+        return make_result(closest_he);
       }
 
       // Check if the point is contained in the interior of the current hole.
-      if (is_in_face)
-      {
-        if (closest_to_target)
-        {
+      if (is_in_face) {
+        if (closest_to_target) {
           // Deal with the following scenario, where the definition of the
           // closest halfedge is not unique (all halfedges around x are
           // closest to p):
@@ -109,8 +110,9 @@ Object Arr_walk_along_line_point_location<Arrangement>::locate
           // In this case, we find the first halfegde whose target is x
           // in a clockwise direction from "6 o'clock" around x and take
           // its incident face.
-          closest_he = _first_around_vertex (closest_he->target(),
-                                             true);      // Shoot up.
+
+          // Shoot up.
+          closest_he = _first_around_vertex(closest_he->target(), true);
         }
         
         // Move inside the faces that constitute the hole, the first one
@@ -121,11 +123,9 @@ Object Arr_walk_along_line_point_location<Arrangement>::locate
 
         // Perform a vertical walk along the faces of the hole until locating
         // a face that contains the qeury point.
-        do
-        {
-          CGAL_assertion_code (
-            Halfedge_const_handle  old_closest_he = closest_he;
-          );
+        do {
+          CGAL_assertion_code
+            (Halfedge_const_handle  old_closest_he = closest_he);
 
           is_in_face = _is_in_connected_component (p, face->outer_ccb(),
                                                    true,    // Shoot up.
@@ -135,46 +135,43 @@ Object Arr_walk_along_line_point_location<Arrangement>::locate
                                                    closest_to_target);
 
           // Check if the query point was located on an edge.
-          if (is_on_edge)
-          {
+          if (is_on_edge) {
             // Check if the point is located on one of the edge endpoints.
             if (! closest_he->source()->is_at_open_boundary() &&
                 equal (p, closest_he->source()->point()))
             {
               // The query point is located on the source vertex:
               Vertex_const_handle  vh = closest_he->source();
-              return (CGAL::make_object (vh));
+              return make_result(vh);
             }
             else if (! closest_he->target()->is_at_open_boundary() &&
                      equal (p, closest_he->target()->point()))
             {
               // The query point is located on the target vertex:
               Vertex_const_handle  vh = closest_he->target();
-              return (CGAL::make_object (vh));
+              return make_result(vh);
             }
 
             // The query point is located in the edge iterior:
-            return (CGAL::make_object (closest_he));
+            return make_result(closest_he);
           }
 
           // If the point is not contained in the face, move to the neighboring
           // face from below, using the closest halfedge located so far.
-          if (! is_in_face)
-          {
-            if (closest_to_target)
-            {
+          if (! is_in_face) {
+            if (closest_to_target) {
               // The query point lies below the target vertex of the closest
               // halfedge: In this case we have to locate the first halfedge
               // we encounter when going around this target vertex from
               // "6 o'clock" (see above).
-              closest_he = _first_around_vertex (closest_he->target(),
-                                                 true);      // Shoot up.
+
+              // Shoot up.
+              closest_he = _first_around_vertex(closest_he->target(), true);
             }
 
             CGAL_assertion (old_closest_he != closest_he);
             face = closest_he->twin()->face();
           }
-
         } while (! is_in_face);
 
         // We have located a face in the hole that contains the query point.
@@ -184,7 +181,6 @@ Object Arr_walk_along_line_point_location<Arrangement>::locate
         found_containing_hole = true;
       }
     } // End loop on the current face's holes.
-
   } while (found_containing_hole);
 
   // If we reached here, we did not locate the query point in any of the holes
@@ -196,15 +192,14 @@ Object Arr_walk_along_line_point_location<Arrangement>::locate
   for (iso_verts_it = face->isolated_vertices_begin();
        iso_verts_it != face->isolated_vertices_end(); ++iso_verts_it)
   {
-    if (equal (p, iso_verts_it->point()))
-    {
+    if (equal(p, iso_verts_it->point())) {
       Vertex_const_handle  vh = iso_verts_it;
-      return (CGAL::make_object (vh));
+      return make_result(vh);
     }
   }
 
   // The query point is contained in the face interior:
-  return (CGAL::make_object (face));
+  return make_result(face);
 }
 
 //-----------------------------------------------------------------------------
@@ -212,9 +207,9 @@ Object Arr_walk_along_line_point_location<Arrangement>::locate
 // given point hits.
 //
 template <class Arrangement>
-Object Arr_walk_along_line_point_location<Arrangement>::
-_vertical_ray_shoot (const Point_2& p,
-                     bool shoot_up) const
+typename Arr_walk_along_line_point_location<Arrangement>::result_type
+Arr_walk_along_line_point_location<Arrangement>::
+_vertical_ray_shoot(const Point_2& p, bool shoot_up) const
 {
   // Start from the initial face (as determined by the topology-traits class),
   // and an invalid halfedge representing the closest edge to p from above it
@@ -230,8 +225,7 @@ _vertical_ray_shoot (const Point_2& p,
   bool                                found_containing_hole;
 
   face = Face_const_handle (top_traits->initial_face());
-  do
-  {
+  do {
     // Go over the holes in the current face.
     found_containing_hole = false;
     for (ic_it = face->inner_ccbs_begin();
@@ -248,24 +242,20 @@ _vertical_ray_shoot (const Point_2& p,
 
       // Check if the query point is located on the returned edge.
       // This can happen only if the edge is vertical.
-      if (is_on_edge)
-      {
+      if (is_on_edge) {
         CGAL_assertion (is_vertical (closest_he->curve()));
-        return (CGAL::make_object (closest_he));
+        return make_result(closest_he);
       }
 
       // Check if the point is contained in the interior of the current hole.
-      if (is_in_face)
-      {
-        if (closest_to_target)
-        {
+      if (is_in_face) {
+        if (closest_to_target) {
           // The query point lies below the target vertex of the closest
           // halfedge: In this case we have to locate the first halfedge
           // we encounter when going around this target vertex from
           // "6 o'clock" (when shooting up) or from "12 o'clock" (when
           // shooting down).
-          closest_he = _first_around_vertex (closest_he->target(),
-                                             shoot_up);
+          closest_he = _first_around_vertex(closest_he->target(), shoot_up);
         }
 
         // Move inside the faces that constitute the hole, the first one
@@ -276,8 +266,7 @@ _vertical_ray_shoot (const Point_2& p,
 
         // Perform a vertical walk along the faces of the hole until locating
         // a face that contains the qeury point.
-        do
-        {
+        do {
           CGAL_assertion_code (
             Halfedge_const_handle  old_closest_he = closest_he;
           );
@@ -291,29 +280,24 @@ _vertical_ray_shoot (const Point_2& p,
 
           // Check if the query point was located on an edge.
           // This can happen only if the edge is vertical.
-          if (is_on_edge)
-          {
+          if (is_on_edge) {
             CGAL_assertion (is_vertical (closest_he->curve()));
-            return (CGAL::make_object (closest_he));
+            return make_result(closest_he);
           }
 
           // If the point is not contained in the face, move to the neighboring
           // face from below (or above, if we shoot downward), using the
           // closest halfedge located so far.
-          if (! is_in_face)
-          {
-            if (closest_to_target)
-            {
+          if (! is_in_face) {
+            if (closest_to_target) {
               // The query point lies below the target vertex of the closest
               // halfedge: In this case we have to locate the first halfedge
               // we encounter when going around this target vertex from
               // "6 o'clock" (when shooting up) or from "12 o'clock" (when
               // shooting down).
-              closest_he = _first_around_vertex (closest_he->target(),
-                                                 shoot_up);
+              closest_he = _first_around_vertex(closest_he->target(), shoot_up);
             }
 
-
             CGAL_assertion (old_closest_he != closest_he);
             face = closest_he->twin()->face();
           }
@@ -361,13 +345,11 @@ _vertical_ray_shoot (const Point_2& p,
 
     // Check if the current isolated vertex lies closer to the query point than
     // the closest feature so far.
-    if (closest_iso_v == invalid_v)
-    {
+    if (closest_iso_v == invalid_v) {
       // Compare the current isolated vertex with the closest halfedge.
-      if (closest_he == invalid_he ||
-          closest_he->is_fictitious() ||
-          compare_y_at_x (iso_verts_it->point(),
-                          closest_he->curve()) == point_above_under)
+      if (closest_he == invalid_he || closest_he->is_fictitious() ||
+          compare_y_at_x(iso_verts_it->point(), closest_he->curve()) ==
+          point_above_under)
       {
         // The isolated vertex is closer to the query point than thr closest
         // edge found so far (note that a fictitious edge means the edge is
@@ -375,8 +357,8 @@ _vertical_ray_shoot (const Point_2& p,
         closest_iso_v = iso_verts_it;
       }
     }
-    else if (compare_xy (iso_verts_it->point(),
-                         closest_iso_v->point()) == point_above_under)
+    else if (compare_xy(iso_verts_it->point(), closest_iso_v->point()) ==
+             point_above_under)
     {
       // The current isolated vertex is closer to the query point than the
       // closest isolated vertex found so far.
@@ -384,25 +366,22 @@ _vertical_ray_shoot (const Point_2& p,
     }
   }
 
-  if (closest_iso_v != invalid_v)
-  {
+  if (closest_iso_v != invalid_v) {
     // The first object we encounter when we shoot a vertical ray from p is
     // an isolated vertex:
-    return (CGAL::make_object (closest_iso_v));
+    return make_result(closest_iso_v);
   }
 
   // If we have no halfedge above, return the initial face.
-  if (closest_he == invalid_he)
-  {
+  if (closest_he == invalid_he) {
     Face_const_handle  uf = Face_const_handle (top_traits->initial_face());
 
-    return (CGAL::make_object (uf));
+    return make_result(uf);
   }
 
   // If we reached here, closest_he is the closest edge from above (below)
   // the query point.
-  if (closest_he->is_fictitious())
-  {
+  if (closest_he->is_fictitious()) {
     // If we have a fictitious edge, the ray we shoot is completely contained
     // in an unbounded face. This face is incident to closest_he:
     if ((shoot_up && closest_he->direction() == ARR_LEFT_TO_RIGHT) ||
@@ -412,48 +391,44 @@ _vertical_ray_shoot (const Point_2& p,
     Face_const_handle  uf = closest_he->face();
 
     CGAL_assertion (! uf->is_fictitious());
-    return (CGAL::make_object (uf));
+    return make_result(uf);
   }
 
   // Check if one of closest_he's end vertices lies directly above (below) the
   // query point, and if so, return this vertex.
-  if (! is_vertical (closest_he->curve()))
-  {
+  if (! is_vertical(closest_he->curve())) {
     if (! closest_he->source()->is_at_open_boundary() &&
         compare_x (p, closest_he->source()->point()) == EQUAL)
     {
       Vertex_const_handle  vh = closest_he->source();
-      return (CGAL::make_object (vh));
+      return make_result(vh);
     }
 
     if (! closest_he->target()->is_at_open_boundary() &&
         compare_x (p, closest_he->target()->point()) == EQUAL)
     {
       Vertex_const_handle  vh = closest_he->target();
-      return (CGAL::make_object (vh));
+      return make_result(vh);
     }
   }
-  else
-  {
+  else {
     // The entire vertical segment is above (below) the query point: Return the
     // endpoint closest to it.
     const bool is_directed_up = (closest_he->direction() == ARR_LEFT_TO_RIGHT);
 
-    if ((shoot_up && is_directed_up) ||
-        (! shoot_up && ! is_directed_up))
+    if ((shoot_up && is_directed_up) || (! shoot_up && ! is_directed_up))
     {
       Vertex_const_handle  vh = closest_he->source();
-      return (CGAL::make_object (vh));
+      return make_result(vh);
     }
-    else
-    {
+    else {
       Vertex_const_handle  vh = closest_he->target();
-      return (CGAL::make_object (vh));
+      return make_result(vh);
     }
   }
 
   // The interior of the edge is closest to the query point:
-  return (CGAL::make_object (closest_he));
+  return make_result(closest_he);
 }
 
 //-----------------------------------------------------------------------------
@@ -485,99 +460,91 @@ _is_in_connected_component (const Point_2& p,
   unsigned int              n_ray_intersections = 0;
 
   typename Traits_adaptor_2::Equal_2              equal =
-                                 geom_traits->equal_2_object();
+    geom_traits->equal_2_object();
   typename Traits_adaptor_2::Compare_x_2          compare_x =
-                                 geom_traits->compare_x_2_object();
+    geom_traits->compare_x_2_object();
   typename Traits_adaptor_2::Is_vertical_2        is_vertical =
-                                 geom_traits->is_vertical_2_object();
+    geom_traits->is_vertical_2_object();
   typename Traits_adaptor_2::Compare_y_position_2 compare_y_position =
-                                 geom_traits->compare_y_position_2_object();
+    geom_traits->compare_y_position_2_object();
   typename Traits_adaptor_2::Compare_y_at_x_right_2  compare_y_at_x_right =
-                                 geom_traits->compare_y_at_x_right_2_object();
+    geom_traits->compare_y_at_x_right_2_object();
   typename Traits_adaptor_2::Compare_y_at_x_left_2   compare_y_at_x_left =
-                                 geom_traits->compare_y_at_x_left_2_object();
+    geom_traits->compare_y_at_x_left_2_object();
 
   // Start from the first non-vertical segment in the connected component.
   const Halfedge_const_handle        invalid_he;
   Ccb_halfedge_const_circulator      first = circ;
   bool                               found_non_vertical = false;
 
-  do
-  {
+  do {
     // In case of a fictitious edge, check whether it is horizontal; otherwise
     // skip it.
-    if (first->is_fictitious())
-    {
-      if (first->source()->parameter_space_in_y() != ARR_INTERIOR &&
-          first->target()->parameter_space_in_y() != ARR_INTERIOR)
+    if (first->is_fictitious()) {
+      if ((first->source()->parameter_space_in_y() != ARR_INTERIOR) &&
+          (first->target()->parameter_space_in_y() != ARR_INTERIOR))
       {
         found_non_vertical = true;
         break;
       }
-      else
-      {
+      else {
         ++first;
         continue;
       }
     }
 
     // Stop if we found a non-vertical curve.
-    if (! is_vertical (first->curve()))
-    {
+    if (! is_vertical(first->curve())) {
       found_non_vertical = true;
       break;
     }
 
-    if (inclusive)
-    {
+    if (inclusive) {
       // Check if the current vertical curve contains the query point.
       if (! first->source()->is_at_open_boundary() &&
-          compare_x (p, first->source()->point()) == EQUAL &&
-          top_traits->compare_y_at_x (p, &(*first)) == EQUAL)
+          compare_x(p, first->source()->point()) == EQUAL &&
+          top_traits->compare_y_at_x(p, &(*first)) == EQUAL)
       {
         closest_he = first;
         is_on_edge = true;
-        return (true);
+        return true;
       }
     }
-    else
-    {
+    else {
       // Check if the current vertical curve contains the query point in its
       // x-range.
       if (! first->source()->is_at_open_boundary() &&
-          compare_x (p, first->source()->point()) == EQUAL)
+          (compare_x(p, first->source()->point()) == EQUAL))
       {
         // Check if the current vertical curve contains the query point in its
         // iterior.
         const Comparison_result  res1 = 
-          top_traits->compare_xy (p, &(*(first->source())));
+          top_traits->compare_xy(p, &(*(first->source())));
         const Comparison_result  res2 =
-          top_traits->compare_xy (p, &(*(first->target())));
+          top_traits->compare_xy(p, &(*(first->target())));
         
-        if (res1 != res2)
-        {
+        if (res1 != res2) {
           if (! ((res1 == EQUAL && res2 == curve_above_under) ||
                  (res1 == curve_above_under && res2 == EQUAL)))
           {
             closest_he = first;
             is_on_edge = true;
-            return (true);
+            return true;
           }
         }
-        else if (res1 == point_above_under)
-        {
+        else if (res1 == point_above_under) {
           // Check if the vertical segment is the closest to the query point so
           // far.
           if (closest_he == invalid_he ||
               (closest_he != first->twin() &&
                ((! first->source()->is_at_open_boundary() &&
-                 top_traits->compare_y_at_x
-                     (first->source()->point(),
-                      &(*closest_he)) == point_above_under) ||
+                 (top_traits->compare_y_at_x(first->source()->point(),
+                                            &(*closest_he)) ==
+                  point_above_under)) ||
                 (! first->target()->is_at_open_boundary() &&
-                 top_traits->compare_y_at_x
-                     (first->target()->point(),
-                      &(*closest_he)) == point_above_under))))
+                 (top_traits->compare_y_at_x(first->target()->point(),
+                                            &(*closest_he)) ==
+                  point_above_under)))))
           {
             closest_he = first;
             closest_to_target =
@@ -594,11 +561,9 @@ _is_in_connected_component (const Point_2& p,
   } while (first != circ);
 
   if (! found_non_vertical)
-  {
     // In this case the entire component is comprised of vertical segments,
     // so it has an empty interior and p cannot lie inside it.
     return (false);
-  }
 
   // Go over all curves of the boundary, starting from the non-vertical curve
   // we have located, and count those which are above p.
@@ -606,56 +571,50 @@ _is_in_connected_component (const Point_2& p,
   Comparison_result   source_res, target_res;
   Comparison_result   res;
   Comparison_result   y_res;
-  bool                closest_in_ccb = (closest_he != invalid_he &&
-                                        closest_he->face() == circ->face());
+  bool closest_in_ccb =
+    ((closest_he != invalid_he) && (closest_he->face() == circ->face()));
 
-  source_res = top_traits->compare_x (p, &(*(curr->source())));
-  do
-  {
+  source_res = top_traits->compare_x(p, &(*(curr->source())));
+  do {
     // Ignore the current edge if p is not in its x-range.
-    target_res = top_traits->compare_x (p, &(*(curr->target())));
+    target_res = top_traits->compare_x(p, &(*(curr->target())));
 
-    if (source_res == target_res && source_res != EQUAL)
-    {
+    if ((source_res == target_res) && (source_res != EQUAL)) {
       ++curr;
       source_res = target_res;
       continue;
     }
 
     // Check whether p lies above or below the current edge.
-    res = top_traits->compare_y_at_x (p, &(*curr));
+    res = top_traits->compare_y_at_x(p, &(*curr));
 
-    if (res == EQUAL)
-    {
+    if (res == EQUAL) {
       // The current edge contains the query point. If the seach is inclusive
       // we return the edge. Otherwise, we return it only if it is vertical,
       // and contains p in its interior.
-      if (inclusive)
-      {
+      if (inclusive) {
         closest_he = curr;
         is_on_edge = true;
-        return (true);
+        return true;
       }
-      else
-      {
+      else {
         if (! curr->is_fictitious() &&
-            is_vertical (curr->curve()) &&
+            is_vertical(curr->curve()) &&
             (curr->source()->is_at_open_boundary() ||
-             ! equal (curr->source()->point(), p)) &&
+             ! equal(curr->source()->point(), p)) &&
             (curr->target()->is_at_open_boundary() ||
-             ! equal (curr->target()->point(), p)))
+             ! equal(curr->target()->point(), p)))
         {
           closest_he = curr;
           is_on_edge = true;
-          return (true);
+          return true;
         }
       }
     }
 
     // If the point is above the current edge (or below it, if we shoot down),
     // move to the next edge.
-    if (res != point_above_under)
-    {
+    if (res != point_above_under) {
       ++curr;
       source_res = target_res;
       continue;
@@ -664,26 +623,20 @@ _is_in_connected_component (const Point_2& p,
     // Note that we do not have count intersections (actually these are
     // overlaps) of the vertical ray we shoot with vertical segments along
     // the boundary.
-    if (curr->is_fictitious() || 
-        ! is_vertical (curr->curve()))
-    {
+    if (curr->is_fictitious() || ! is_vertical(curr->curve())) {
       // The current curve is not vertical. Check the query point is in the
       // semi-open x-range (source, target] of this curve and lies below it.
-      if (source_res != EQUAL)
-      {
-        if (closest_he == invalid_he ||
-            (! closest_in_ccb && closest_he->twin() == curr))
-        {
-          // If we have no closests halfedge, we have just found one.
-          // Alternatively, if the closest halfedge is not in our CCB, but it
-          // the twin of our current halfedge, we can take our halfedge to be
-          // the closest one.
+      if (source_res != EQUAL) {
+        if (closest_he == invalid_he || (closest_he->twin() == curr)) {
+          // 1. If we have no closests halfedge, we have just found one.
+          // 2. If the closest halfedge is the twin of our current halfedge,
+          // we can take our halfedge to be the closest one. This covers the
+          // case where our closest halfedge is not in our CCB.
           closest_he = curr;
           closest_in_ccb = true;
           closest_to_target = (target_res == EQUAL);
         }
-        else
-        {
+        else {
           // Compare with the vertically closest curve so far and detemine the
           // curve closest to p. We first check the case that the two curves
           // have a common endpoint (note that the two curves do not intersect
@@ -696,20 +649,16 @@ _is_in_connected_component (const Point_2& p,
                (closest_he->source() == curr->target() &&
                 closest_he->direction() != curr->direction())))
           {
-            if (closest_he->direction() == ARR_LEFT_TO_RIGHT)
-            {
+            if (closest_he->direction() == ARR_LEFT_TO_RIGHT) {
               // Both curves extend to the right from a common point.
-              y_res = compare_y_at_x_right (closest_he->curve(),
-                                            curr->curve(), 
-                                            closest_he->source()->point());
-            }
-            else
-            {
-              // Both curves extend to the left from a common point.
-              y_res = compare_y_at_x_left (closest_he->curve(),
-                                           curr->curve(), 
+              y_res = compare_y_at_x_right(closest_he->curve(), curr->curve(), 
                                            closest_he->source()->point());
             }
+            else {
+              // Both curves extend to the left from a common point.
+              y_res = compare_y_at_x_left(closest_he->curve(), curr->curve(), 
+                                          closest_he->source()->point());
+            }
           }
           else if (! closest_he->is_fictitious() && ! curr->is_fictitious() &&
                    ((closest_he->target() == curr->source() &&
@@ -717,23 +666,18 @@ _is_in_connected_component (const Point_2& p,
                     (closest_he->target() == curr->target() &&
                      closest_he->direction() == curr->direction())))
           {
-            if (closest_he->direction() == ARR_LEFT_TO_RIGHT)
-            {
+            if (closest_he->direction() == ARR_LEFT_TO_RIGHT) {
               // Both curves extend to the left from a common point.
-              y_res = compare_y_at_x_left (closest_he->curve(),
-                                           curr->curve(), 
+              y_res = compare_y_at_x_left(closest_he->curve(), curr->curve(), 
+                                          closest_he->target()->point());
+            }
+            else {
+              // Both curves extend to the right from a common point.
+              y_res = compare_y_at_x_right(closest_he->curve(), curr->curve(), 
                                            closest_he->target()->point());
             }
-            else
-            {
-              // Both curves extend to the right from a common point.
-              y_res = compare_y_at_x_right (closest_he->curve(),
-                                            curr->curve(), 
-                                            closest_he->target()->point());
-            }
           }
-          else
-          {
+          else {
             // In case the two curves do not have a common endpoint, but
             // overlap in their x-range (both contain p), just compare their
             // positions. Note that in this case one of the edges may be
@@ -744,14 +688,12 @@ _is_in_connected_component (const Point_2& p,
             else if (curr->is_fictitious())
               y_res = point_above_under;
             else
-              y_res = compare_y_position (closest_he->curve(),
-                                          curr->curve());
+              y_res = compare_y_position(closest_he->curve(), curr->curve());
           }
 
           // If the current curve is closer to p than the closest curve found
           // so far, assign its halfedge as the closest one.
-          if (y_res == curve_above_under)
-          {
+          if (y_res == curve_above_under) {
             closest_he = curr;
             closest_in_ccb = true;
             closest_to_target = (target_res == EQUAL);
@@ -760,23 +702,21 @@ _is_in_connected_component (const Point_2& p,
 
         // In the degenerate case where p lies below the target vertex of
         // the current halfedge, we have to be a bit careful:
-        if (target_res == EQUAL)
-        {
+        if (target_res == EQUAL) {
           // Locate the next halfedge along the boundary that does not
           // contain a vertical segment.
           Halfedge_const_handle  next_non_vert = curr;
 
-          do
-          {
+          do {
             next_non_vert = next_non_vert->next();
 
             CGAL_assertion (next_non_vert != curr);
-
-          } while ((! next_non_vert->is_fictitious() &&
-                    is_vertical (next_non_vert->curve())) ||
-                   (next_non_vert->is_fictitious() &&
-                    next_non_vert->source()->parameter_space_in_x() != 
-                    next_non_vert->target()->parameter_space_in_x()));
+          }
+          while ((! next_non_vert->is_fictitious() &&
+                  is_vertical(next_non_vert->curve())) ||
+                 (next_non_vert->is_fictitious() &&
+                  next_non_vert->source()->parameter_space_in_x() != 
+                  next_non_vert->target()->parameter_space_in_x()));
 
           // In case the source of the current curve and the target of
           // the next non-vertical curve lie on opposite sides of the
@@ -795,36 +735,31 @@ _is_in_connected_component (const Point_2& p,
           //
           //          (a)                        (b)
           //
-          target_res = top_traits->compare_x (p,
-                                              &(*(next_non_vert->target())));
+          target_res = top_traits->compare_x(p, &(*(next_non_vert->target())));
 
           CGAL_assertion (source_res != EQUAL && target_res != EQUAL);
 
           if (source_res != target_res)
             n_ray_intersections++;
         }
-        else
-        {
+        else {
           // In this case p lies under the interior of the current x-montone
           // curve, so the vertical ray we shoot intersects it exactly once.
           n_ray_intersections++;
         }
       }
     }
-    else
-    {
+    else {
       // Assign an edge associated with a vertical curve as the closest edge
       // only if the vertical curve has an endpoint closer to p than the
       // closest edge so far.
       if (closest_he == invalid_he ||
           (! closest_in_ccb && closest_he->twin() == curr) ||
           (! curr->source()->is_at_open_boundary() &&
-           top_traits->compare_y_at_x
-               (curr->source()->point(),
+           top_traits->compare_y_at_x(curr->source()->point(),
                 &(*closest_he)) == point_above_under) ||
           (! curr->target()->is_at_open_boundary() &&
-           top_traits->compare_y_at_x
-               (curr->target()->point(),
+           top_traits->compare_y_at_x(curr->target()->point(),
                 &(*closest_he)) == point_above_under))
       {
         closest_he = curr;
@@ -871,12 +806,10 @@ _first_around_vertex (Vertex_const_handle v, bool shoot_up) const
     v->incident_halfedges();
   typename Arrangement_2::Halfedge_around_vertex_const_circulator curr = first;
 
-  do 
-  {
+  do {
     // Check whether the current halfedge is defined to the left or to the
     // right of the given vertex.
-    if (curr->direction() == ARR_LEFT_TO_RIGHT)
-    {
+    if (curr->direction() == ARR_LEFT_TO_RIGHT) {
       // The curve associated with the current halfedge is defined to the left
       // of v.
       if (lowest_left == invalid_handle ||
@@ -889,8 +822,7 @@ _first_around_vertex (Vertex_const_handle v, bool shoot_up) const
         lowest_left = curr;
       }
     }
-    else
-    {
+    else {
       // The curve associated with the current halfedge is defined to the right
       // of v.
       if (top_right == invalid_handle ||
@@ -907,19 +839,14 @@ _first_around_vertex (Vertex_const_handle v, bool shoot_up) const
     curr++;
   } while (curr != first);
 
-  if (shoot_up)
-  {
+  if (shoot_up) {
     // As we start from "6 o'clock" in a clockwsie direction, the first
     // halfedge we encounter is the lowest to the left. However, if there
     // is no edge to the left, we first encounter the topmost halfedge to the
     // right.
-    if (lowest_left != invalid_handle)
-      return (lowest_left);
-    else
-      return (top_right);
+    return (lowest_left != invalid_handle) ? lowest_left : top_right;
   }
-  else
-  {
+  else {
     // As we start from "12 o'clock" in a clockwsie direction, the first
     // halfedge we encounter is the topmost to the right. However, if there
     // is no edge to the right, we first encounter the lowest halfedge to the
