@@ -12,17 +12,23 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Arrangement_on_surface_2/include/CGAL/Arr_rat_arc/Singleton.h $
+// $Id: Singleton.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Oren Salzman <orenzalz@post.tau.ac.il >
 //                 Michael Hemmer <Michael.Hemmer@sophia.inria.fr>
 
 
 
-#ifndef SINGLETON_H_
+#ifndef CGAL_SINGLETON_H_
 #define SINGLETON_H_
 
+#include <CGAL/license/Arrangement_on_surface_2.h>
+
+
+#include <CGAL/assertions.h>
+
 namespace CGAL {
 namespace Arr_rational_arc {
 template <class T>
@@ -33,7 +39,7 @@ public:
   {
     if(!m_pInstance) 
       m_pInstance = new T;
-    assert(m_pInstance !=NULL);
+    CGAL_assertion(m_pInstance !=NULL);
     return m_pInstance;
   }
       
@@ -53,4 +59,4 @@ template <class T> T* Singleton<T>::m_pInstance=NULL;
 
 }   // namespace Arr_rational_arc
 }   //namespace CGAL { 
-#endif // SINGLETON_H_
+#endif // CGAL_SINGLETON_H_
