@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Arrangement_on_surface_2/include/CGAL/Arr_rational_function_traits_2.h $
+// $Id: Arr_rational_function_traits_2.h 18acb2e 2018-01-18T15:05:34+00:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Oren Salzman <orenzalz@post.tau.ac.il >
 //                 Michael Hemmer <Michael.Hemmer@sophia.inria.fr>
@@ -21,6 +22,11 @@
 #ifndef CGAL_ARR_RATIONAL_ARC_TRAITS_D_1_H
 #define CGAL_ARR_RATIONAL_ARC_TRAITS_D_1_H
 
+#include <CGAL/license/Arrangement_on_surface_2.h>
+
+#include <CGAL/disable_warnings.h>
+
+#include <CGAL/assertions.h>
 #include <CGAL/tags.h>
 #include <CGAL/Fraction_traits.h>
 #include <CGAL/Arr_tags.h>
@@ -1295,7 +1301,7 @@ public:
     Approximate_number_type operator()(const Point_2& p, int i){
       if(i==0) return approx_x(p); 
       if(i==1) return approx_y(p);
-      assert(false);
+      CGAL_assertion(false);
       return Approximate_number_type(0);
     }
   };
@@ -1310,5 +1316,6 @@ public:
 
 }   //namespace CGAL {
 
+#include <CGAL/enable_warnings.h>
 
 #endif  //CGAL_ARR_RATIONAL_ARC_TRAITS_D_1_H
