@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Arrangement_on_surface_2/include/CGAL/Arr_topology_traits/Arr_unb_planar_topology_traits_2_impl.h $
+// $Id: Arr_unb_planar_topology_traits_2_impl.h 7936109 2017-11-16T16:31:52+02:00 Efi Fogel
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Ron Wein  <wein@post.tau.ac.il>
@@ -22,7 +23,10 @@
 #ifndef CGAL_ARR_UNB_PLANAR_TOPOLOGY_TRAITS_2_IMPL_H
 #define CGAL_ARR_UNB_PLANAR_TOPOLOGY_TRAITS_2_IMPL_H
 
+#include <CGAL/license/Arrangement_on_surface_2.h>
+
 /*! \file
+ *
  * Member-function definitions for the
  * Arr_unb_planar_topology_traits_2<GeomTraits> class.
  */
@@ -32,22 +36,22 @@ namespace CGAL {
 //-----------------------------------------------------------------------------
 // Default constructor.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 Arr_unb_planar_topology_traits_2():
-  Base (),
-  v_bl (NULL),
-  v_tl (NULL),
-  v_br (NULL),
-  v_tr (NULL),
-  n_inf_verts (0),
-  fict_face (NULL)
+  Base(),
+  v_bl(NULL),
+  v_tl(NULL),
+  v_br(NULL),
+  v_tr(NULL),
+  n_inf_verts(0),
+  fict_face(NULL)
 {}
 
 //-----------------------------------------------------------------------------
 // Constructor with a geometry-traits class.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 Arr_unb_planar_topology_traits_2 (const Geometry_traits_2 * geom_traits) :
   Base (geom_traits),
@@ -62,7 +66,7 @@ Arr_unb_planar_topology_traits_2 (const Geometry_traits_2 * geom_traits) :
 //-----------------------------------------------------------------------------
 // Assign the contents of another topology-traits class.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 void Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 assign(const Self& other)
 {
@@ -78,7 +82,7 @@ assign(const Self& other)
 //-----------------------------------------------------------------------------
 // Make the necessary updates after the DCEL structure have been updated.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 void Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::dcel_updated ()
 {
   // Go over the DCEL vertices and locate the four fictitious ones.
@@ -143,7 +147,7 @@ void Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::dcel_updated ()
 //-----------------------------------------------------------------------------
 // Initialize an empty DCEL structure.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 void Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::init_dcel ()
 {
   // Clear the current DCEL.
@@ -248,7 +252,7 @@ void Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::init_dcel ()
 //-----------------------------------------------------------------------------
 // Check if the given vertex is associated with the given curve end.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 bool Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 are_equal(const Vertex *v,
           const X_monotone_curve_2& cv, Arr_curve_end ind,
@@ -300,7 +304,7 @@ are_equal(const Vertex *v,
 // interior of the curve, find a place for a boundary vertex that will
 // represent the curve end along the face boundary.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 CGAL::Object
 Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 place_boundary_vertex(Face *f,
@@ -338,7 +342,7 @@ place_boundary_vertex(Face *f,
 //-----------------------------------------------------------------------------
 // Locate a DCEL feature that contains the given unbounded curve end.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 CGAL::Object Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 locate_curve_end (const X_monotone_curve_2& cv, Arr_curve_end ind,
                   Arr_parameter_space ps_x, Arr_parameter_space ps_y)
@@ -399,7 +403,7 @@ locate_curve_end (const X_monotone_curve_2& cv, Arr_curve_end ind,
 //-----------------------------------------------------------------------------
 // Split a fictitious edge using the given vertex.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 typename Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::Halfedge*
 Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 split_fictitious_edge (Halfedge *e, Vertex *v)
@@ -476,7 +480,7 @@ split_fictitious_edge (Halfedge *e, Vertex *v)
 //-----------------------------------------------------------------------------
 // Determine whether the given face is unbounded.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 bool Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 is_unbounded(const Face *f) const
 {
@@ -502,7 +506,7 @@ is_unbounded(const Face *f) const
 //-----------------------------------------------------------------------------
 // Determine whether the given boundary vertex is redundant.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 bool Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 is_redundant(const Vertex *v) const
 {
@@ -525,7 +529,7 @@ is_redundant(const Vertex *v) const
 //-----------------------------------------------------------------------------
 // Erase the given redundant vertex.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 typename Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::Halfedge*
 Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 erase_redundant_vertex (Vertex *v)
@@ -597,7 +601,7 @@ erase_redundant_vertex (Vertex *v)
 // Compare the x-coordinates of a given vertex (which may lie at infinity) and
 // the given point.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 Comparison_result
 Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 compare_x (const Point_2& p, const Vertex* v) const
@@ -634,7 +638,7 @@ compare_x (const Point_2& p, const Vertex* v) const
 //-----------------------------------------------------------------------------
 // Compare the given vertex (which may lie at infinity) and the given point.
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 Comparison_result
 Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
 compare_xy (const Point_2& p, const Vertex* v) const
@@ -675,10 +679,10 @@ compare_xy (const Point_2& p, const Vertex* v) const
 // Compare the relative y-position of the given point and the given edge
 // (which may be fictitious).
 //
-template <class GeomTraits, class Dcel_>
+template <typename GeomTraits, typename Dcel_>
 Comparison_result
 Arr_unb_planar_topology_traits_2<GeomTraits, Dcel_>::
-compare_y_at_x (const Point_2& p, const Halfedge* he) const
+compare_y_at_x(const Point_2& p, const Halfedge* he) const
 {
   // In case of a valid edge, just compare p to its associated curve.
   if (! he->has_null_curve())
