@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Curve_with_halfedge.h $
+// $Id: Curve_with_halfedge.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Baruch Zukerman <baruchzu@post.tau.ac.il>
@@ -21,6 +22,9 @@
 #ifndef CGAL_CURVE_WITH_HALFEDGE_H
 #define CGAL_CURVE_WITH_HALFEDGE_H
 
+#include <CGAL/license/Boolean_set_operations_2.h>
+
+
 namespace CGAL {
 
 template <class Arrangement_>
