@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Boolean_set_operations_2/include/CGAL/Boolean_set_operations_2/Gps_on_surface_base_2_impl.h $
+// $Id: Gps_on_surface_base_2_impl.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Baruch Zukerman <baruchzu@post.tau.ac.il>
@@ -24,6 +25,9 @@
 #ifndef CGAL_GPS_ON_SURFACE_BASE_2_IMPL_H
 #define CGAL_GPS_ON_SURFACE_BASE_2_IMPL_H
 
+#include <CGAL/license/Boolean_set_operations_2.h>
+
+
 #include <CGAL/iterator.h>
 #include <CGAL/function_objects.h>
 #include <CGAL/circulator.h>
@@ -33,10 +37,12 @@
 #include <queue>
 #include <list>
 
+namespace CGAL {
+
 template <class Traits_, class TopTraits_, class ValidationPolicy>
 void Gps_on_surface_base_2<Traits_, TopTraits_,ValidationPolicy>::
 construct_polygon(Ccb_halfedge_const_circulator ccb, Polygon_2 & pgn,
-                  Traits_ * tr)
+                  const Traits_* tr)
 {
   typedef CGAL::Ccb_curve_iterator<Arrangement_on_surface_2>
     Ccb_curve_iterator;
@@ -77,19 +83,18 @@ public:
 
 
 protected:
-
-  Gps_traits*                            m_traits;
-  std::queue<Face_const_iterator>        m_holes_q;
-  std::list<Polygon_2>                   m_pgn_holes;
-  OutputIterator                         m_oi;
+  const Gps_traits* m_traits;
+  std::queue<Face_const_iterator> m_holes_q;
+  std::list<Polygon_2> m_pgn_holes;
+  OutputIterator m_oi;
 
 public:
-
   /*! Constructor */
-  Arr_bfs_scanner(Gps_traits* tr, OutputIterator oi) : m_traits(tr), m_oi(oi)
+  Arr_bfs_scanner(const Gps_traits* tr, OutputIterator oi) :
+    m_traits(tr),
+    m_oi(oi)
   {}
 
-
   void scan(Arrangement& arr)
   {
     Face_const_iterator   ubf;
@@ -443,9 +448,9 @@ template <class Traits_, class TopTraits_, class ValidationPolicy>
 {
   typename std::iterator_traits<PolygonIter>::value_type pgn;
   //check validity of all polygons
-  for( ; p_begin != p_end; ++p_begin)
+  for(PolygonIter pitr = p_begin; pitr != p_end; ++pitr)
   {
-    ValidationPolicy::is_valid(*p_begin, *m_traits);
+    ValidationPolicy::is_valid(*pitr, *m_traits);
   }
 
   _insert(p_begin, p_end, pgn);
@@ -583,8 +588,7 @@ template <class Traits_, class TopTraits_, class ValidationPolicy>
   Gps_on_surface_base_2<Traits_, TopTraits_, ValidationPolicy>::
   _construct_curves(const Polygon_2 & pgn, OutputIterator oi)
 {
-  std::pair<Curve_const_iterator,
-    Curve_const_iterator> itr_pair =
+  std::pair<Curve_const_iterator, Curve_const_iterator> itr_pair =
     m_traits->construct_curves_2_object()(pgn);
   std::copy (itr_pair.first, itr_pair.second, oi);
 }
@@ -597,9 +601,9 @@ template <class Traits_, class TopTraits_, class ValidationPolicy>
   //if (!pgn.is_unbounded())
   if (!m_traits->construct_is_unbounded_object()(pgn))
   {
-    const Polygon_2& pgn_boundary = m_traits->construct_outer_boundary_object ()(pgn);
-    std::pair<Curve_const_iterator,
-      Curve_const_iterator> itr_pair =
+    const Polygon_2& pgn_boundary =
+      m_traits->construct_outer_boundary_object()(pgn);
+    std::pair<Curve_const_iterator, Curve_const_iterator> itr_pair =
       m_traits->construct_curves_2_object()(pgn_boundary);
     std::copy (itr_pair.first, itr_pair.second, oi);
   }
@@ -609,8 +613,7 @@ template <class Traits_, class TopTraits_, class ValidationPolicy>
   for (hit = hpair.first; hit != hpair.second; ++hit)
   {
     const Polygon_2& pgn_hole = *hit;
-    std::pair<Curve_const_iterator,
-      Curve_const_iterator> itr_pair =
+    std::pair<Curve_const_iterator, Curve_const_iterator> itr_pair =
       m_traits->construct_curves_2_object()(pgn_hole);
     std::copy (itr_pair.first, itr_pair.second, oi);
   }
@@ -790,4 +793,6 @@ template <class Traits_, class TopTraits_, class ValidationPolicy>
   return false;
 }
 
+} // namespace CGAL
+
 #endif // CGAL_GPS_UTILS_H
