@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Circular_kernel_2/include/CGAL/Circular_kernel_2/internal_functions_on_line_arc_2.h $
+// $Id: internal_functions_on_line_arc_2.h 0d3ac75 2018-02-17T12:37:15+00:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Monique Teillaud, Sylvain Pion, Pedro Machado
 
@@ -26,6 +27,9 @@
 #ifndef CGAL_CIRCULAR_KERNEL_PREDICATES_ON_LINE_ARC_2_H
 #define CGAL_CIRCULAR_KERNEL_PREDICATES_ON_LINE_ARC_2_H
 
+#include <CGAL/license/Circular_kernel_2.h>
+
+
 #include <CGAL/Circular_kernel_2/internal_functions_on_line_2.h>
 #include <CGAL/Circular_kernel_2/internal_functions_on_circular_arc_2.h>
 #include <CGAL/Circular_kernel_2/Intersection_traits.h>
@@ -484,10 +488,10 @@ namespace CircularFunctors {
     }
 
     typename Intersection_traits<CK, typename CK::Line_2, typename CK::Line_2>::result_type
-      v = CGAL::internal::intersection(a1.supporting_line(), a2.supporting_line(), CK());
+      v = CGAL::Intersections::internal::intersection(a1.supporting_line(), a2.supporting_line(), CK());
     if(!v) return res;
 
-    const Point_2 *pt = CGAL::internal::intersect_get<Point_2>(v);
+    const Point_2 *pt = CGAL::Intersections::internal::intersect_get<Point_2>(v);
     if(pt == NULL) return res;
     Circular_arc_point_2 intersect_point = Circular_arc_point_2(*pt);
     //      (Root_for_circles_2_2(Root_of_2(pt->x()),Root_of_2(pt->y())));
@@ -508,6 +512,13 @@ namespace CircularFunctors {
     bool operator()(const std::pair<typename CK::Circular_arc_point_2, unsigned>& pair) const {
       return has_on<CK>(*l,pair.first,true);
     }
+
+#ifdef CGAL_ADDITIONAL_VARIANT_FOR_ICL
+    bool operator()(const int) const
+    {
+      return true;
+    }
+#endif
   };
 
   template< class CK, class OutputIterator>
@@ -557,9 +568,6 @@ namespace CircularFunctors {
 	       const typename CK::Circular_arc_2 &c,
 	       OutputIterator res )
   {
-    typedef std::vector<CGAL::Object > solutions_container;
-    typedef typename CK::Circular_arc_point_2 Circular_arc_point_2;
-
 #if defined(CGAL_CK_EXPLOIT_IDENTITY) || \
   defined(CGAL_INTERSECTION_MAP_FOR_SUPPORTING_CIRCLES)
     typedef typename CK::Line_arc_2 Line_arc_2;
@@ -667,7 +675,8 @@ namespace CircularFunctors {
     }
 #endif // CGAL_CK_EXPLOIT_IDENTITY
 
-    typedef std::vector<CGAL::Object > solutions_container;
+    typedef std::vector<typename CK2_Intersection_traits<CK, typename CK::Line_2, typename CK::Circle_2>::type>
+      solutions_container;
     solutions_container solutions;
 
 #ifdef CGAL_INTERSECTION_MAP_FOR_SUPPORTING_CIRCLES
@@ -687,25 +696,30 @@ namespace CircularFunctors {
       }
 #endif
 
+
     for (typename solutions_container::iterator it = solutions.begin();
-	 it != solutions.end(); ++it) {
-      const std::pair<Circular_arc_point_2, unsigned>
-        *result = CGAL::object_cast
-	  <std::pair<Circular_arc_point_2, unsigned> > (&(*it));
-#ifdef CGAL_CK_TEST_BBOX_BEFORE_HAS_ON
-	  Bbox_2 rb = result->first.bbox();
-	  if(do_overlap(l.bbox(), rb) && do_overlap(c.bbox(),rb)){
-	    if (has_on<CK>(l,result->first,true) &&
-		has_on<CK>(c,result->first,true)) {
-	      *res++ = *it;
-	    }
-	  }
-#else
-      if (has_on<CK>(l,result->first,true) &&
-          has_on<CK>(c,result->first,true)) {
+	 it != solutions.end(); ++it)
+    {
+      #if CGAL_INTERSECTION_VERSION  < 2
+      if(const std::pair<typename CK::Circular_arc_point_2, unsigned>* p =
+         object_cast< std::pair< typename CK::Circular_arc_point_2, unsigned> >(& (*it)))
+      {
+        #ifdef CGAL_CK_TEST_BBOX_BEFORE_HAS_ON
+        Bbox_2 rb = p->first.bbox();
+        if(!do_overlap(l.bbox(), rb) || !do_overlap(c.bbox(),rb)) continue;
+        #endif
+        Has_on_visitor<CK, typename CK::Line_arc_2> vis1(&l);
+        Has_on_visitor<CK, typename CK::Circular_arc_2> vis2(&c);
+        if(vis1(*p) && vis2(*p))
+          *res++ = *it;
+      }
+      #else
+      if(boost::apply_visitor(Has_on_visitor<CK, typename CK::Line_arc_2>(&l), *it) &&
+         boost::apply_visitor(Has_on_visitor<CK, typename CK::Circular_arc_2>(&c), *it) )
+      {
 	*res++ = *it;
       }
-#endif
+      #endif
     }
     return res;
   }
