@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Surface_mesher/include/CGAL/Complex_2_in_triangulation_cell_base_3.h $
+// $Id: Complex_2_in_triangulation_cell_base_3.h a0427ec 2018-04-26T13:06:21+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Steve Oudot, David Rey, Mariette Yvinec, Laurent Rineau, Andreas Fabri
@@ -23,8 +24,9 @@
 #ifndef CGAL_COMPLEX_2_IN_TRIANGULATION_CELL_BASE_3_H
 #define CGAL_COMPLEX_2_IN_TRIANGULATION_CELL_BASE_3_H
 
+#include <CGAL/license/Surface_mesher.h>
 
-#include <CGAL/Triangulation_cell_base_3.h>
+#include <CGAL/Delaunay_triangulation_cell_base_3.h>
 
 #include <bitset>
 #include <string>
@@ -32,7 +34,7 @@
 namespace CGAL {
 
 
-  template < class GT, class Cb = Triangulation_cell_base_3 <GT> >
+  template < class GT, class Cb = Delaunay_triangulation_cell_base_3 <GT> >
   class Complex_2_in_triangulation_cell_base_3 : public Cb {
 
   public:
