@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Triangulation_2/include/CGAL/Constrained_Delaunay_triangulation_2.h $
+// $Id: Constrained_Delaunay_triangulation_2.h 630066d 2019-01-10T17:31:54+01:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Mariette Yvinec, Jean Daniel Boissonnat
@@ -21,6 +22,9 @@
 #ifndef CGAL_CONSTRAINED_DELAUNAY_TRIANGULATION_2_H
 #define CGAL_CONSTRAINED_DELAUNAY_TRIANGULATION_2_H
 
+#include <CGAL/license/Triangulation_2.h>
+
+
 #include <CGAL/triangulation_assertions.h>
 #include <CGAL/Constrained_triangulation_2.h>
 #include <CGAL/Triangulation_2/insert_constraints.h>
@@ -55,17 +59,18 @@ struct Get_iterator_value_type<T,true>{
 namespace CGAL {
 
 
-template <class Gt, 
-          class Tds = Triangulation_data_structure_2 <
-                      Triangulation_vertex_base_2<Gt>,
-		      Constrained_triangulation_face_base_2<Gt> >,
-	  class Itag = No_intersection_tag >		
+template <class Gt,
+          class Tds_ = Default ,
+	  class Itag_ = Default>
 class Constrained_Delaunay_triangulation_2
-  : public  Constrained_triangulation_2<Gt, Tds, Itag> 
+  : public  Constrained_triangulation_2<Gt, Tds_, Itag_>
 {
 public:
-  typedef Constrained_triangulation_2<Gt,Tds,Itag>             Ctr;
-  typedef Constrained_Delaunay_triangulation_2<Gt,Tds,Itag>    CDt;
+  typedef Constrained_triangulation_2<Gt,Tds_,Itag_>            Ctr;
+  typedef typename Ctr::Tds Tds;
+  typedef typename Ctr::Itag Itag;
+
+  typedef Constrained_Delaunay_triangulation_2<Gt,Tds_,Itag_>    CDt;
   typedef typename Ctr::Geom_traits      Geom_traits;
   typedef typename Ctr::Intersection_tag Intersection_tag;
 
@@ -74,6 +79,7 @@ public:
   typedef typename Ctr::Face_handle   Face_handle;
   typedef typename Ctr::Edge          Edge;
   typedef typename Ctr::Finite_faces_iterator Finite_faces_iterator;
+  typedef typename Ctr::Constrained_edges_iterator Constrained_edges_iterator;
   typedef typename Ctr::Face_circulator       Face_circulator;
   typedef typename Ctr::size_type             size_type;
   typedef typename Ctr::Locate_type           Locate_type;
@@ -85,6 +91,12 @@ public:
   typedef typename Ctr::Less_edge less_edge;
   typedef typename Ctr::Edge_set Edge_set;
 
+  //Tag to distinguish Delaunay from regular triangulations
+  typedef Tag_false Weighted_tag;
+
+  // Tag to distinguish periodic triangulations from others
+  typedef Tag_false Periodic_tag;
+
 #ifndef CGAL_CFG_USING_BASE_MEMBER_BUG_2
   using Ctr::geom_traits;
   using Ctr::number_of_vertices;
@@ -113,21 +125,14 @@ public:
 
   typedef typename Geom_traits::Point_2  Point;
 
-
   Constrained_Delaunay_triangulation_2(const Geom_traits& gt=Geom_traits()) 
     : Ctr(gt) { }
 
-  Constrained_Delaunay_triangulation_2(const CDt& cdt)
-    : Ctr(cdt) {}
-
-  Constrained_Delaunay_triangulation_2(List_constraints& lc, 
+  Constrained_Delaunay_triangulation_2(const List_constraints& lc,
 				       const Geom_traits& gt=Geom_traits())
     : Ctr(gt) 
-    {   
-      typename List_constraints::iterator itc = lc.begin();
-      for( ; itc != lc.end(); ++itc) {
-	insert((*itc).first, (*itc).second);
-      }
+    {
+      insert_constraints(lc.begin(), lc.end());
       CGAL_triangulation_postcondition( is_valid() );
     }
 
@@ -137,9 +142,7 @@ public:
 				       const Geom_traits& gt=Geom_traits() )
     : Ctr(gt) 
     {
-      for ( ; it != last; it++) {
-      	insert((*it).first, (*it).second);
-      }
+      insert_constraints(it, last);
       CGAL_triangulation_postcondition( is_valid() );
     }
 
@@ -181,7 +184,6 @@ public:
   // get_boundary_of_conflicts(const Point  &p, 
   // 			       OutputItBoundaryEdges eit, 
   // 			       Face_handle start ) const;
-   
 
   // INSERTION-REMOVAL
   Vertex_handle insert(const Point & a, Face_handle start = Face_handle());
@@ -311,10 +313,10 @@ private:
   std::ptrdiff_t insert_with_info(InputIterator first,InputIterator last)
   {
     size_type n = this->number_of_vertices();
-    std::vector<std::ptrdiff_t> indices;
+    std::vector<std::size_t> indices;
     std::vector<Point> points;
     std::vector<typename Tds::Vertex::Info> infos;
-    std::ptrdiff_t index=0;
+    std::size_t index=0;
     for (InputIterator it=first;it!=last;++it){
       Tuple_or_pair value=*it;
       points.push_back( top_get_first(value)  );
@@ -322,13 +324,16 @@ private:
       indices.push_back(index++);
     }
 
-    typedef Spatial_sort_traits_adapter_2<Geom_traits,Point*> Search_traits;
+    typedef typename Pointer_property_map<Point>::type Pmap;
+    typedef Spatial_sort_traits_adapter_2<Geom_traits,Pmap> Search_traits;
 
-    spatial_sort(indices.begin(),indices.end(),Search_traits(&(points[0]),geom_traits()));
+    spatial_sort(indices.begin(),
+                 indices.end(),
+                 Search_traits(make_property_map(points),geom_traits()));
 
     Vertex_handle v_hint;
     Face_handle hint;
-    for (typename std::vector<std::ptrdiff_t>::const_iterator
+    for (typename std::vector<std::size_t>::const_iterator
       it = indices.begin(), end = indices.end();
       it != end; ++it){
       v_hint = insert(points[*it], hint);
@@ -742,9 +747,9 @@ propagating_flip(Face_handle f,int i, int depth)
 
   Face_handle ni = f->neighbor(i);
   flip(f, i); // flip for constrained triangulations
-  propagating_flip(f,i);
+  propagating_flip(f,i, depth+1);
   i = ni->index(f->vertex(i));
-  propagating_flip(ni,i);
+  propagating_flip(ni,i, depth+1);
 #endif
 }
 #else 
