@@ -1,10 +1,10 @@
-// Copyright (c) 2007-2008 Max-Planck-Institute Saarbruecken (Germany), 
+// Copyright (c) 2007,2008,2009,2010,2011 Max-Planck-Institute Saarbruecken (Germany), 
 // and Tel-Aviv University (Israel).  All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org); you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; version 2.1 of the License.
-// See the file LICENSE.LGPL distributed with CGAL.
+// published by the Free Software Foundation; either version 3 of the License,
+// or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Sweep_curves_adapter_2.h $
-// $Id: Sweep_curves_adapter_2.h 59002 2010-10-04 11:00:27Z lrineau $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/Sweep_curves_adapter_2.h $
+// $Id: Sweep_curves_adapter_2.h 6283291 2018-06-13T14:43:49+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     : Pavel Emeliyanenko <asm@mpi-sb.mpg.de>
@@ -32,7 +33,7 @@
 
 #include <boost/optional.hpp>
 #include <boost/none.hpp>
-
+#include <CGAL/iterator.h>
 #include <CGAL/Handle_with_policy.h>
 
 #include <CGAL/Arr_enums.h>
@@ -117,12 +118,12 @@ public:
                         return (loc1 == CGAL::ARR_LEFT_BOUNDARY ?
                                  CGAL::SMALLER : CGAL::LARGER);
                     return (_m_adapter->kernel().
-                       compare_y_near_boundary_2_object()(arc, p2.arc(), end));
+                       compare_y_curve_ends_2_object()(arc, p2.arc(), end));
                 }
                 // compare curve ends at +/-oo in y
                 if(Native_arc_2::is_on_bottom_top(loc2)) {
                     CGAL::Comparison_result res = (_m_adapter->kernel().
-                        compare_x_near_boundary_2_object()
+                        compare_x_curve_ends_2_object()
                             (arc, end, p2.arc(), p2.curve_end()));
                     if(res == CGAL::EQUAL && end == p2.curve_end() &&
                             loc1 != loc2) {
@@ -143,8 +144,6 @@ public:
                  CGAL::SMALLER);
         else {
             // point is p1, arc is p2
-            //res = _m_adapter->kernel().
-              //  compare_x_near_boundary_2_object()(pt, arc, end);
             // compares a finite point with a curve end at y=+/-oo:
             res = _m_adapter->kernel().kernel().compare_1_object()
                 (pt.x(), arc.curve_end_x(end));
@@ -254,7 +253,7 @@ public:
         if(Native_arc_2::is_on_left_right(locp)) {
             CGAL_precondition(locp == cv.arc().location(end));
             // compare two curve ends at +/-oo in x
-            return _m_adapter->kernel().compare_y_near_boundary_2_object()
+            return _m_adapter->kernel().compare_y_curve_ends_2_object()
                 (p.arc(), cv.arc(), end);
         }
         // p.arc() has vertical asymptote; cases:
@@ -295,7 +294,7 @@ public:
         // compare either two asymptotic ends or one vertical arc + asymptote
         // check whether result need to be reversed
         CGAL::Comparison_result res =
-             (_m_adapter->kernel().compare_x_near_boundary_2_object()
+             (_m_adapter->kernel().compare_x_curve_ends_2_object()
                   (p.arc(), end, cv.arc(), end2));
 
         if(locp == cv.arc().location(end2) &&
@@ -595,8 +594,9 @@ public:
      * lexicographic order of endpoints is ensured automatically, hence no
      * special handling is required
      */
-    result_type operator()(const Arc_2& cv, const Point_2& p,
-                                          const Point_2& q) const {
+    result_type operator()(const Arc_2& cv, 
+                           const Point_2& /* p */,
+                           const Point_2& /* q */) const {
         SCA_CERR("\n\nWARNING!! New_endpoints_opposite_2: cv: " << cv <<
             "\n p: " << p << "\n q: " << q << std::endl);
         CGAL_error_msg("New_endpoints_opposite_2 deprecated and must not be \
@@ -693,7 +693,7 @@ class Make_x_monotone_2
     typedef typename SweepCurvesAdapter_2::Generic_arc_2 Generic_arc_2;
    
 public:
-    typedef std::iterator<std::output_iterator_tag, Generic_arc_2> 
+    typedef CGAL::cpp98::iterator<std::output_iterator_tag, Generic_arc_2>
       result_type;
     
     //! standard constructor
