@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org); you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; version 2.1 of the License.
-// See the file LICENSE.LGPL distributed with CGAL.
+// published by the Free Software Foundation; either version 3 of the License,
+// or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h $
-// $Id: Curve_renderer_traits.h 59002 2010-10-04 11:00:27Z lrineau $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Arrangement_on_surface_2/include/CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h $
+// $Id: Curve_renderer_traits.h ff26773 2017-11-11T20:20:26+01:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 //
 // Author(s)     : Pavel Emeliyanenko <asm@mpi-sb.mpg.de>
 //
@@ -24,6 +25,7 @@
 
 #include <CGAL/basic.h>
 #include <CGAL/function_objects.h>
+#include <boost/functional.hpp>
 
 /*! \file CGAL/Curved_kernel_via_analysis_2/gfx/Curve_renderer_traits.h
  * \brief
@@ -117,8 +119,8 @@ struct Transform {
 
         Transform<typename OutputPoly_2::NT, typename InputPoly_2::NT, Op> tr;
         return OutputPoly_2(
-            ::boost::make_transform_iterator(p.begin(), std::bind2nd(tr, op)),
-            ::boost::make_transform_iterator(p.end(), std::bind2nd(tr, op)));
+            ::boost::make_transform_iterator(p.begin(), boost::bind2nd(tr, op)),
+            ::boost::make_transform_iterator(p.end(), boost::bind2nd(tr, op)));
     }
 
     OutputPoly_2 operator()(
@@ -153,9 +155,9 @@ struct Curve_renderer_traits_base
     struct Rat_to_float {
         typedef Float result_type;
 
-        template <class X, class Y>
-        Float operator()(const Sqrt_extension<X, Y>& x) const { 
-            typename CGAL::Coercion_traits<Sqrt_extension<X, Y>, Float>::Cast
+        template <class X, class Y,class ACDE_TAG,class FP_TAG>
+        Float operator()(const Sqrt_extension<X, Y, ACDE_TAG, FP_TAG>& x) const { 
+            typename CGAL::Coercion_traits<Sqrt_extension<X, Y, ACDE_TAG, FP_TAG>, Float>::Cast
                 cast;        
             return cast(x); 
         }
@@ -502,9 +504,9 @@ struct Curve_renderer_traits<leda::bigfloat, class leda::rational>
     struct Rat_to_float {
         typedef Float result_type;
 
-        template <class X, class Y>
-        Float operator()(const Sqrt_extension<X, Y>& x) const { 
-            typename CGAL::Coercion_traits<Sqrt_extension<X, Y>, Float>::Cast
+        template <class X, class Y,class ACDE_TAG,class FP_TAG>
+        Float operator()(const Sqrt_extension<X, Y, ACDE_TAG, FP_TAG>& x) const { 
+            typename CGAL::Coercion_traits<Sqrt_extension<X, Y, ACDE_TAG, FP_TAG>, Float>::Cast
                 cast;        
             return cast(x); 
         }
