@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Envelope_2/include/CGAL/Env_default_diagram_1.h $
+// $Id: Env_default_diagram_1.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Ron Wein   <wein@post.tau.ac.il>
 
@@ -22,6 +23,12 @@
 #ifndef CGAL_ENV_DEFAULT_DIAGRAM_1_H
 #define CGAL_ENV_DEFAULT_DIAGRAM_1_H
 
+#include <CGAL/license/Envelope_2.h>
+
+#define CGAL_DEPRECATED_HEADER "<CGAL/Env_default_diagram_1.h>"
+#define CGAL_REPLACEMENT_HEADER "<CGAL/Envelope_diagram_1>"
+#include <CGAL/internal/deprecation_warning.h>
+
 #if (defined __GNUC__)
   #if !(defined __STRICT_ANSI__)
   #warning "Env_default_diagram.h is DEPRECATED, please include Envelope_diagram_1.h instead."
