@@ -1,9 +1,9 @@
-// Copyright (c) 1998-2008  
+// Copyright (c) 1998-2008
 // Utrecht University (The Netherlands),
 // ETH Zurich (Switzerland),
 // INRIA Sophia-Antipolis (France),
 // Max-Planck-Institute Saarbruecken (Germany),
-// and Tel-Aviv University (Israel).  All rights reserved. 
+// and Tel-Aviv University (Israel).  All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org); you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public License as
@@ -16,8 +16,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Number_types/include/CGAL/FPU.h $
+// $Id: FPU.h 42cbce9 2019-03-28T08:55:49+01:00 Laurent Rineau
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     : Sylvain Pion
@@ -26,6 +27,7 @@
 #define CGAL_FPU_H
 
 #include <CGAL/assertions.h>
+#include <CGAL/use.h>
 
 #ifndef __INTEL_COMPILER
 #include <cmath> // for HUGE_VAL
@@ -66,7 +68,7 @@ extern "C" {
 #  if defined CGAL_CFG_DENORMALS_COMPILE_BUG
      // For compilers crashing when dealing with denormalized values.
      // So we have to generate it at run time instead.
-#    define CGAL_IA_MIN_DOUBLE (CGAL::internal::minimin)
+#    define CGAL_IA_MIN_DOUBLE (CGAL::internal::get_static_minimin())
 #  else
 #    define CGAL_IA_MIN_DOUBLE (5e-324)
 #  endif
@@ -115,21 +117,18 @@ extern "C" {
 #  define CGAL_USE_SSE2 1
 #endif
 
-namespace CGAL {
-
-namespace internal {
-
 #ifdef CGAL_CFG_DENORMALS_COMPILE_BUG
-CGAL_EXPORT extern double minimin;
+double& get_static_minimin(); // Defined in Interval_arithmetic_impl.h
 #endif
 
-#ifdef __INTEL_COMPILER
-const double infinity = std::numeric_limits<double>::infinity();
-#else
-const double infinity = HUGE_VAL;
-#endif
+namespace  CGAL {
 
-} // namespace internal
+#ifdef CGAL_HEADER_ONLY
+// Defined in test_FPU_rounding_mode_impl.h
+struct Check_FPU_rounding_mode_is_restored;
+inline const Check_FPU_rounding_mode_is_restored&
+get_static_check_fpu_rounding_mode_is_restored();
+#endif
 
 // Inline function to stop compiler optimizations that shouldn't happen with
 // pragma fenv on.
@@ -384,6 +383,22 @@ typedef unsigned int FPU_CW_t;
 #define CGAL_FE_TOWARDZERO   _RC_CHOP
 #define CGAL_FE_UPWARD       _RC_UP
 #define CGAL_FE_DOWNWARD     _RC_DOWN
+# elif defined __VFP_FP__ && !defined __SOFTFP__
+#define CGAL_IA_SETFPCW(CW) asm volatile ("VMSR FPSCR, %0" : :"r" (CW))
+#define CGAL_IA_GETFPCW(CW) asm volatile ("VMRS %0, FPSCR" : "=r" (CW)); CW &= CGAL_FE_TOWARDZERO
+typedef unsigned int FPU_CW_t;
+#define CGAL_FE_TONEAREST    (0x0)
+#define CGAL_FE_TOWARDZERO   (0xC00000)
+#define CGAL_FE_UPWARD       (0x400000)
+#define CGAL_FE_DOWNWARD     (0x800000)
+# elif defined  __aarch64__
+#define CGAL_IA_SETFPCW(CW) asm volatile ("MSR FPCR, %0" : :"r" (CW))
+#define CGAL_IA_GETFPCW(CW) asm volatile ("MRS %0, FPCR" : "=r" (CW))
+typedef unsigned int FPU_CW_t;
+#define CGAL_FE_TONEAREST    (0x0)
+#define CGAL_FE_TOWARDZERO   (0xC00000)
+#define CGAL_FE_UPWARD       (0x400000)
+#define CGAL_FE_DOWNWARD     (0x800000)
 
 #else
 // This is a version following the ISO C99 standard, which aims at portability.
@@ -410,11 +425,13 @@ FPU_get_cw (void)
     return cw;
 }
 
+// User interface (cont):
+
 inline
 void
 FPU_set_cw (FPU_CW_t cw)
 {
-    CGAL_IA_SETFPCW(cw);
+  CGAL_IA_SETFPCW(cw);
 }
 
 inline
@@ -508,4 +525,8 @@ inline void force_ieee_double_precision()
 
 } //namespace CGAL
 
+#ifdef CGAL_HEADER_ONLY
+#include <CGAL/test_FPU_rounding_mode_impl.h>
+#endif // CGAL_HEADER_ONLY
+
 #endif // CGAL_FPU_H
