@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Alpha_shapes_3/include/CGAL/Fixed_alpha_shape_3.h $
+// $Id: Fixed_alpha_shape_3.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Sebastien Loriot
@@ -22,6 +23,9 @@
 #ifndef CGAL_FIXED_ALPHA_SHAPE_3_H
 #define CGAL_FIXED_ALPHA_SHAPE_3_H
 
+#include <CGAL/license/Alpha_shapes_3.h>
+
+
 #include <CGAL/basic.h>
 
 #include <set>
@@ -149,7 +153,7 @@ public:
   typedef Coord_type                                NT;
   typedef Coord_type                                FT;
 
-  typedef typename Gt::Point_3                      Point;
+  typedef typename Dt::Point                        Point;
   
   typedef typename Dt::Cell_handle                  Cell_handle;
   typedef typename Dt::Vertex_handle                Vertex_handle;
@@ -407,7 +411,7 @@ public:
     //Do remove the vertex from the underlying triangulation
     Dt tmp;
     typename Dt:: template Vertex_remover<Dt> remover(tmp);
-    typedef CGAL::Triangulation_3<typename Dt::Geom_traits,typename Dt::Triangulation_data_structure> Tr_Base;
+    typedef typename Dt::Tr_Base Tr_Base;
     Tr_Base::remove(vertex_to_remove,remover);
     
     if (this->dimension()<3){
