@@ -11,8 +11,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Number_types/include/CGAL/GMP/Gmpfi_type.h $
+// $Id: Gmpfi_type.h a154aa1 2018-02-19T14:28:35+01:00 Laurent Rineau
+// SPDX-License-Identifier: LGPL-3.0+
 //
 // Author: Luis Peñaranda <luis.penaranda@gmx.com>
 
@@ -23,12 +24,13 @@
 #include <CGAL/gmp.h>
 #include <mpfr.h>
 #include <CGAL/GMP/Gmpfr_type.h>
+#include <CGAL/GMP/Gmpq_type.h>
 #include <mpfi.h>
 #include <boost/operators.hpp>
 #include <CGAL/Uncertain.h>
-#ifdef CGAL_HAS_THREADS
-#  include <boost/thread/tss.hpp>
-#endif
+#include <CGAL/tss.h>
+#include <CGAL/IO/io.h>
+
 #include <limits>
 #include <algorithm>
 
@@ -80,11 +82,8 @@ Uncertain<bool> operator==(const Gmpfi&,const Gmpq&);
 
 // the default precision is a variable local to each thread in multithreaded
 // environments, or a global variable otherwise
-#ifdef CGAL_HAS_THREADS
-        static boost::thread_specific_ptr<mp_prec_t> Gmpfi_default_precision_;
-#else
-        static mp_prec_t Gmpfi_default_precision=CGAL_GMPFI_DEFAULT_PRECISION;
-#endif
+
+
 
 class Gmpfi:
         boost::ordered_euclidian_ring_operators1<Gmpfi,
@@ -109,6 +108,13 @@ class Gmpfi:
         Gmpfr _left,_right;
         mutable __mpfi_struct _interval;
 
+  static mp_prec_t&  default_precision()
+  {
+    CGAL_STATIC_THREAD_LOCAL_VARIABLE(mp_prec_t, Gmpfi_default_precision, CGAL_GMPFI_DEFAULT_PRECISION);
+    return Gmpfi_default_precision;
+  }
+        
+
         bool is_unique(){
 #ifdef CGAL_GMPFR_NO_REFCOUNT
                 return true;
@@ -296,9 +302,7 @@ CGAL_GMPFI_CONSTRUCTOR_FROM_SCALAR(Gmpz);
 
         // default precision
 
-#ifdef CGAL_HAS_THREADS
-        static void init_precision_for_thread();
-#endif
+
         static Gmpfi::Precision_type get_default_precision();
         static Gmpfi::Precision_type set_default_precision(
                                                 Gmpfi::Precision_type prec);
@@ -400,35 +404,20 @@ CGAL_GMPFI_CONSTRUCTOR_FROM_SCALAR(Gmpz);
 // --------------
 
 // default precision
-#ifdef CGAL_HAS_THREADS
-inline
-void Gmpfi::init_precision_for_thread(){
-        CGAL_precondition(Gmpfi_default_precision_.get()==NULL);
-        Gmpfi_default_precision_.reset(
-                new mp_prec_t(CGAL_GMPFI_DEFAULT_PRECISION));
-}
-#endif
+
 
 inline
 Gmpfi::Precision_type Gmpfi::get_default_precision(){
-#ifdef CGAL_HAS_THREADS
-        if(Gmpfi_default_precision_.get()==NULL)
-                Gmpfi::init_precision_for_thread();
-        return *Gmpfi_default_precision_.get();
-#else
-        return Gmpfi_default_precision;
-#endif
+
+  return default_precision();
 }
 
 inline
 Gmpfi::Precision_type Gmpfi::set_default_precision(Gmpfi::Precision_type prec){
-        Gmpfi::Precision_type old_prec=Gmpfi::get_default_precision();
+        Gmpfi::Precision_type old_prec= default_precision();
         CGAL_assertion(prec>=MPFR_PREC_MIN&&prec<=MPFR_PREC_MAX);
-#ifdef CGAL_HAS_THREADS
-        *Gmpfi_default_precision_.get()=prec;
-#else
-        Gmpfi_default_precision=prec;
-#endif
+        default_precision() = prec;
+
         return old_prec;
 }
 
@@ -630,8 +619,13 @@ Gmpfi Gmpfi::kthroot(int k,Gmpfi::Precision_type p)const{
         // MPFI does not provide k-th root functions
         CGAL_assertion(p>=MPFR_PREC_MIN&&p<=MPFR_PREC_MAX);
         Gmpfi result (0, p);
-        mpfr_root(&(result.mpfi())->left, left_mpfr(), k,GMP_RNDD);
-        mpfr_root(&(result.mpfi())->right,right_mpfr(),k,GMP_RNDU);
+        #if(MPFR_VERSION_MAJOR < 4)
+            mpfr_root(&(result.mpfi())->left, left_mpfr(), k,GMP_RNDD);
+            mpfr_root(&(result.mpfi())->right,right_mpfr(),k,GMP_RNDU);
+        #else
+            mpfr_rootn_ui(&(result.mpfi())->left, left_mpfr(), k,GMP_RNDD);
+            mpfr_rootn_ui(&(result.mpfi())->right,right_mpfr(),k,GMP_RNDU);
+        #endif
         *(result._left.fr()) = result._interval.left;
         *(result._right.fr()) = result._interval.right;
         return result;
@@ -832,7 +826,7 @@ std::istream& operator>>(std::istream& is,Gmpfi &f){
         std::istream::int_type c;
         std::ios::fmtflags old_flags = is.flags();
         is.unsetf(std::ios::skipws);
-        gmpz_eat_white_space(is);
+        internal::eat_white_space(is);
         c=is.get();
         if(c!='['){
                 invalid_number:
@@ -840,13 +834,13 @@ std::istream& operator>>(std::istream& is,Gmpfi &f){
                 is.flags(old_flags);
                 return is;
         }
-        gmpz_eat_white_space(is);
+        internal::eat_white_space(is);
         is>>left;
         c=is.get();
         if(c!=',')
                 goto invalid_number;
         is>>right;
-        gmpz_eat_white_space(is);
+        internal::eat_white_space(is);
         c=is.get();
         if(c!=']')
                 goto invalid_number;
