@@ -16,8 +16,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Homogeneous_kernel/include/CGAL/Homogeneous/Aff_transformationH3.h $
+// $Id: Aff_transformationH3.h 23d701c 2018-10-12T16:34:19+02:00 Maxime Gimeno
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Stefan Schirra
@@ -363,6 +364,20 @@ public:
   RT
   hm(int i, int j) const
   { return this->Ptr()->homogeneous(i,j); }
+  
+  bool operator==(const Aff_transformationH3 &t)const
+  {
+    for(int i=0; i<3; ++i)
+      for(int j = 0; j< 4; ++j)
+        if(homogeneous(i,j)!=t.homogeneous(i,j))
+          return false;
+    return true;
+  }
+  
+  bool operator!=(const Aff_transformationH3 &t)const
+  {
+    return !(*this == t);
+  }
 };
 
 template < class R >
@@ -550,29 +565,37 @@ homogeneous(int i, int j) const
               case 1: return t01;
               case 2: return t02;
               case 3: return t03;
+              default: CGAL_assume(false);
             }
+            break;
     case 1: switch (j)
             {
               case 0: return t10;
               case 1: return t11;
               case 2: return t12;
               case 3: return t13;
+              default: CGAL_assume(false);
             }
+            break;
     case 2: switch (j)
             {
               case 0: return t20;
               case 1: return t21;
               case 2: return t22;
               case 3: return t23;
+              default: CGAL_assume(false);
             }
+            break;
     case 3: switch (j)
             {
               case 0: return RT0;
               case 1: return RT0;
               case 2: return RT0;
               case 3: return t33;
+              default: CGAL_assume(false);
             }
   }
+  CGAL_assume(false);
   return RT0;
 }
 
@@ -694,29 +717,37 @@ Translation_repH3<R>::homogeneous(int i, int j) const
               case 1: return RT0;
               case 2: return RT0;
               case 3: return tv.hx();
+              default: CGAL_assume(false);
             }
+            break;
     case 1: switch (j)
             {
               case 0: return RT0;
               case 1: return tv.hw();
               case 2: return RT0;
               case 3: return tv.hy();
+              default: CGAL_assume(false);
             }
+            break;
     case 2: switch (j)
             {
               case 0: return RT0;
               case 1: return RT0;
               case 2: return tv.hw();
               case 3: return tv.hz();
+              default: CGAL_assume(false);
             }
+            break;
     case 3: switch (j)
             {
               case 0: return RT0;
               case 1: return RT0;
               case 2: return RT0;
               case 3: return tv.hw();
+              default: CGAL_assume(false);
             }
   }
+  CGAL_assume(false);
   return RT0;
 }
 
@@ -731,7 +762,6 @@ cartesian(int i, int j) const
 
 
 template < class R >
-CGAL_KERNEL_INLINE
 Aff_transformationH3<R>
 _general_transformation_composition(
     Aff_transformation_repH3<R> l,
