@@ -13,8 +13,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Mesh_3/include/CGAL/IO/File_medit.h $
+// $Id: File_medit.h b820b11 2019-03-04T17:48:43+01:00 goran-w
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Laurent RINEAU, Stephane Tayeb
@@ -22,20 +23,28 @@
 #ifndef CGAL_IO_FILE_MEDIT_H
 #define CGAL_IO_FILE_MEDIT_H
 
+#include <CGAL/license/Mesh_3.h>
+
 #include <CGAL/Mesh_3/config.h>
 
+#include <CGAL/utility.h>
+#include <CGAL/basic.h>
+
+#include <boost/type_traits/is_same.hpp>
+#include <boost/unordered_map.hpp>
+#include <boost/utility/enable_if.hpp>
+
+#include <iomanip>
 #include <iostream>
 #include <map>
 #include <set>
-#include <vector>
 #include <string>
-#include <CGAL/utility.h>
+#include <vector>
 
 namespace CGAL {
 
 namespace Mesh_3 {
 
-
 //-------------------------------------------------------
 // Needed in verbose mode
 //-------------------------------------------------------
@@ -50,7 +59,7 @@ operator<<(std::ostream &os, const std::pair<T,T>& pair)
 #endif
 
 // -----------------------------------
-// Rebin_cell_pmap
+// Rebind_cell_pmap
 // -----------------------------------
 template <typename C3T3>
 class Rebind_cell_pmap
@@ -74,15 +83,14 @@ public:
          ++cell_it)
     {
       // Add subdomain index in internal map if needed
-      if ( subdomain_map_.end() ==
-              subdomain_map_.find(r_c3t3_.subdomain_index(cell_it)) )
-      {
-        subdomain_map_.insert(std::make_pair(r_c3t3_.subdomain_index(cell_it),
-                                             index_counter));
+      std::pair<typename Subdomain_map::iterator, bool> is_insert_successful =
+          subdomain_map_.insert(std::make_pair(r_c3t3_.subdomain_index(cell_it),
+                                               index_counter));
+
+      if(is_insert_successful.second)
         ++index_counter;
-      }
     }
-    
+
     // Rebind indices in alphanumeric order
     index_counter = first_index + 1;
     for ( typename Subdomain_map::iterator mit = subdomain_map_.begin() ;
@@ -177,11 +185,8 @@ public:
         cell_it != r_c3t3_.cells_in_complex_end();
         ++cell_it)
     {
-      // Add subdomain index in set if new
-      if ( subdomain_set.end() == subdomain_set.find(subdomain_index(cell_it)) )
-      {
-        subdomain_set.insert(subdomain_index(cell_it));
-      }
+      // Add subdomain index in set
+      subdomain_set.insert(subdomain_index(cell_it));
     }
     
     return subdomain_set.size();
@@ -227,14 +232,11 @@ public:
         ++facet_it)
     {
       // Add surface index in internal map if needed
-      if ( surface_map_.end() ==
-          surface_map_.find(c3t3.surface_patch_index((*facet_it).first,
-                                                     (*facet_it).second)) )
-      {
-        surface_map_.insert(std::make_pair(r_c3t3_.surface_patch_index(*facet_it),
-                                           index_counter));
+      std::pair<typename Surface_map::iterator, bool> is_insert_successful =
+          surface_map_.insert(std::make_pair(r_c3t3_.surface_patch_index(*facet_it),
+                                             index_counter));
+      if(is_insert_successful.second)
         ++index_counter;
-      }
     }
     
     // Find cell_pmap_ unused indices
@@ -245,12 +247,8 @@ public:
         cell_it != r_c3t3_.cells_in_complex_end();
         ++cell_it)
     {
-      // Add subdomain index in set if new
-      if ( cell_label_set.end()
-          == cell_label_set.find(get(cell_pmap_,cell_it)) )
-      {
-        cell_label_set.insert(get(cell_pmap_,cell_it));
-      }
+      // Add subdomain index in set
+      cell_label_set.insert(get(cell_pmap_, cell_it));
     }
     
     // Rebind indices
@@ -575,7 +573,7 @@ public:
         {
           // If another index is found, return value for edge vertice
           if (   r_c3t3_.is_in_complex(*it_facet)
-              && facet_index != r_c3t3_.surface_patch_index(*it_facet) )
+              && !( facet_index == r_c3t3_.surface_patch_index(*it_facet) ) )
             return edge_index_;
         }
 
@@ -765,7 +763,7 @@ output_to_medit(std::ostream& os,
 
   typedef typename Tr::Finite_vertices_iterator Finite_vertices_iterator;
   typedef typename Tr::Vertex_handle Vertex_handle;
-  typedef typename Tr::Point Point_3;
+  typedef typename Tr::Weighted_point Weighted_point;
 
   const Tr& tr = c3t3.triangulation();
 
@@ -778,29 +776,28 @@ output_to_medit(std::ostream& os,
   //-------------------------------------------------------
   os << std::setprecision(17);
 
-  os << "MeshVersionFormatted 1" << std::endl
-     << "Dimension 3" << std::endl;
+  os << "MeshVersionFormatted 1\n"
+     << "Dimension 3\n";
 
 
   //-------------------------------------------------------
   // Vertices
   //-------------------------------------------------------
-  os << "Vertices" << std::endl
-     << tr.number_of_vertices() << std::endl;
+  os << "Vertices\n" << tr.number_of_vertices() << '\n';
 
-  std::map<Vertex_handle, int> V;
+  boost::unordered_map<Vertex_handle, int> V;
   int inum = 1;
   for( Finite_vertices_iterator vit = tr.finite_vertices_begin();
        vit != tr.finite_vertices_end();
        ++vit)
   {
     V[vit] = inum++;
-    Point_3 p = vit->point();
-    os << CGAL::to_double(p.x()) << " "
-       << CGAL::to_double(p.y()) << " "
-       << CGAL::to_double(p.z()) << " "
+    Weighted_point p = tr.point(vit);
+    os << CGAL::to_double(p.x()) << ' '
+       << CGAL::to_double(p.y()) << ' '
+       << CGAL::to_double(p.z()) << ' '
        << get(vertex_pmap, vit)
-       << std::endl;
+       << '\n';
   }
 
   //-------------------------------------------------------
@@ -811,66 +808,68 @@ output_to_medit(std::ostream& os,
   if ( print_each_facet_twice )
     number_of_triangles += number_of_triangles;
   
-  os << "Triangles" << std::endl
-     << number_of_triangles << std::endl;
+  os << "Triangles\n" 
+     << number_of_triangles << '\n';
 
   for( Facet_iterator fit = c3t3.facets_in_complex_begin();
        fit != c3t3.facets_in_complex_end();
        ++fit)
   {
-    for (int i=0; i<4; i++)
-    {
-      if (i != fit->second)
-      {
-        const Vertex_handle& vh = (*fit).first->vertex(i);
-        os << V[vh] << " ";
-      }
-    }
-    os << get(facet_pmap, *fit) << std::endl;
+    typename C3T3::Facet f = (*fit);
     
-    // Print triangle again if needed
+    // Apply priority among subdomains, to get consistent facet orientation per subdomain-pair interface.
     if ( print_each_facet_twice )
     {
-      for (int i=0; i<4; i++)
-      {
-        if (i != fit->second)
-        {
-          const Vertex_handle& vh = (*fit).first->vertex(i);
-          os << V[vh] << " ";
-        }
-      }
-      os << get(facet_twice_pmap, *fit) << std::endl;
+      // NOTE: We mirror a facet when needed to make it consistent with No_patch_facet_pmap_first/second.
+      if (f.first->subdomain_index() > f.first->neighbor(f.second)->subdomain_index())
+        f = tr.mirror_facet(f);
+    }
+    
+    // Get facet vertices in CCW order.
+    Vertex_handle vh1 = f.first->vertex((f.second + 1) % 4);
+    Vertex_handle vh2 = f.first->vertex((f.second + 2) % 4);
+    Vertex_handle vh3 = f.first->vertex((f.second + 3) % 4);
+    
+    // Facet orientation also depends on parity.
+    if (f.second % 2 != 0)
+      std::swap(vh2, vh3);
+    
+    os << V[vh1] << ' ' << V[vh2] << ' ' << V[vh3] << ' '; 
+    os << get(facet_pmap, *fit) << '\n';
+    
+    // Print triangle again if needed, with opposite orientation
+    if ( print_each_facet_twice )
+    {
+      os << V[vh3] << ' ' << V[vh2] << ' ' << V[vh1] << ' '; 
+      os << get(facet_twice_pmap, *fit) << '\n';
     }
   }
 
   //-------------------------------------------------------
   // Tetrahedra
   //-------------------------------------------------------
-  os << "Tetrahedra" << std::endl
-     << c3t3.number_of_cells_in_complex() << std::endl;
+  os << "Tetrahedra\n"
+     << c3t3.number_of_cells_in_complex() << '\n';
 
   for( Cell_iterator cit = c3t3.cells_in_complex_begin() ;
        cit != c3t3.cells_in_complex_end() ;
        ++cit )
   {
     for (int i=0; i<4; i++)
-      os << V[cit->vertex(i)] << " ";
+      os << V[cit->vertex(i)] << ' ';
 
-    os << get(cell_pmap, cit) << std::endl;
+    os << get(cell_pmap, cit) << '\n';
   }
 
   //-------------------------------------------------------
   // End
   //-------------------------------------------------------
-  os << "End" << std::endl;
+  os << "End\n";
 
 } // end output_to_medit(...)
 
 } // end namespace Mesh_3
 
-  
-
-  
 /**
  * @brief outputs mesh to medit format
  * @param os the stream
@@ -885,7 +884,7 @@ void
 output_to_medit(std::ostream& os,
                 const C3T3& c3t3,
                 bool rebind = false,
-                bool show_patches = false) 
+                bool show_patches = false)
 {
   if ( rebind )
   {
