@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Triangulation_3/include/CGAL/IO/Triangulation_geomview_ostream_3.h $
+// $Id: Triangulation_geomview_ostream_3.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Sylvain Pion
@@ -21,6 +22,9 @@
 #ifndef CGAL_IO_TRIANGULATION_GEOMVIEW_OSTREAM_3_H
 #define CGAL_IO_TRIANGULATION_GEOMVIEW_OSTREAM_3_H
 
+#include <CGAL/license/Triangulation_3.h>
+
+
 #include <CGAL/IO/Geomview_stream.h>
 #include <CGAL/Triangulation_3.h>
 
