@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Interval_skip_list/include/CGAL/Interval_skip_list.h $
+// $Id: Interval_skip_list.h f0c1d3a 2018-04-19T14:49:05+01:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Andreas Fabri
@@ -22,6 +23,9 @@
 #ifndef CGAL_INTERVAL_SKIP_LIST_H
 #define CGAL_INTERVAL_SKIP_LIST_H
 
+#include <CGAL/license/Interval_skip_list.h>
+
+
 #include <CGAL/basic.h>
 #include <list>
 #include <iostream>
@@ -354,7 +358,8 @@ class Interval_for_container : public Interval_
 #ifdef CGAL_ISL_USE_CCC
     static Compact_container<IntervalListElt<Interval> > compact_container;
 #endif
-    std::allocator<IntervalListElt<Interval> > alloc;
+    typedef std::allocator<IntervalListElt<Interval> > Alloc;
+    Alloc alloc;
 
   public:
 
@@ -379,7 +384,11 @@ class Interval_for_container : public Interval_
       return it;
 #else
       IntervalListElt<Interval> *elt_ptr = alloc.allocate(1);
+#ifdef CGAL_CXX11
+      std::allocator_traits<Alloc>::construct(alloc,elt_ptr, I);
+#else      
       alloc.construct(elt_ptr, I);
+#endif
       return elt_ptr;
       //return new IntervalListElt<Interval>(I);
 #endif
@@ -390,7 +399,12 @@ class Interval_for_container : public Interval_
 #ifdef CGAL_ISL_USE_CCC
       compact_container.erase(I);
 #else
+
+#ifdef CGAL_CXX11
+      std::allocator_traits<Alloc>::destroy(alloc,I);
+#else
       alloc.destroy(I);
+#endif
       alloc.deallocate(I,1);
       //delete I;
 #endif
