@@ -16,8 +16,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Kernel_23/include/CGAL/Kernel/global_functions_2.h $
+// $Id: global_functions_2.h 0698f79 2017-10-20T23:34:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Sylvain Pion
@@ -287,6 +288,16 @@ compare_distance_to_point(const Point_2<K>& p,
   return internal::compare_distance_to_point(p, q, r, K());
 }
 
+template <class K >
+inline
+typename K::Comparison_result
+compare_power_distance(const Point_2<K> &r,
+                       const Weighted_point_2<K> &p,
+                       const Weighted_point_2<K> &q)
+{
+  return internal::compare_power_distance(r, p, q, K());
+}
+
 template <class K >
 inline
 typename K::Comparison_result
@@ -332,18 +343,41 @@ compare_lexicographically_xy(const Point_2<K> &p,
 template < class K >
 inline
 typename K::Comparison_result
-compare_slopes(const Line_2<K> &l1, const Line_2<K> &l2)
+compare_slope(const Line_2<K> &l1, const Line_2<K> &l2)
 {
-  return internal::compare_slopes(l1, l2, K());
+  return internal::compare_slope(l1, l2, K());
 }
 
 template < class K >
 inline
 typename K::Comparison_result
+compare_slope(const Segment_2<K> &s1, const Segment_2<K> &s2)
+{
+  return internal::compare_slope(s1, s2, K());
+}
+
+
+#ifndef CGAL_NO_DEPRECATED_CODE
+// kept for backward compatibility
+template < class K >
+CGAL_DEPRECATED_MSG("This function is deprecated. CGAL::compare_slope() should be used instead")
+inline
+typename K::Comparison_result
+compare_slopes(const Line_2<K> &l1, const Line_2<K> &l2)
+{
+  return internal::compare_slope(l1, l2, K());
+}
+
+// kept for backward compatibility
+template < class K >
+CGAL_DEPRECATED_MSG("This function is deprecated. CGAL::compare_slope() should be used instead")
+inline
+typename K::Comparison_result
 compare_slopes(const Segment_2<K> &s1, const Segment_2<K> &s2)
 {
-  return internal::compare_slopes(s1, s2, K());
+  return internal::compare_slope(s1, s2, K());
 }
+#endif
 
 template < class K >
 inline
@@ -714,6 +748,13 @@ lexicographically_yx_larger_or_equal(const Point_2<K> &p, const Point_2<K> &q)
   return internal::lexicographically_yx_larger_or_equal(p, q, K());
 }
 
+template < class K >
+typename K::FT
+l_infinity_distance(const Point_2<K> &p, const Point_2<K> &q)
+{
+  return internal::l_infinity_distance(p,q, K());
+}
+
 template < class K >
 inline
 typename K::Point_2
@@ -932,11 +973,89 @@ orientation(const Vector_2<K> &u, const Vector_2<K> &v)
 
 // parallel() functions are in global_functions.h
 
+template <class K >
+inline
+typename K::FT
+power_product(const Weighted_point_2<K> &p,
+              const Weighted_point_2<K> &q)
+{
+  return internal::power_product(p, q, K());
+}
+
+template <class K >
+inline
+typename K::Bounded_side
+power_side_of_bounded_power_circle(const Weighted_point_2<K> &p,
+                                   const Weighted_point_2<K> &q)
+{
+  return internal::power_side_of_bounded_power_circle(p, q, K());
+}
+
+template <class K >
+inline
+typename K::Bounded_side
+power_side_of_bounded_power_circle(const Weighted_point_2<K> &p,
+                                   const Weighted_point_2<K> &q,
+                                   const Weighted_point_2<K> &r)
+{
+  return internal::power_side_of_bounded_power_circle(p, q, r, K());
+}
+
+template <class K >
+inline
+typename K::Bounded_side
+power_side_of_bounded_power_circle(const Weighted_point_2<K> &p,
+                                   const Weighted_point_2<K> &q,
+                                   const Weighted_point_2<K> &r,
+                                   const Weighted_point_2<K> &s)
+{
+  return internal::power_side_of_bounded_power_circle(p, q, r, s, K());
+}
+
+template <typename K>
+inline
+typename K::Orientation
+power_side_of_oriented_power_circle(const Weighted_point_2<K> &p,
+                                    const Weighted_point_2<K> &q)
+{
+  return internal::power_side_of_oriented_power_circle(p, q, K());
+}
+
+template <typename K>
+inline
+typename K::Orientation
+power_side_of_oriented_power_circle(const Weighted_point_2<K> &p,
+                                    const Weighted_point_2<K> &q,
+                                    const Weighted_point_2<K> &r)
+{
+  return internal::power_side_of_oriented_power_circle(p, q, r, K());
+}
+
+template <typename K>
+inline
+typename K::Orientation
+power_side_of_oriented_power_circle(const Weighted_point_2<K> &p,
+                                    const Weighted_point_2<K> &q,
+                                    const Weighted_point_2<K> &r,
+                                    const Weighted_point_2<K> &s)
+{
+  return internal::power_side_of_oriented_power_circle(p, q, r, s, K());
+}
+
+template <class K>
+inline
+typename K::Line_2
+radical_axis(const Weighted_point_2<K> &p,
+             const Weighted_point_2<K> &q)
+{
+  return internal::radical_axis(p, q, K());
+}
+
 template <class K>
 inline
 typename K::Line_2
 radical_line(const Circle_2<K> &s1,
-              const Circle_2<K> &s2)
+             const Circle_2<K> &s2)
 {
   return K().construct_radical_line_2_object()(s1,s2);
 }
@@ -949,6 +1068,14 @@ right_turn(const Point_2<K> &p, const Point_2<K> &q, const Point_2<K> &r)
   return internal::right_turn(p, q, r, K());
 }
 
+template < class K >
+inline
+typename K::FT
+scalar_product(const Vector_2<K> &v, const Vector_2<K> &w)
+{
+  return K().compute_scalar_product_2_object()(v, w);
+}
+
 template <class K>
 inline
 typename K::Bounded_side
@@ -1005,6 +1132,43 @@ squared_radius(const Point_2<K>& p, const Point_2<K>& q, const Point_2<K>& r)
   return internal::squared_radius(p, q, r, K());
 }
 
+template < class K >
+inline
+typename K::FT
+squared_radius_smallest_orthogonal_circle(const Weighted_point_2<K> &p)
+{
+  return internal::squared_radius_smallest_orthogonal_circle(p, K());
+}
+
+template < class K >
+inline
+typename K::FT
+squared_radius_smallest_orthogonal_circle(const Weighted_point_2<K> &p,
+                                          const Weighted_point_2<K> &q)
+{
+  return internal::squared_radius_smallest_orthogonal_circle(p, q, K());
+}
+
+template < class K >
+inline
+typename K::FT
+squared_radius_smallest_orthogonal_circle(const Weighted_point_2<K> &p,
+                                          const Weighted_point_2<K> &q,
+                                          const Weighted_point_2<K> &r)
+{
+  return internal::squared_radius_smallest_orthogonal_circle(p, q, r, K());
+}
+
+template < class K >
+inline
+typename K::Point_2
+weighted_circumcenter(const Weighted_point_2<K> &p,
+                      const Weighted_point_2<K> &q,
+                      const Weighted_point_2<K> &r)
+{
+  return internal::weighted_circumcenter(p, q, r, K());
+}
+
 template < class K >
 inline
 typename K::Boolean
