@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Inscribed_areas/include/CGAL/Largest_empty_iso_rectangle_2.h $
+// $Id: Largest_empty_iso_rectangle_2.h bc455b4 2019-01-15T19:25:45+01:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Eli Packer (algorithm), Andreas Fabri (cgal conformance)
@@ -21,6 +22,9 @@
 #ifndef CGAL_LARGEST_EMPTY_ISO_RECTANGLE_2_H
 #define CGAL_LARGEST_EMPTY_ISO_RECTANGLE_2_H
 
+#include <CGAL/license/Inscribed_areas.h>
+
+
 /*! \file
  * The implementation of the Largest_empty_iso_rectangle_2<Traits> class.
  */
@@ -250,16 +254,6 @@ public:
     Point_2 y_part;// the y coordinate of the point
     Point_2 original;
 
-    Internal_point &
-     operator=(const Internal_point &other)
-    {
-      x_part = other.x_part;
-      y_part = other.y_part;
-      original = other.original;
-
-      return(*this);
-    }
-
     Internal_point() // no real value - just to allow construction of LER
       : x_part(Point_2(0,0)), y_part(Point_2(0,0)), original(Point_2(0,0)) {}
 
