@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Mesh_3/include/CGAL/Mesh_3/Sliver_perturber.h $
+// $Id: Sliver_perturber.h c28e05b 2018-02-23T12:50:06+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Stephane Tayeb
@@ -25,6 +26,10 @@
 #ifndef CGAL_MESH_3_SLIVER_PERTURBER_H
 #define CGAL_MESH_3_SLIVER_PERTURBER_H
 
+#include <CGAL/license/Mesh_3.h>
+
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/Mesh_3/config.h>
 
 
@@ -43,10 +48,11 @@
 #include <CGAL/Mesh_3/vertex_perturbation.h>
 #include <CGAL/Mesh_3/C3T3_helpers.h>
 #include <CGAL/Mesh_optimization_return_code.h>
-#include <CGAL/Timer.h>
+#include <CGAL/Real_timer.h>
 #include <CGAL/Mesh_3/Null_perturber_visitor.h>
 #include <CGAL/Mesh_3/sliver_criteria.h>
 #include <CGAL/Has_timestamp.h>
+#include <CGAL/Hash_handles_with_or_without_timestamps.h>
 
 #include <CGAL/Mesh_3/Concurrent_mesher_config.h>
 #include <CGAL/Mesh_3/Worksharing_data_structures.h>
@@ -62,12 +68,13 @@
 
 #include <boost/format.hpp>
 #ifdef CGAL_MESH_3_USE_RELAXED_HEAP
+#  error This option CGAL_MESH_3_USE_RELAXED_HEAP is no longer supported
+// The reason is that the Boost relaxed heap does not ensure a strict order
+// of the priority queue.
 #include <boost/pending/relaxed_heap.hpp>
 #else
 #include <CGAL/Modifiable_priority_queue.h>
 #endif //CGAL_MESH_3_USE_RELAXED_HEAP
-#include <boost/lambda/lambda.hpp>
-#include <boost/lambda/bind.hpp>
 #include <boost/ptr_container/ptr_vector.hpp>
 #include <boost/type_traits/is_convertible.hpp>
 
@@ -77,32 +84,8 @@
 
 namespace CGAL {
 
-namespace internal { namespace Mesh_3 {
-
-  // Hash function for boost::unordered_map<Vertex_handle,...>
-  template <typename Tr, bool HasTimestamp>
-  struct VHash
-  {
-    typedef typename Tr::Vertex_handle Vertex_handle;
-    std::size_t operator()(Vertex_handle vh) const
-    {
-      return vh->time_stamp();
-    }
-  };
-  template <typename Tr>
-  struct VHash<Tr, false>
-  {
-    typedef typename Tr::Vertex_handle Vertex_handle;
-    std::size_t operator()(Vertex_handle vh) const
-    {
-      return boost::hash_value(&*vh);
-    }
-  };
-}} // end internal::Mesh_3
-
 namespace Mesh_3 {
 
-
 /**
 * @class PVertex
 * Vertex with associated perturbation datas
@@ -468,13 +451,15 @@ class Sliver_perturber
   typedef Sliver_perturber_base<
     typename C3T3::Triangulation, Concurrency_tag>                      Base;
 
-  typedef typename C3T3::Triangulation  Tr;
-  typedef typename Tr::Geom_traits      Gt;
+  typedef typename C3T3::Triangulation          Tr;
+  typedef typename Tr::Geom_traits              Gt;
 
   typedef typename Tr::Cell_handle              Cell_handle;
   typedef typename Base::Vertex_handle          Vertex_handle;
   typedef typename Tr::Vertex                   Vertex;
-  typedef typename MeshDomain::Point_3          Point_3;
+
+  typedef typename Tr::Bare_point               Bare_point;
+  typedef typename Tr::Weighted_point           Weighted_point;
 
   typedef typename std::vector<Cell_handle>     Cell_vector;
   typedef typename std::vector<Vertex_handle>   Vertex_vector;
@@ -503,7 +488,7 @@ private:
 
   typedef PVertex_<FT,
                    Vertex_handle,
-                   Point_3,
+                   Bare_point,
                    SliverCriterion,
                    Perturbation,
                    Concurrency_tag> PVertex;
@@ -749,7 +734,7 @@ private:
 
   // Timer
   double time_limit_;
-  CGAL::Timer running_time_;
+  CGAL::Real_timer running_time_;
 };
 
 
@@ -787,7 +772,7 @@ operator()(Visitor visitor)
 {
   //check criterion bound
   if ( sliver_criterion_.sliver_bound() == 0 )
-    sliver_criterion_.set_sliver_bound(Sc::default_value);
+    sliver_criterion_.set_sliver_bound(sliver_criterion_.get_default_value());
 
   // Reset sliver value cache
   helper_.reset_cache();
@@ -929,7 +914,7 @@ Sliver_perturber<C3T3,Md,Sc,V_>::
 perturb(const FT& sliver_bound, PQueue& pqueue, Visitor& visitor) const
 {
 #ifdef CGAL_MESH_3_PERTURBER_HIGH_VERBOSITY
-  CGAL::Timer timer;
+  CGAL::Real_timer timer;
   timer.start();
   std::streamsize prec = std::cerr.precision(4);
   std::cerr << "Perturb sliver vertices (bound: " << sliver_bound
@@ -1115,18 +1100,15 @@ build_priority_queue(const FT& sliver_bound, PQueue& pqueue) const
   CGAL_precondition(pqueue.empty());
 
 #ifdef CGAL_MESH_3_PERTURBER_HIGH_VERBOSITY
-  CGAL::Timer timer;
+  CGAL::Real_timer timer;
   timer.start();
   std::cerr << "Build pqueue...";
 #endif
 
   int pqueue_size = 0;
 
-  typedef typename std::iterator_traits<Vertex_handle>::value_type Vertex;
-  typedef CGAL::internal::Has_timestamp<Vertex> Vertex_has_timestamp;
-  using CGAL::internal::Mesh_3::VHash;
-  typedef VHash<Tr, Vertex_has_timestamp::value> Hash_fct;
-  typedef boost::unordered_map<Vertex_handle,PVertex,Hash_fct> M;
+  typedef CGAL::Hash_handles_with_or_without_timestamps            Hash_fct;
+  typedef boost::unordered_map<Vertex_handle, PVertex, Hash_fct>   M;
 
   M vpm;
   for ( typename Tr::Finite_cells_iterator cit = tr_.finite_cells_begin();
@@ -1180,7 +1162,7 @@ build_priority_queue(const FT& sliver_bound, PQueue& pqueue) const
   CGAL_precondition(pqueue.empty());
 
 #ifdef CGAL_MESH_3_PERTURBER_HIGH_VERBOSITY
-  CGAL::Timer timer;
+  CGAL::Real_timer timer;
   timer.start();
   std::cerr << "Build pqueue...";
 #endif
@@ -1294,6 +1276,8 @@ perturb_vertex( PVertex pv
               , bool *could_lock_zone
               ) const
 {
+  typename Gt::Construct_point_3 cp = tr_.geom_traits().construct_point_3_object();
+
 #ifdef CGAL_CONCURRENT_MESH_3_PROFILING
   static Profile_branch_counter_3 bcounter(
     "early withdrawals / late withdrawals / successes [Perturber]");
@@ -1307,8 +1291,9 @@ perturb_vertex( PVertex pv
     return;
   }
 
-  Point_3 p = pv.vertex()->point();
-  if (!helper_.try_lock_point_no_spin(p) || p != pv.vertex()->point())
+  Bare_point p = cp(pv.vertex()->point());
+  if (!helper_.try_lock_point_no_spin(pv.vertex()->point()) ||
+      ! tr_.geom_traits().equal_3_object()(p, cp(pv.vertex()->point())))
   {
 #ifdef CGAL_CONCURRENT_MESH_3_PROFILING
     bcounter.increment_branch_2(); // THIS is an early withdrawal!
@@ -1551,11 +1536,11 @@ void
 Sliver_perturber<C3T3,Md,Sc,V_>::
 initialize_vertices_id() const
 {
-  namespace bl = boost::lambda;
   int cur_id = 0;
-
-  std::for_each(tr_.finite_vertices_begin(), tr_.finite_vertices_end(),
-                bl::bind(&Vertex::set_meshing_info, &bl::_1, bl::var(cur_id)++));
+  for(typename Tr::Finite_vertices_iterator it = tr_.finite_vertices_begin();
+      it != tr_.finite_vertices_end(); ++it) {
+    it->set_meshing_info(cur_id++);
+  }
 }
 
 
@@ -1660,4 +1645,6 @@ enqueue_task(const PVertex &pv,
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_MESH_3_SLIVERS_PERTURBER_H
