@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Mesh_3/include/CGAL/Mesh_3/mesh_standard_facet_criteria.h $
+// $Id: mesh_standard_facet_criteria.h 0fcc6c0 2017-12-08T11:31:30+01:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Stephane Tayeb
@@ -27,6 +28,9 @@
 #ifndef CGAL_MESH_3_MESH_STANDARD_FACET_CRITERIA_H
 #define CGAL_MESH_3_MESH_STANDARD_FACET_CRITERIA_H
 
+#include <CGAL/license/Mesh_3.h>
+
+
 #include <CGAL/Mesh_3/config.h>
 
 #include <CGAL/number_utils.h> // for to_double
@@ -65,7 +69,7 @@ private:
 
   typedef Mesh_3::Abstract_criterion<Tr,Visitor_> Base;
   typedef typename Base::Quality Quality;
-  typedef typename Base::Badness Badness;
+  typedef typename Base::Is_bad  Is_bad;
 
   typedef Aspect_ratio_criterion<Tr,Visitor_> Self;
 
@@ -102,27 +106,33 @@ protected:
     }
   }
 
-  virtual Badness do_is_bad (const Facet& f) const
+  virtual Is_bad do_is_bad(const Tr& tr, const Facet& f) const
   {
     CGAL_assertion (f.first->is_facet_on_surface(f.second));
     CGAL_assertion (B_ != 0);
 
-    typedef typename Tr::Geom_traits Gt;
-    typedef typename Tr::Point Point_3;
+    typedef typename Tr::Geom_traits      Gt;
+    typedef typename Tr::Bare_point       Bare_point;
+    typedef typename Tr::Weighted_point   Weighted_point;
 
-    const typename Gt::Construct_triangle_3 triangle =
-        Gt().construct_triangle_3_object();
-    const typename Gt::Compute_squared_distance_3 distance =
-        Gt().compute_squared_distance_3_object();
-    const typename Gt::Compute_squared_area_3 area =
-        Gt().compute_squared_area_3_object();
+    typedef typename Gt::Compute_squared_area_3     Area;
+    typedef typename Gt::Compute_squared_distance_3 Distance;
+    typedef typename Gt::Construct_point_3          Construct_point_3;
+    typedef typename Gt::Construct_triangle_3       Construct_triangle_3;
 
-    const Point_3& p1 = f.first->vertex((f.second+1)&3)->point();
-    const Point_3& p2 = f.first->vertex((f.second+2)&3)->point();
-    const Point_3& p3 = f.first->vertex((f.second+3)&3)->point();
+    Area area = tr.geom_traits().compute_squared_area_3_object();
+    Distance distance = tr.geom_traits().compute_squared_distance_3_object();
+    Construct_point_3 cp = tr.geom_traits().construct_point_3_object();
+    Construct_triangle_3 triangle = tr.geom_traits().construct_triangle_3_object();
+
+    const Weighted_point& wp1 = tr.point(f.first, (f.second+1)&3);
+    const Weighted_point& wp2 = tr.point(f.first, (f.second+2)&3);
+    const Weighted_point& wp3 = tr.point(f.first, (f.second+3)&3);
+    const Bare_point& p1 = cp(wp1);
+    const Bare_point& p2 = cp(wp2);
+    const Bare_point& p3 = cp(wp3);
 
     const FT triangle_area = area(triangle(p1,p2,p3));
-
     const FT d12 = distance(p1,p2);
     const FT d13 = distance(p1,p3);
     const FT d23 = distance(p2,p3);
@@ -138,10 +148,10 @@ protected:
       std::cerr << "Bad facet (angle criterion): aspect_ratio[" << aspect_ratio
                 << "] bound[" << B_ << "]" << std::endl;
 #endif
-      return Badness(Quality(aspect_ratio));
+      return Is_bad(Quality(aspect_ratio));
     }
     else
-      return Badness();
+      return Is_bad();
   }
 
 private:
@@ -154,6 +164,11 @@ private:
 template <typename Tr, typename Visitor_>
 class Curvature_size_criterion :
   public Mesh_3::Abstract_criterion<Tr, Visitor_>
+{};
+
+template <typename Tr, typename Visitor_>
+class Uniform_curvature_size_criterion :
+  public Curvature_size_criterion<Tr, Visitor_>
 {
 private:
   typedef typename Tr::Facet Facet;
@@ -161,14 +176,14 @@ private:
 
   typedef Mesh_3::Abstract_criterion<Tr,Visitor_> Base;
   typedef typename Base::Quality Quality;
-  typedef typename Base::Badness Badness;
+  typedef typename Base::Is_bad  Is_bad;
 
-  typedef Curvature_size_criterion<Tr,Visitor_> Self;
+  typedef Uniform_curvature_size_criterion<Tr,Visitor_> Self;
 
 public:
   // Nb: the default bound of the criterion is such that the criterion
   // is always fulfilled
-  Curvature_size_criterion(const FT b = 0) : B_(b * b) {}
+  Uniform_curvature_size_criterion(const FT b = 0) : B_(b * b) {}
 
 protected:
   virtual void do_accept(Visitor_& v) const
@@ -182,26 +197,26 @@ protected:
     return new Self(*this);
   }
 
-  virtual Badness do_is_bad (const Facet& f) const
+  virtual Is_bad do_is_bad(const Tr& tr, const Facet& f) const
   {
     CGAL_assertion(f.first->is_facet_on_surface(f.second));
     CGAL_assertion (B_ != 0);
 
-    typedef typename Tr::Geom_traits Gt;
-    typedef typename Tr::Point Point_3;
+    typedef typename Tr::Geom_traits    Gt;
+    typedef typename Tr::Weighted_point Weighted_point;
+    typedef typename Tr::Bare_point Bare_point;
 
-    typename Gt::Compute_squared_distance_3 distance =
-        Gt().compute_squared_distance_3_object();
-    typename Gt::Construct_weighted_circumcenter_3 circumcenter =
-        Gt().construct_weighted_circumcenter_3_object();
+    typename Gt::Construct_weighted_circumcenter_3 weighted_circumcenter =
+        tr.geom_traits().construct_weighted_circumcenter_3_object();
 
-    const Point_3& p1 = f.first->vertex((f.second+1)&3)->point();
-    const Point_3& p2 = f.first->vertex((f.second+2)&3)->point();
-    const Point_3& p3 = f.first->vertex((f.second+3)&3)->point();
+    const Weighted_point& p1 = tr.point(f.first, (f.second+1)&3);
+    const Weighted_point& p2 = tr.point(f.first, (f.second+2)&3);
+    const Weighted_point& p3 = tr.point(f.first, (f.second+3)&3);
 
-    const Point_3 c = circumcenter(p1,p2,p3);
+    const Bare_point c = weighted_circumcenter(p1,p2,p3);
+    const Bare_point& center = f.first->get_facet_surface_center(f.second);
 
-    const FT sq_dist = distance(c, f.first->get_facet_surface_center(f.second));
+    const FT sq_dist = tr.min_squared_distance(c, center);
 
     if ( sq_dist > B_ )
     {
@@ -209,17 +224,91 @@ protected:
       std::cerr << "Bad facet (curvature size): sq_dist[" << sq_dist
                 << "] bound[" << B_ << "]\n";
 #endif
-      return Badness(Quality(B_/sq_dist));
+      return Is_bad(Quality(B_/sq_dist));
     }
     else
-      return Badness();
+      return Is_bad();
   }
 
 private:
   FT B_;
 
-};  // end Curvature_size_criterion
+};  // end Uniform_curvature_size_criterion
 
+// Variable size Criterion class
+template <typename Tr, typename Visitor_, typename SizingField>
+class Variable_curvature_size_criterion :
+  public Curvature_size_criterion<Tr, Visitor_>
+{
+private:
+  typedef typename Tr::Facet            Facet;
+  typedef typename Tr::Geom_traits::FT  FT;
+  typedef typename Tr::Vertex::Index    Index;
+
+  typedef Mesh_3::Abstract_criterion<Tr,Visitor_> Base;
+  typedef typename Base::Quality Quality;
+  typedef typename Base::Is_bad  Is_bad;
+
+  typedef Variable_curvature_size_criterion<Tr,Visitor_,SizingField> Self;
+  typedef SizingField Sizing_field;
+
+public:
+  // Nb: the default bound of the criterion is such that the criterion
+  // is always fulfilled
+  Variable_curvature_size_criterion(const Sizing_field& s) : size_(s) {}
+
+protected:
+  virtual void do_accept(Visitor_& v) const
+  {
+    v.visit(*this);
+  }
+
+  virtual Self* do_clone() const
+  {
+    // Call copy ctor on this
+    return new Self(*this);
+  }
+
+  virtual Is_bad do_is_bad(const Tr& tr, const Facet& f) const
+  {
+    CGAL_assertion (f.first->is_facet_on_surface(f.second));
+
+    typedef typename Tr::Geom_traits    Gt;
+    typedef typename Tr::Weighted_point Weighted_point;
+    typedef typename Tr::Bare_point Bare_point;
+
+    typename Gt::Construct_weighted_circumcenter_3 weighted_circumcenter =
+      tr.geom_traits().construct_weighted_circumcenter_3_object();
+
+    const Weighted_point& p1 = tr.point(f.first, (f.second+1)&3);
+    const Weighted_point& p2 = tr.point(f.first, (f.second+2)&3);
+    const Weighted_point& p3 = tr.point(f.first, (f.second+3)&3);
+
+    const Bare_point c = weighted_circumcenter(p1,p2,p3);
+    const Bare_point& ball_center = f.first->get_facet_surface_center(f.second);
+    const Index& index = f.first->get_facet_surface_center_index(f.second);
+
+    const FT sq_dist = tr.min_squared_distance(c, ball_center);
+    const FT sq_bound = CGAL::square(size_(ball_center, 2, index));
+
+    CGAL_assertion(sq_bound > FT(0));
+
+    if ( sq_dist > sq_bound )
+    {
+#ifdef CGAL_MESH_3_DEBUG_FACET_CRITERIA
+      std::cerr << "Bad facet (curvature size): sq_dist[" << sq_dist
+      << "] bound[" << sq_bound << "]\n";
+#endif
+      return Is_bad(Quality(sq_bound/sq_dist));
+    }
+    else
+      return Is_bad();
+  }
+
+private:
+  Sizing_field size_;
+
+};  // end Variable_curvature_size_criterion
 
 // Size Criterion base class
 template < typename Tr, typename Visitor_ >
@@ -240,7 +329,7 @@ private:
   
   typedef Mesh_3::Abstract_criterion<Tr,Visitor_> Base;
   typedef typename Base::Quality Quality;
-  typedef typename Base::Badness Badness;
+  typedef typename Base::Is_bad  Is_bad;
   
   typedef Variable_size_criterion<Tr,Visitor_,SizingField> Self;
   typedef SizingField Sizing_field;
@@ -261,22 +350,23 @@ protected:
     // Call copy ctor on this
     return new Self(*this);
   }
-  
-  virtual Badness do_is_bad (const Facet& f) const
+
+  virtual Is_bad do_is_bad(const Tr& tr, const Facet& f) const
   {
     CGAL_assertion (f.first->is_facet_on_surface(f.second));
     
-    typedef typename Tr::Geom_traits Gt;
-    typedef typename Tr::Point Point_3;
-    
-    typename Gt::Compute_squared_distance_3 distance =
-      Gt().compute_squared_distance_3_object();
-    
-    const Point_3& p1 = f.first->vertex((f.second+1)&3)->point();
-    const Point_3& ball_center = f.first->get_facet_surface_center(f.second);
+    typedef typename Tr::Geom_traits    Gt;
+    typedef typename Tr::Bare_point     Bare_point;
+    typedef typename Tr::Weighted_point Weighted_point;
+
+    typename Gt::Construct_point_3 cp = tr.geom_traits().construct_point_3_object();
+
+    const Weighted_point& wp1 = tr.point(f.first, (f.second+1)&3);
+    const Bare_point& p1 = cp(wp1);
+    const Bare_point& ball_center = f.first->get_facet_surface_center(f.second);
     const Index& index = f.first->get_facet_surface_center_index(f.second);
-    
-    const FT sq_radius = distance(p1,ball_center);
+
+    const FT sq_radius = tr.min_squared_distance(p1, ball_center);
     const FT sq_bound = CGAL::square(size_(ball_center, 2, index));
     CGAL_assertion(sq_bound > FT(0));
     
@@ -286,10 +376,10 @@ protected:
       std::cerr << "Bad facet (uniform size): sq_radius[" << sq_radius
       << "] bound[" << sq_bound << "]\n";
 #endif
-      return Badness(Quality(sq_bound/sq_radius));
+      return Is_bad(Quality(sq_bound/sq_radius));
     }
     else
-      return Badness();
+      return Is_bad();
   }
   
 private:
@@ -310,7 +400,7 @@ private:
 
   typedef Mesh_3::Abstract_criterion<Tr,Visitor_> Base;
   typedef typename Base::Quality Quality;
-  typedef typename Base::Badness Badness;
+  typedef typename Base::Is_bad  Is_bad;
 
   typedef Uniform_size_criterion<Tr,Visitor_> Self;
   
@@ -331,21 +421,22 @@ protected:
     return new Self(*this);
   }
 
-  virtual Badness do_is_bad (const Facet& f) const
+  virtual Is_bad do_is_bad(const Tr& tr, const Facet& f) const
   {
     CGAL_assertion (f.first->is_facet_on_surface(f.second));
     CGAL_assertion (B_ != 0);
 
-    typedef typename Tr::Geom_traits Gt;
-    typedef typename Tr::Point Point_3;
+    typedef typename Tr::Geom_traits        Gt;
+    typedef typename Tr::Bare_point         Bare_point;
+    typedef typename Tr::Weighted_point     Weighted_point;
 
-    typename Gt::Compute_squared_distance_3 distance =
-        Gt().compute_squared_distance_3_object();
+    typename Gt::Construct_point_3 cp = tr.geom_traits().construct_point_3_object();
 
-    const Point_3& p1 = f.first->vertex((f.second+1)&3)->point();
+    const Weighted_point& wp1 = tr.point(f.first, (f.second+1)&3);
+    const Bare_point p1 = cp(wp1);
+    const Bare_point& ball_center = f.first->get_facet_surface_center(f.second);
 
-    const FT sq_radius = distance(
-        p1, f.first->get_facet_surface_center(f.second));
+    const FT sq_radius = tr.min_squared_distance(p1, ball_center);
 
     if ( sq_radius > B_ )
     {
@@ -353,10 +444,10 @@ protected:
       std::cerr << "Bad facet (uniform size): sq_radius[" << sq_radius
                 << "] bound[" << B_ << "]\n";
 #endif
-      return Badness(Quality(B_/sq_radius));
+      return Is_bad(Quality(B_/sq_radius));
     }
     else
-      return Badness();
+      return Is_bad();
   }
 
 private:
@@ -375,7 +466,7 @@ private:
 
   typedef Mesh_3::Abstract_criterion<Tr,Visitor_> Base;
   typedef typename Base::Quality Quality;
-  typedef typename Base::Badness Badness;
+  typedef typename Base::Is_bad  Is_bad;
 
   typedef Facet_on_surface_criterion<Tr,Visitor_> Self;
 
@@ -397,7 +488,7 @@ protected:
     return new Self(*this);
   }
 
-  virtual Badness do_is_bad (const Facet& f) const
+  virtual Is_bad do_is_bad(const Tr& /* tr */, const Facet& f) const
   {
     typedef typename Tr::Vertex_handle Vertex_handle;
     typedef typename Tr::Cell_handle Cell_handle;
@@ -416,10 +507,10 @@ protected:
 #ifdef CGAL_MESH_3_DEBUG_FACET_CRITERIA
       std::cerr << "Bad facet (on surface criterion)" << std::endl;
 #endif
-      return Badness(Quality(1));
+      return Is_bad(Quality(1));
     }
     else
-      return Badness();
+      return Is_bad();
   }
 }; // end class Facet_on_surface_criterion
 
@@ -433,7 +524,7 @@ private:
   
   typedef Mesh_3::Abstract_criterion<Tr,Visitor_> Base;
   typedef typename Base::Quality Quality;
-  typedef typename Base::Badness Badness;
+  typedef typename Base::Is_bad  Is_bad;
   
   typedef Facet_on_same_surface_criterion<Tr,Visitor_> Self;
   
@@ -454,8 +545,8 @@ protected:
     // Call copy ctor on this
     return new Self(*this);
   }
-  
-  virtual Badness do_is_bad (const Facet& f) const
+
+  virtual Is_bad do_is_bad(const Tr& /* tr */, const Facet& f) const
   {
     typedef typename Tr::Vertex_handle  Vertex_handle;
     typedef typename Tr::Cell_handle    Cell_handle;
@@ -483,7 +574,7 @@ protected:
       {
         if ( !(v2->index() == index) )
         {
-          return Badness(Quality(1));
+          return Is_bad(Quality(1));
         }
       }
       else
@@ -497,11 +588,11 @@ protected:
     {
       if ( is_index_initialized && !(v3->index() == index) )
       {
-        return Badness(Quality(1));
+        return Is_bad(Quality(1));
       } 
     }
     
-    return  Badness();			
+    return Is_bad();
   }
   
 }; // end class Facet_on_same_surface_criterion
@@ -518,7 +609,7 @@ class Facet_criterion_visitor
 public:
   typedef Mesh_3::Abstract_criterion<Tr, Self> Criterion;
   typedef typename Base::Quality Facet_quality;
-  typedef typename Base::Badness Facet_badness;
+  typedef typename Base::Is_bad  Is_facet_bad;
   typedef typename Base::Handle Handle;
   typedef Handle Facet;
 
@@ -557,13 +648,13 @@ class Facet_criterion_visitor_with_features
 
 public:  
   typedef typename Base::Quality  Facet_quality;
-  typedef typename Base::Badness  Facet_badness;
+  typedef typename Base::Is_bad   Is_facet_bad;
   typedef typename Base::Handle   Handle;
   typedef Handle                  Facet;
   
   // Constructor
-  Facet_criterion_visitor_with_features(const Facet& fh)
-    : Base(fh)
+  Facet_criterion_visitor_with_features(const Tr& tr, const Facet& fh)
+    : Base(tr, fh)
     , wp_nb_(0)
     , do_spheres_intersect_(false)
     , ratio_(0.)
@@ -571,14 +662,16 @@ public:
     , angle_ratio_(0.5*0.5*4.)
     , size_ratio_(0.4*0.4*4.)
   {
-    typedef typename Tr::Point        Point_3;
-    typedef typename Tr::Cell_handle  Cell_handle;
-    
-    typename Gt::Compare_weighted_squared_radius_3 compare =
-      Gt().compare_weighted_squared_radius_3_object();
-    
+    typedef typename Tr::Geom_traits    Gt;
+    typedef typename Tr::Weighted_point Weighted_point;
+    typedef typename Tr::Cell_handle    Cell_handle;
+
     typename Gt::Compute_squared_radius_smallest_orthogonal_sphere_3 sq_radius =
-      Gt().compute_squared_radius_smallest_orthogonal_sphere_3_object();
+      tr.geom_traits().compute_squared_radius_smallest_orthogonal_sphere_3_object();
+    typename Gt::Compute_weight_3 cw =
+      tr.geom_traits().compute_weight_3_object();
+    typename Gt::Compare_weighted_squared_radius_3 compare =
+      tr.geom_traits().compare_weighted_squared_radius_3_object();
 
     const Cell_handle& c = fh.first;
     const int& k = fh.second;
@@ -589,42 +682,45 @@ public:
     
     // Get number of weighted points, and ensure that they will be accessible
     // using k1...ki, if i is the number of weighted points.
-    if(c->vertex(k1)->point().weight() > FT(0))
-    { 
+    const Weighted_point& wpk1 = tr.point(c, k1);
+    if(compare(wpk1, FT(0)) == CGAL::SMALLER)
+    {
       ++wp_nb_;
     }
-    
-    if(c->vertex(k2)->point().weight() > FT(0))
-    { 
+
+    const Weighted_point& wpk2 = tr.point(c, k2);
+    if(compare(wpk2, FT(0)) == CGAL::SMALLER)
+    {
       if ( 0 == wp_nb_ ) { std::swap(k1,k2); }
       ++wp_nb_;
     }
-    
-    if(c->vertex(k3)->point().weight() > FT(0))
-    { 
+
+    const Weighted_point& wpk3 = tr.point(c, k3);
+    if(compare(wpk3, FT(0)) == CGAL::SMALLER)
+    {
       if ( 0 == wp_nb_ ) { std::swap(k1,k3); }
       if ( 1 == wp_nb_ ) { std::swap(k2,k3); }
       ++wp_nb_;
     }
-    
-    const Point_3& p1 = c->vertex(k1)->point();
-    const Point_3& p2 = c->vertex(k2)->point();
-    const Point_3& p3 = c->vertex(k3)->point();
-    
+
+    const Weighted_point& p1 = tr.point(c, k1);
+    const Weighted_point& p2 = tr.point(c, k2);
+    const Weighted_point& p3 = tr.point(c, k3);
+
     // Compute ratio
     switch ( wp_nb_ )
     {
       case 1:
       {
         FT r = (std::max)(sq_radius(p1,p2),sq_radius(p1,p3));
-        ratio_ = r / p1.weight();
+        ratio_ = r / cw(p1);
         break;
       }
         
       case 2:
       {
-        FT r13 = sq_radius(p1,p3) / p1.weight();
-        FT r23 = sq_radius(p2,p3) / p2.weight();
+        FT r13 = sq_radius(p1,p3) / cw(p1);
+        FT r23 = sq_radius(p2,p3) / cw(p2);
         ratio_ = (std::max)(r13, r23);
         
         do_spheres_intersect_ = (compare(p1,p2,FT(0)) != CGAL::LARGER);
