@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Mesh_3/include/CGAL/Mesh_facet_topology.h $
+// $Id: Mesh_facet_topology.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Stephane Tayeb
@@ -25,13 +26,19 @@
 #ifndef CGAL_MESH_FACET_TOPOLOGY_H
 #define CGAL_MESH_FACET_TOPOLOGY_H
 
+#include <CGAL/license/Mesh_3.h>
+
+
 namespace CGAL {
 
 enum Mesh_facet_topology {
   FACET_VERTICES_ON_SURFACE = 1,
-  FACET_VERTICES_ON_SAME_SURFACE_PATCH,
-  FACET_VERTICES_ON_SAME_SURFACE_PATCH_WITH_ADJACENCY_CHECK
-};  
+  FACET_VERTICES_ON_SAME_SURFACE_PATCH = 2,
+  FACET_VERTICES_ON_SAME_SURFACE_PATCH_WITH_ADJACENCY_CHECK = 3,
+  MANIFOLD_WITH_BOUNDARY = 8,
+  NO_BOUNDARY = 16,
+  MANIFOLD = 24
+};
   
 } // end namespace CGAL
 
