@@ -1,9 +1,10 @@
 // Copyright (c) 2009 INRIA Sophia-Antipolis (France).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org); you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; either version 3 of the License,
+// or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Mesh_3/include/CGAL/Mesh_optimization_return_code.h $
-// $Id: Mesh_optimization_return_code.h 56840 2010-06-17 14:02:02Z stayeb $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Mesher_level/include/CGAL/Mesh_optimization_return_code.h $
+// $Id: Mesh_optimization_return_code.h 0698f79 2017-10-20T23:34:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     : Stephane Tayeb
@@ -33,7 +35,8 @@ enum Mesh_optimization_return_code
   TIME_LIMIT_REACHED,
   CANT_IMPROVE_ANYMORE,
   CONVERGENCE_REACHED,
-  MAX_ITERATION_NUMBER_REACHED
+  MAX_ITERATION_NUMBER_REACHED,
+  ALL_VERTICES_FROZEN
 };
 
 
