@@ -1,9 +1,10 @@
 // Copyright (c) 1999-2003  ETH Zurich (Switzerland).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Min_quadrilateral_2/include/CGAL/Minimum_enclosing_quadrilateral_traits_2.h $
-// $Id: Minimum_enclosing_quadrilateral_traits_2.h 53957 2010-02-01 12:55:28Z spion $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Bounding_volumes/include/CGAL/Minimum_enclosing_quadrilateral_traits_2.h $
+// $Id: Minimum_enclosing_quadrilateral_traits_2.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Michael Hoffmann <hoffmann@inf.ethz.ch> and
@@ -21,9 +23,13 @@
 #ifndef CGAL_MINIMUM_ENCLOSING_QUADRILATERAL_TRAITS_2_H
 #define CGAL_MINIMUM_ENCLOSING_QUADRILATERAL_TRAITS_2_H 1
 
-#ifndef CGAL_NO_DEPRECATED_CODE
+#include <CGAL/license/Bounding_volumes.h>
+
+#define CGAL_DEPRECATED_HEADER "<CGAL/Minimum_enclosing_quadrilateral_traits_2.h>"
+#define CGAL_REPLACEMENT_HEADER "<CGAL/Min_quadrilateral_traits_2.h>"
+#include <CGAL/internal/deprecation_warning.h>
+
 #include <CGAL/Min_quadrilateral_traits_2.h>
-#endif // CGAL_NO_DEPRECATED_CODE
 
 #endif // ! (CGAL_MINIMUM_ENCLOSING_QUADRILATERAL_TRAITS_2_H)
 
