@@ -13,6 +13,10 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Number_types/include/CGAL/Mpzf.h $
+// $Id: Mpzf.h d715ddd 2019-10-25T14:38:26+02:00 Marc Glisse
+// SPDX-License-Identifier: LGPL-3.0+
+//
 // Author(s)	:  Marc Glisse
 
 #ifndef CGAL_MPZF_H
@@ -75,6 +79,11 @@
 #include <builtins.h>
 #endif
 
+#include <CGAL/assertions.h>
+#include <boost/config.hpp>
+#include <boost/detail/workaround.hpp>
+#include <boost/version.hpp>
+
 #if defined(BOOST_MSVC)
 #  pragma warning(push)
 #  pragma warning(disable:4146 4244 4267 4800)
@@ -84,11 +93,6 @@
      // int to bool performance
 #endif
 
-#if defined(__GNUC__) && defined(__GNUC_MINOR__) \
-    && (__GNUC__ * 100 + __GNUC_MINOR__) >= 408 \
-    && __cplusplus >= 201103L
-#define CGAL_CAN_USE_CXX11_THREAD_LOCAL
-#endif
 
 /*
 #ifdef CGAL_MPZF_NO_USE_CACHE
@@ -154,7 +158,7 @@ template <class T, class = void> struct pool2 {
   static bool empty() { return data() == 0; }
   static const int extra = 1; // TODO: handle the case where a pointer is larger than a mp_limb_t
   private:
-  BOOST_STATIC_ASSERT(sizeof(T) >= sizeof(T*));
+  CGAL_static_assertion(sizeof(T) >= sizeof(T*));
   static T& data () {
     static CGAL_MPZF_TLS T data_ = 0;
     return data_;
@@ -169,7 +173,7 @@ template <class T, class = void> struct pool3 {
   static bool empty() { return data() == 0; }
   static const int extra = 1; // TODO: handle the case where a pointer is larger than a mp_limb_t
   private:
-  BOOST_STATIC_ASSERT(sizeof(T) >= sizeof(T*));
+  CGAL_static_assertion(sizeof(T) >= sizeof(T*));
   struct cleaner {
     T data_ = 0;
     ~cleaner(){
@@ -224,7 +228,7 @@ inline int clz (boost::uint64_t x) {
 // In C++11, std::fill_n returns a pointer to the end, but in C++03,
 // it returns void.
 inline mp_limb_t* fill_n_ptr(mp_limb_t* p, int n, int c) {
-#if __cplusplus >= 201103L
+#if CGAL_CXX11
   return std::fill_n (p, n, c);
 #else
   mp_limb_t* q = p + n;
@@ -266,14 +270,16 @@ struct Mpzf {
 //#endif
 
   mp_limb_t* data_; /* data_[0] is never 0 (except possibly for 0). */
-  inline mp_limb_t*& data() { return data_; };
-  inline mp_limb_t const* data() const { return data_; };
+  inline mp_limb_t*& data() { return data_; }
+  inline mp_limb_t const* data() const { return data_; }
 
 #ifdef CGAL_MPZF_USE_CACHE
   mp_limb_t cache[cache_size + 1];
 #endif
   int size; /* Number of relevant limbs in data_. */
   int exp; /* The number is data_ (an integer) * 2 ^ (64 * exp). */
+  typedef int Exponent_type;
+  typedef int Size_type;
 
   struct allocate{};
   struct noalloc{};
@@ -296,7 +302,13 @@ struct Mpzf {
     data()[-1] = mini;
   }
   void clear(){
-    while(*--data()==0); // in case we skipped final zeroes
+    // while(*--data()==0);
+    // This line gave a misscompilation by Intel Compiler 2019
+    // (19.0.0.117). I replaced it by the following two lines:
+    // -- Laurent Rineau, sept. 2018
+    --data();
+    while(*data()==0) { --data(); } // in case we skipped final zeroes
+
 #ifdef CGAL_MPZF_USE_CACHE
     if (data() == cache) return;
 #endif
@@ -418,7 +430,7 @@ struct Mpzf {
     }
     int e1 = (int)dexp+13;
     // FIXME: make it more general! But not slower...
-    BOOST_STATIC_ASSERT(GMP_NUMB_BITS == 64);
+    CGAL_static_assertion(GMP_NUMB_BITS == 64);
     int e2 = e1 % 64;
     exp = e1 / 64 - 17;
     // 52+1023+13==17*64 ?
@@ -459,7 +471,7 @@ struct Mpzf {
     }
 #endif
     if(u.s.sig) size=-size;
-    //assert(to_double()==IA_force_to_double(d));
+    //CGAL_assertion(to_double()==IA_force_to_double(d));
   }
 
 #ifdef CGAL_USE_GMPXX
@@ -471,8 +483,8 @@ struct Mpzf {
     init_from_mpz_t(z.mpz());
   }
   void init_from_mpz_t(mpz_t const z){
-    exp=mpz_scan1(z,0)/GMP_NUMB_BITS;
-    size=mpz_size(z)-exp;
+    exp=Exponent_type(mpz_scan1(z,0)/GMP_NUMB_BITS);
+    size=Size_type(mpz_size(z)-exp);
     init(size);
     mpn_copyi(data(),z->_mp_d+exp,size);
   }
@@ -660,7 +672,7 @@ struct Mpzf {
 	  rdata=Mpzf_impl::fill_n_ptr(rdata,xexp-absysize,-1);
 	  mpn_sub_1(rdata, xdata, absxsize, 1);
 	  res.size=absxsize+xexp;
-	  if(res.data()[res.size-1]==0) --res.size;
+	  while(/*res.size>0&&*/res.data()[res.size-1]==0) --res.size;
 	  if(xsize<0) res.size=-res.size;
 	  return res;
 	} else {
@@ -800,9 +812,9 @@ struct Mpzf {
     else { mpn_copyi(res.data(), b.data(), bsize); }
     res.exp = 0; // Pick b.exp? or the average? 0 helps return 1 more often.
     if (asize < bsize)
-      res.size = mpn_gcd(res.data(), res.data(), bsize, tmp.data(), asize);
+      res.size = Size_type(mpn_gcd(res.data(), res.data(), bsize, tmp.data(), asize));
     else
-      res.size = mpn_gcd(res.data(), tmp.data(), asize, res.data(), bsize);
+      res.size = Size_type(mpn_gcd(res.data(), tmp.data(), asize, res.data(), bsize));
     if(rtz!=0) {
       mp_limb_t c = mpn_lshift(res.data(), res.data(), res.size, rtz);
       if(c) { res.data()[res.size]=c; ++res.size; }
@@ -859,6 +871,7 @@ struct Mpzf {
   Mpzf& operator+=(Mpzf const&x){ *this=*this+x; return *this; }
   Mpzf& operator-=(Mpzf const&x){ *this=*this-x; return *this; }
   Mpzf& operator*=(Mpzf const&x){ *this=*this*x; return *this; }
+  Mpzf& operator/=(Mpzf const&x){ *this=*this/x; return *this; }
 
   bool is_canonical () const {
     if (size == 0) return true;
@@ -883,7 +896,7 @@ struct Mpzf {
     if(size==0) return 0;
     int asize = std::abs(size);
     mp_limb_t top = data()[asize-1];
-    double dtop = top;
+    double dtop = (double)top;
     if(top >= (1LL<<53) || asize == 1) /* ok */ ;
     else { dtop += (double)data()[asize-2] * ldexp(1.,-GMP_NUMB_BITS); }
     return ldexp( (size<0) ? -dtop : dtop, (asize-1+exp) * GMP_NUMB_BITS);
@@ -902,13 +915,13 @@ struct Mpzf {
 	e += (11 - lz);
 	x >>= (11 - lz);
       }
-      dl = x;
-      dh = x + 1;
+      dl = double(x);
+      dh = double(x + 1);
       // Check for the few cases where dh=x works (asize==1 and the evicted
       // bits from x were 0s)
     }
     else if (asize == 1) {
-      dl = dh = x; // conversion is exact
+      dl = dh = double(x); // conversion is exact
     }
     else {
       mp_limb_t y = data()[asize-2];
@@ -916,8 +929,8 @@ struct Mpzf {
       x <<= (lz - 11);
       y >>= (75 - lz);
       x |= y;
-      dl = x;
-      dh = x + 1;
+      dl = double(x);
+      dh = double(x + 1);
       // Check for the few cases where dh=x works (asize==2 and the evicted
       // bits from y were 0s)
     }
@@ -1017,21 +1030,21 @@ std::istream& operator>> (std::istream& is, Mpzf& a)
       typedef Tag_false            Is_numerical_sensitive;
 
       struct Is_zero
-	: public std::unary_function< Type, bool > {
+	: public CGAL::cpp98::unary_function< Type, bool > {
 	  bool operator()( const Type& x ) const {
 	    return x.is_zero();
 	  }
 	};
 
       struct Is_one
-	: public std::unary_function< Type, bool > {
+	: public CGAL::cpp98::unary_function< Type, bool > {
 	  bool operator()( const Type& x ) const {
 	    return x.is_one();
 	  }
 	};
 
       struct Gcd
-	: public std::binary_function< Type, Type, Type > {
+	: public CGAL::cpp98::binary_function< Type, Type, Type > {
 	  Type operator()(
 	      const Type& x,
 	      const Type& y ) const {
@@ -1040,14 +1053,14 @@ std::istream& operator>> (std::istream& is, Mpzf& a)
 	};
 
       struct Square
-	: public std::unary_function< Type, Type > {
+	: public CGAL::cpp98::unary_function< Type, Type > {
 	  Type operator()( const Type& x ) const {
 	    return Mpzf_square(x);
 	  }
 	};
 
       struct Integral_division
-	: public std::binary_function< Type, Type, Type > {
+	: public CGAL::cpp98::binary_function< Type, Type, Type > {
 	  Type operator()(
 	      const Type& x,
 	      const Type& y ) const {
@@ -1056,14 +1069,14 @@ std::istream& operator>> (std::istream& is, Mpzf& a)
 	};
 
       struct Sqrt
-	: public std::unary_function< Type, Type > {
+	: public CGAL::cpp98::unary_function< Type, Type > {
 	  Type operator()( const Type& x) const {
 	    return Mpzf_sqrt(x);
 	  }
 	};
 
       struct Is_square
-	: public std::binary_function< Type, Type&, bool > {
+	: public CGAL::cpp98::binary_function< Type, Type&, bool > {
 	  bool operator()( const Type& x, Type& y ) const {
 	    // TODO: avoid doing 2 calls.
 	    if (!Mpzf_is_square(x)) return false;
@@ -1079,21 +1092,21 @@ std::istream& operator>> (std::istream& is, Mpzf& a)
   template <> struct Real_embeddable_traits< Mpzf >
     : public INTERN_RET::Real_embeddable_traits_base< Mpzf , CGAL::Tag_true > {
       struct Sgn
-	: public std::unary_function< Type, ::CGAL::Sign > {
+	: public CGAL::cpp98::unary_function< Type, ::CGAL::Sign > {
 	  ::CGAL::Sign operator()( const Type& x ) const {
 	    return x.sign();
 	  }
 	};
 
       struct To_double
-	: public std::unary_function< Type, double > {
+	: public CGAL::cpp98::unary_function< Type, double > {
 	    double operator()( const Type& x ) const {
 	      return x.to_double();
 	    }
 	};
 
       struct Compare
-	: public std::binary_function< Type, Type, Comparison_result > {
+	: public CGAL::cpp98::binary_function< Type, Type, Comparison_result > {
 	    Comparison_result operator()(
 		const Type& x,
 		const Type& y ) const {
@@ -1102,7 +1115,7 @@ std::istream& operator>> (std::istream& is, Mpzf& a)
 	};
 
       struct To_interval
-	: public std::unary_function< Type, std::pair< double, double > > {
+	: public CGAL::cpp98::unary_function< Type, std::pair< double, double > > {
 	    std::pair<double, double> operator()( const Type& x ) const {
 	      return x.to_interval();
 	    }
@@ -1123,6 +1136,36 @@ CGAL_DEFINE_COERCION_TRAITS_FROM_TO(mpz_class,Mpzf)
 
 }
 
+/* There isn't much Eigen can do with such a type,
+ * mostly this is here for IsInteger to protect people.
+ */
+namespace Eigen {
+  template<class> struct NumTraits;
+  template<> struct NumTraits<CGAL::Mpzf>
+  {
+    typedef CGAL::Mpzf Real;
+    /* Should this be Quotient<Mpzf>? Gmpq?  */
+    typedef CGAL::Mpzf NonInteger;
+    typedef CGAL::Mpzf Nested;
+    typedef CGAL::Mpzf Literal;
+
+    static inline Real epsilon() { return 0; }
+    static inline Real dummy_precision() { return 0; }
+
+    enum {
+      /* Only exact divisions are supported, close enough to an integer.
+       * This way we get compilation failures instead of runtime.  */
+      IsInteger = 1,
+      IsSigned = 1,
+      IsComplex = 0,
+      RequireInitialization = 1,
+      ReadCost = 6,
+      AddCost = 30,
+      MulCost = 50
+    };
+  };
+}
+
 #if defined(BOOST_MSVC)
 #  pragma warning(pop)
 #endif
