@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Nef_3/include/CGAL/Nef_3/SNC_external_structure.h $
+// $Id: SNC_external_structure.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Michael Seel       <seel@mpi-sb.mpg.de> 
@@ -24,6 +25,9 @@
 #ifndef CGAL_SNC_EXTERNAL_STRUCTURE_H
 #define CGAL_SNC_EXTERNAL_STRUCTURE_H
 
+#include <CGAL/license/Nef_3.h>
+
+
 #include <CGAL/Nef_S2/Normalizing.h>
 #include <CGAL/Nef_3/Pluecker_line_3.h>
 #include <CGAL/Nef_3/SNC_decorator.h>
@@ -40,6 +44,8 @@
 #define CGAL_NEF_DEBUG 43
 #include <CGAL/Nef_2/debug.h>
 
+#include <CGAL/use.h>
+
 namespace CGAL {
 
 struct int_lt {
@@ -121,21 +127,64 @@ int sign_of(const CGAL::Plane_3<R>& h)
 }
 
 struct Plane_lt {
-  template <typename R>
-  bool operator()(const CGAL::Plane_3<R>& h1,
-                  const CGAL::Plane_3<R>& h2) const
-  { 
-    typedef typename R::RT     RT;
-    RT diff = h1.a()-h2.a();
-    if ( (diff) != 0 ) return CGAL_NTS sign(diff) < 0;
-    diff = h1.b()-h2.b();
-    if ( (diff) != 0 ) return CGAL_NTS sign(diff) < 0;
-    diff = h1.c()-h2.c();
-    if ( (diff) != 0 ) return CGAL_NTS sign(diff) < 0;
-    diff = h1.d()-h2.d(); return CGAL_NTS sign(diff) < 0;
-  }
+    template <typename R>
+    bool operator()(const CGAL::Plane_3<R>& h1,
+                    const CGAL::Plane_3<R>& h2) const
+    {
+        typedef typename R::RT     RT;
+        typedef typename R::FT     FT;
+
+        bool flag=false;
+        FT ratioa,ratiob,ratioc,ratiod;
+        FT a1,b1,c1,d1,a2,b2,c2,d2;
+        a1=h1.a();
+        a2=h2.a();
+        b1=h1.b();
+        b2=h2.b();
+        c1=h1.c();
+        c2=h2.c();
+        d1=h1.d();
+        d2=h2.d();
+        if(a2==0 || a1==0)
+        {
+            if(a2==a1) ratioa=1;
+            else flag=true;
+        }
+        else{ratioa=a1/a2;}
+        if(b2==0 || b1==0)
+        {
+            if(b2==b1) ratiob=ratioa;
+            else flag=true;
+        }
+        else{ ratiob=b1/b2;}
+        if(c2==0 || c1==0)
+        {
+            if(c2==c1) ratioc=ratioa;
+            else flag=true;
+        }
+        else{ ratioc=c1/c2;}
+        if(d2==0 || d1==0)
+        {
+            if(d2==d1) ratiod=ratioc;
+            else flag=true;
+        }
+        else{ ratiod=d1/d2;}
+        if(flag || !(ratioa==ratiob && ratioc==ratiod && ratioa==ratioc))
+        {
+            RT diff = h1.a()-h2.a();
+            if ( (diff) != 0 ) return CGAL_NTS sign(diff) < 0;
+            diff = h1.b()-h2.b();
+            if ( (diff) != 0 ) return CGAL_NTS sign(diff) < 0;
+            diff = h1.c()-h2.c();
+            if ( (diff) != 0 ) return CGAL_NTS sign(diff) < 0;
+            diff = h1.d()-h2.d();
+            if ( (diff) != 0 ) return CGAL_NTS sign(diff) < 0;
+        }
+        return 0;
+    }
 };
 
+
 struct Plane_RT_lt {
   template <typename R>
   bool operator()(const CGAL::Plane_3<R>& h1,
@@ -245,10 +294,12 @@ public:
     }
 
     void visit(Vertex_handle h) { 
+      CGAL_USE(h);
       CGAL_NEF_TRACEN("visit v  "<<h->point());
     }
 
     void visit(Halfedge_handle h) { 
+      CGAL_USE(h);
       CGAL_NEF_TRACEN("visit he "<< h->source()->point());
     }
 
