@@ -1,9 +1,10 @@
 // Copyright (c) 1997-2000  Max-Planck-Institute Saarbruecken (Germany).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Nef_3/include/CGAL/Nef_3/SNC_simplify.h $
-// $Id: SNC_simplify.h 56667 2010-06-09 07:37:13Z sloriot $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Nef_3/include/CGAL/Nef_3/SNC_simplify.h $
+// $Id: SNC_simplify.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Peter Hachenberger <hachenberger@mpi-sb.mpg.de>
@@ -21,6 +23,9 @@
 #ifndef CGAL_SNC_SIMPLIFY_H
 #define CGAL_SNC_SIMPLIFY_H
 
+#include <CGAL/license/Nef_3.h>
+
+
 #include <CGAL/basic.h>
 #include <CGAL/Nef_3/SNC_decorator.h>
 #include <CGAL/Nef_S2/SM_decorator.h>
@@ -469,7 +474,7 @@ class SNC_simplify_base : public SNC_decorator<SNC_structure> {
      if( s1 == s2) {
        //       CGAL_NEF_TRACEN(IO->index(s1)<<'('<<IO->index(s2->twin())<<") to sloop");
        Halffacet_handle f = s1->facet();
-       Halffacet_handle facet = s1->facet();
+       // Halffacet_handle facet = s1->facet();
        SD.convert_edge_to_loop(s1);
        CGAL_assertion(SD.shalfloop() != SHalfloop_handle());
        D.add_sloop_to_facet( SD.shalfloop(), f);
@@ -590,7 +595,22 @@ class SNC_simplify_base : public SNC_decorator<SNC_structure> {
     SHalfloop_handle sl;
     CGAL_forall_shalfloops(sl, *this->sncp()) {
       SM_decorator SD(&*sl->incident_sface()->center_vertex());
-      SD.store_sm_boundary_object( sl, sl->incident_sface());
+      //I added the following 'if' because even if the map has been cleared, when merging edges
+      //if one sloop is created and the vertex not simplified, then the map is updated.
+      //In the example below, the edge e is removed and a sloop is create on the sphere map
+      //of a vertex v. But since three edges are still incident to v, v is not simplified. 
+      //There is one point where the distance between two non-edge-adjacent facets is 0.
+      //
+      //      |               |
+      // |    |  \  |  / |    |
+      // |    /   \ | /  |    /
+      // |   /     \|/   |   /
+      // |  /      /     |  /
+      // | /      / e    | /
+      // |/______/_______|/
+      //   
+      if (!SD.map()->is_sm_boundary_object(sl)) 
+        SD.store_sm_boundary_object( sl, sl->incident_sface());
     }
   }
 
