@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Partition_2/include/CGAL/Partition_is_valid_traits_2.h $
+// $Id: Partition_is_valid_traits_2.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Susan Hert <hert@mpi-sb.mpg.de>
@@ -21,6 +22,9 @@
 #ifndef CGAL_PARTITION_2_IS_VALID_TRAITS_H
 #define CGAL_PARTITION_2_IS_VALID_TRAITS_H
 
+#include <CGAL/license/Partition_2.h>
+
+
 namespace CGAL {
 
 template <class Traits, class PolygonIsValid>
