@@ -1,9 +1,9 @@
-// Copyright (c) 1999  
+// Copyright (c) 1999
 // Utrecht University (The Netherlands),
 // ETH Zurich (Switzerland),
 // INRIA Sophia-Antipolis (France),
 // Max-Planck-Institute Saarbruecken (Germany),
-// and Tel-Aviv University (Israel).  All rights reserved. 
+// and Tel-Aviv University (Israel).  All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org); you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public License as
@@ -16,8 +16,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Kernel_23/include/CGAL/Point_3.h $
+// $Id: Point_3.h 0698f79 2017-10-20T23:34:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     : Andreas Fabri, Stefan Schirra
@@ -39,7 +40,6 @@ template <class R_>
 class Point_3 : public R_::Kernel_base::Point_3
 {
   typedef typename R_::RT                    RT;
-  typedef typename R_::FT                    FT;
   typedef typename R_::Vector_3              Vector_3;
   typedef typename R_::Aff_transformation_3  Aff_transformation_3;
 
@@ -51,6 +51,7 @@ public:
   typedef Dimension_tag<3>  Ambient_dimension;
   typedef Dimension_tag<0>  Feature_dimension;
 
+  typedef typename R_::Weighted_point_3 Weighted_point_3;
   typedef typename R_::Kernel_base::Point_3  Rep;
   typedef typename R_::Cartesian_const_iterator_3 Cartesian_const_iterator;
 
@@ -75,6 +76,11 @@ public:
   Point_3(const Rep& p)
       : Rep(p) {}
 
+  explicit
+  Point_3(const Weighted_point_3& wp)
+    : Rep(wp.point())
+  {}
+
   template < typename T1, typename T2, typename T3 >
   Point_3(const T1& x, const T2& y, const T3& z)
     : Rep(typename R::Construct_point_3()(Return_base_tag(), x, y, z))
@@ -176,6 +182,21 @@ public:
     return t.transform(*this);
   }
 
+  Point_3<R_>&
+  operator+=(const Vector_3 &v)
+  {
+    *this = R().construct_translated_point_3_object()(*this, v);
+    return *this;
+  }
+
+  Point_3<R_>&
+  operator-=(const Vector_3 &v)
+  {
+    *this = R().construct_translated_point_3_object()(*this,
+                  R().construct_opposite_vector_3_object()(v));
+    return *this;
+  }
+
 };
 
 template <class R>
@@ -195,7 +216,7 @@ template <class R >
 std::ostream&
 insert(std::ostream& os, const Point_3<R>& p,const Cartesian_tag&)
 {
-    switch(os.iword(IO::mode)) {
+    switch(get_mode(os)) {
     case IO::ASCII :
         return os << p.x() << ' ' << p.y() << ' ' << p.z();
     case IO::BINARY :
@@ -213,7 +234,7 @@ template <class R >
 std::ostream&
 insert(std::ostream& os, const Point_3<R>& p,const Homogeneous_tag&)
 {
-  switch(os.iword(IO::mode))
+  switch(get_mode(os))
   {
     case IO::ASCII :
         return os << p.hx() << ' ' << p.hy() << ' ' << p.hz() << ' ' << p.hw();
@@ -243,8 +264,8 @@ template <class R >
 std::istream&
 extract(std::istream& is, Point_3<R>& p, const Cartesian_tag&)
 {
-    typename R::FT x, y, z;
-    switch(is.iword(IO::mode)) {
+  typename R::FT x(0), y(0), z(0);
+    switch(get_mode(is)) {
     case IO::ASCII :
         is >> iformat(x) >> iformat(y) >> iformat(z);
         break;
@@ -254,12 +275,13 @@ extract(std::istream& is, Point_3<R>& p, const Cartesian_tag&)
         read(is, z);
         break;
     default:
+        is.setstate(std::ios::failbit);
         std::cerr << "" << std::endl;
         std::cerr << "Stream must be in ascii or binary mode" << std::endl;
         break;
     }
     if (is)
-	p = Point_3<R>(x, y, z);
+      p = Point_3<R>(x, y, z);
     return is;
 }
 
@@ -269,7 +291,7 @@ std::istream&
 extract(std::istream& is, Point_3<R>& p, const Homogeneous_tag&)
 {
   typename R::RT hx, hy, hz, hw;
-  switch(is.iword(IO::mode))
+  switch(get_mode(is))
   {
     case IO::ASCII :
         is >> hx >> hy >> hz >> hw;
@@ -281,6 +303,7 @@ extract(std::istream& is, Point_3<R>& p, const Homogeneous_tag&)
         read(is, hw);
         break;
     default:
+        is.setstate(std::ios::failbit);
         std::cerr << "" << std::endl;
         std::cerr << "Stream must be in ascii or binary mode" << std::endl;
         break;
