@@ -12,14 +12,19 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Poisson_surface_reconstruction_3/include/CGAL/Poisson_reconstruction_function.h $
+// $Id: Poisson_reconstruction_function.h 2f9408f 2018-09-04T12:01:27+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Laurent Saboret, Pierre Alliez
 
 #ifndef CGAL_POISSON_RECONSTRUCTION_FUNCTION_H
 #define CGAL_POISSON_RECONSTRUCTION_FUNCTION_H
 
+#include <CGAL/license/Poisson_surface_reconstruction_3.h>
+
+#include <CGAL/disable_warnings.h>
+
 #ifndef CGAL_DIV_NORMALIZED
 #  ifndef CGAL_DIV_NON_NORMALIZED
 #    define CGAL_DIV_NON_NORMALIZED 1
@@ -144,7 +149,7 @@ struct Special_wrapper_of_two_functions_keep_pointers {
 
 
 /*!
-\ingroup PkgSurfaceReconstructionFromPointSets
+\ingroup PkgPoissonSurfaceReconstruction3Ref
 
 \brief Implementation of the Poisson Surface Reconstruction method.
   
@@ -300,7 +305,9 @@ public:
     NormalPMap normal_pmap ///< property map: `value_type of InputIterator` -> `Vector` (the *oriented* normal of an input point).
   )
     : m_tr(new Triangulation), m_Bary(new std::vector<boost::array<double,9> > )
-    , average_spacing(CGAL::compute_average_spacing(first, beyond, point_pmap, 6))
+    , average_spacing(CGAL::compute_average_spacing<CGAL::Sequential_tag>
+                      (CGAL::make_range(first, beyond), 6,
+                       CGAL::parameters::point_map(point_pmap)))
   {
     forward_constructor(first, beyond, point_pmap, normal_pmap, Poisson_visitor());
   }
@@ -318,7 +325,8 @@ public:
     NormalPMap normal_pmap, ///< property map: `value_type of InputIterator` -> `Vector` (the *oriented* normal of an input point).
     Visitor visitor)
     : m_tr(new Triangulation), m_Bary(new std::vector<boost::array<double,9> > )
-    , average_spacing(CGAL::compute_average_spacing(first, beyond, point_pmap, 6))
+    , average_spacing(CGAL::compute_average_spacing<CGAL::Sequential_tag>(CGAL::make_range(first, beyond), 6,
+                                                                          CGAL::parameters::point_map(point_pmap)))
   {
     forward_constructor(first, beyond, point_pmap, normal_pmap, visitor);
   }
@@ -332,19 +340,15 @@ public:
     InputIterator beyond, ///< past-the-end iterator over the input points.
     NormalPMap normal_pmap, ///< property map: `value_type of InputIterator` -> `Vector` (the *oriented* normal of an input point).
     typename boost::enable_if<
-      boost::is_convertible<typename InputIterator::value_type, Point>
+      boost::is_convertible<typename std::iterator_traits<InputIterator>::value_type, Point>
     >::type* = 0
   )
   : m_tr(new Triangulation), m_Bary(new std::vector<boost::array<double,9> > )
-  , average_spacing(CGAL::compute_average_spacing(first, beyond, 6))
+  , average_spacing(CGAL::compute_average_spacing<CGAL::Sequential_tag>(CGAL::make_range(first, beyond), 6))
   {
     forward_constructor(first, beyond, 
-#ifdef CGAL_USE_PROPERTY_MAPS_API_V1
-      make_dereference_property_map(first),
-#else
       make_identity_property_map(
       typename std::iterator_traits<InputIterator>::value_type()),
-#endif
       normal_pmap, Poisson_visitor());
     CGAL::Timer task_timer; task_timer.start();
   }
@@ -437,7 +441,7 @@ public:
                                                      m_tr->input_points_begin()),
                                 Some_points_iterator(m_tr->input_points_end(),
                                                      skip),
-                                Normal_of_point_with_normal_pmap<Geom_traits>() );
+                                Normal_of_point_with_normal_map<Geom_traits>() );
       coarse_poisson_function.compute_implicit_function(solver, Poisson_visitor(),
                                                         0.);
       internal::Poisson::Constant_sizing_field<Triangulation> 
@@ -531,12 +535,14 @@ public:
   }
 
   /// \cond SKIP_IN_MANUAL
+#ifdef CGAL_EIGEN3_ENABLED
   // This variant provides the default sparse linear traits class = Eigen_solver_traits.
   bool compute_implicit_function(bool smoother_hole_filling = false)
   {
     typedef Eigen_solver_traits<Eigen::ConjugateGradient<Eigen_sparse_symmetric_matrix<double>::EigenType> > Solver;
     return compute_implicit_function<Solver>(Solver(), smoother_hole_filling);
   }
+#endif
 
   boost::tuple<FT, Cell_handle, bool> special_func(const Point& p) const
   {
@@ -791,9 +797,19 @@ private:
     // median value set to 0.0
     shift_f(-contouring_value);
 
-    // check value on convex hull (should be positive)
-    Vertex_handle v = any_vertex_on_convex_hull();
-    if(v->f() < 0.0)
+     // Check value on convex hull (should be positive): if more than
+    // half the vertices of the convex hull are negative, we flip the
+    // sign (this is particularly useful if the surface is open, then
+    // it is closed using the smallest part of the sphere).
+    std::vector<Vertex_handle> convex_hull;
+    m_tr->adjacent_vertices (m_tr->infinite_vertex (),
+			     std::back_inserter (convex_hull));
+    unsigned int nb_negative = 0;
+    for (std::size_t i = 0; i < convex_hull.size (); ++ i)
+      if (convex_hull[i]->f() < 0.0)
+        ++ nb_negative;
+    
+    if(nb_negative > convex_hull.size () / 2)
       flip_f();
 
     // Update m_sink
@@ -1207,4 +1223,6 @@ private:
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_POISSON_RECONSTRUCTION_FUNCTION_H
