@@ -16,8 +16,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Polygon/include/CGAL/Polygon_2/Polygon_2_simplicity.h $
+// $Id: Polygon_2_simplicity.h e5118d1 2018-02-23T15:34:15+00:00 Andreas Fabri
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Geert-Jan Giezeman <geert@cs.uu.nl>
@@ -25,6 +26,8 @@
 #ifndef CGAL_POLYGON_2_SIMPLICITY_H
 #define CGAL_POLYGON_2_SIMPLICITY_H
 
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/enum.h>
 #include <CGAL/Polygon_2/polygon_assertions.h>
 #include <set>
@@ -98,10 +101,10 @@ template <class VertexData>
 class Less_segments {
     typedef VertexData         Vertex_data;
     Vertex_data *m_vertex_data;
-    bool less_than_in_tree(Vertex_index i, Vertex_index j);
+    bool less_than_in_tree(Vertex_index i, Vertex_index j) const;
   public:
     Less_segments(Vertex_data *vertex_data) : m_vertex_data(vertex_data) {}
-    bool operator()(Vertex_index i, Vertex_index j);
+    bool operator()(Vertex_index i, Vertex_index j) const;
 };
 
 // The data in Edge_data is attached to an edge when it is (about to be)
@@ -196,7 +199,7 @@ class Less_vertex_data {
 public:
     Less_vertex_data(VertexData *vd)
     : m_vertex_data(vd) {}
-    bool operator()(Vertex_index i, Vertex_index j);
+    bool operator()(Vertex_index i, Vertex_index j) const;
 };
 
 } // end of namespace i_polygon
@@ -206,7 +209,7 @@ public:
 namespace i_polygon {
 template <class VertexData>
 bool Less_segments<VertexData>::
-operator()(Vertex_index i, Vertex_index j)
+operator()(Vertex_index i, Vertex_index j) const
 {
     if (m_vertex_data->edges[j.as_int()].is_in_tree) {
         return less_than_in_tree(i,j);
@@ -217,7 +220,7 @@ operator()(Vertex_index i, Vertex_index j)
 
 template <class VertexData>
 bool Less_segments<VertexData>::
-less_than_in_tree(Vertex_index new_edge, Vertex_index tree_edge)
+less_than_in_tree(Vertex_index new_edge, Vertex_index tree_edge) const
 {
     CGAL_polygon_precondition(
        m_vertex_data->edges[tree_edge.as_int()].is_in_tree);
@@ -241,7 +244,7 @@ less_than_in_tree(Vertex_index new_edge, Vertex_index tree_edge)
 
 template <class VertexData>
 bool Less_vertex_data<VertexData>::
-operator()(Vertex_index i, Vertex_index j)
+operator()(Vertex_index i, Vertex_index j) const
 {
     return m_vertex_data->less_xy_2(
             m_vertex_data->point(i), m_vertex_data->point(j));
@@ -500,4 +503,6 @@ bool is_simple_polygon(Iterator points_begin, Iterator points_end,
 
 } // end of namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif
