@@ -13,8 +13,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Mesh_3/include/CGAL/Polyhedral_mesh_domain_3.h $
+// $Id: Polyhedral_mesh_domain_3.h 27b6e03 2018-06-07T10:18:02+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Stéphane Tayeb
@@ -27,34 +28,50 @@
 #ifndef CGAL_POLYHEDRAL_MESH_DOMAIN_3_H
 #define CGAL_POLYHEDRAL_MESH_DOMAIN_3_H
 
-#include <CGAL/internal/Operations_on_polyhedra/Point_inside_vertical_ray_cast.h>
+#include <CGAL/license/Mesh_3.h>
+
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/Mesh_3/Robust_intersection_traits_3.h>
-#include <CGAL/Mesh_3/Triangle_accessor_primitive.h>
-#include <CGAL/Triangle_accessor_3.h>
-#include <CGAL/AABB_tree.h>
-#include <CGAL/AABB_traits.h>
-#include <sstream>
-
-#include <CGAL/Random.h>
-#include <CGAL/point_generators_3.h>
-#include <CGAL/Mesh_3/Creator_weighted_point_3.h>
 #include <CGAL/Mesh_3/Profile_counter.h>
 
+#include <CGAL/AABB_tree.h>
+#include <CGAL/AABB_face_graph_triangle_primitive.h>
+#include <CGAL/boost/graph/helpers.h>
+#include <CGAL/boost/graph/properties.h>
+#include <CGAL/Default.h>
+#include <CGAL/point_generators_3.h>
+#include <CGAL/Random.h>
+#include <CGAL/Side_of_triangle_mesh.h>
+#include <CGAL/tuple.h>
+
 #include <boost/optional.hpp>
 #include <boost/none.hpp>
 #include <boost/utility/enable_if.hpp>
 #include <boost/mpl/vector.hpp>
 #include <boost/mpl/contains.hpp>
-#include <CGAL/tuple.h>
+#include <boost/mpl/or.hpp>
+#include <boost/type_traits/is_same.hpp>
 #include <boost/format.hpp>
 #include <boost/variant.hpp>
 #include <boost/math/special_functions/round.hpp>
 
+#include <iostream>
+#include <map>
+#include <sstream>
+#include <string>
+#include <utility>
+
 #ifdef CGAL_LINKED_WITH_TBB
 # include <tbb/enumerable_thread_specific.h>
 #endif
 
+// To handle I/O for Surface_patch_index if that is a pair of `int` (the
+// default)
+#include <CGAL/internal/Mesh_3/Handle_IO_for_pair_of_int.h>
+
+#include <CGAL/internal/Mesh_3/indices_management.h>
+
 namespace CGAL {
 
 namespace Mesh_3 {
@@ -69,52 +86,6 @@ max_length(const Bbox_3& b)
                     (std::max)(b.ymax()-b.ymin(),b.zmax()-b.zmin()) );
 }
 
-
-// -----------------------------------
-// Surface_patch_index_generator
-// To use patch_id enclosed in AABB_primitives or not
-// -----------------------------------
-template < typename Subdomain_index, typename Polyhedron, typename Tag >
-struct Surface_patch_index_generator
-{
-  typedef std::pair<Subdomain_index,Subdomain_index>  Surface_patch_index;
-  typedef Surface_patch_index                         type;
-
-  template < typename Primitive_id >
-  Surface_patch_index operator()(const Primitive_id&)
-  { return Surface_patch_index(0,1); }
-};
-
-template < typename Subdomain_index, typename Polyhedron >
-struct Surface_patch_index_generator<Subdomain_index, Polyhedron, CGAL::Tag_true>
-{
-  typedef typename Polyhedron::Face::Patch_id Surface_patch_index;
-  typedef Surface_patch_index   type;
-
-  template < typename Primitive_id >
-  Surface_patch_index operator()(const Primitive_id& primitive_id)
-  { return primitive_id->patch_id(); }
-};
-
-
-// -----------------------------------
-// Index_generator
-// Don't use boost::variant if types are the same type
-// -----------------------------------
-template < typename Subdomain_index, typename Surface_patch_index >
-struct Index_generator
-{
-  typedef boost::variant<Subdomain_index,Surface_patch_index> Index;
-  typedef Index                                         type;
-};
-
-template < typename T >
-struct Index_generator<T, T>
-{
-  typedef T       Index;
-  typedef Index   type;
-};
-
 // -----------------------------------
 // Geometric traits generator
 // -----------------------------------
@@ -141,26 +112,68 @@ struct IGT_generator<Gt,CGAL::Tag_false>
 };
 
 }  // end namespace details
-
 }  // end namespace Mesh_3
 
+namespace Mesh_3 {
+namespace internal {
+
+template <typename Polyhedron_type,
+          bool = CGAL::graph_has_property<Polyhedron_type,
+                                           CGAL::face_index_t>::value>
+class Get_face_index_pmap {
+public:
+  typedef typename boost::property_map<Polyhedron_type,
+                                       CGAL::face_index_t>::const_type Pmap;
+  Get_face_index_pmap(const Polyhedron_type&) {}
+  Pmap operator()(const Polyhedron_type& polyhedron) {
+    return get(CGAL::face_index, polyhedron);
+  }
+};
+
+template <typename Polyhedron_type>
+class Get_face_index_pmap<Polyhedron_type, false> {
+  typedef typename boost::graph_traits<Polyhedron_type>::face_descriptor
+                                                              face_descriptor;
+  typedef std::map<face_descriptor, int> Map;
+public:
+  Get_face_index_pmap(const Polyhedron_type& polyhedron) {
+    int id = 0;
+    BOOST_FOREACH(face_descriptor f, faces(polyhedron))
+    {
+      face_ids[f] = id++;
+    }
+  }
+  typedef boost::associative_property_map<Map> Pmap;
+
+  Pmap operator()(const Polyhedron_type&) {
+    return Pmap(face_ids);
+  }
+private:
+  Map face_ids;
+};
+
+} // end namespace internal
+} // end namespace Mesh_3
 
 /**
  * @class Polyhedral_mesh_domain_3
  *
  *
  */
-template<class Polyhedron,
+template<class Polyhedron,/*FaceGraph*/
          class IGT_,
-         class TriangleAccessor=Triangle_accessor_3<Polyhedron,IGT_>,
-         class Use_patch_id_tag=Tag_false,
+         class = CGAL::Default,
+         class Patch_id_ = void,
          class Use_exact_intersection_construction_tag = CGAL::Tag_true>
 class Polyhedral_mesh_domain_3
 {
+public:
   typedef typename Mesh_3::details::IGT_generator<
     IGT_,Use_exact_intersection_construction_tag>::type IGT;
 
-public:
+
+  typedef Patch_id_ Patch_id;
+
   /// Geometric object types
   typedef typename IGT::Point_3    Point_3;
   typedef typename IGT::Segment_3  Segment_3;
@@ -175,13 +188,17 @@ public:
   /// Type of indexes for cells of the input complex
   typedef int Subdomain_index;
   typedef boost::optional<Subdomain_index> Subdomain;
+
   /// Type of indexes for surface patch of the input complex
-  typedef typename Mesh_3::details::Surface_patch_index_generator<
-    Subdomain_index,Polyhedron,Use_patch_id_tag>::type    Surface_patch_index;
+  typedef typename boost::property_map<Polyhedron,
+                                       face_patch_id_t<Patch_id>
+                                       >::type            Face_patch_id_pmap;
+  typedef typename boost::property_traits<
+    Face_patch_id_pmap>::value_type                       Surface_patch_index;
   typedef boost::optional<Surface_patch_index>            Surface_patch;
   /// Type of indexes to characterize the lowest dimensional face of the input
   /// complex on which a vertex lie
-  typedef typename Mesh_3::details::Index_generator<
+  typedef typename Mesh_3::internal::Index_generator<
     Subdomain_index, Surface_patch_index>::type           Index;
 
   typedef CGAL::cpp11::tuple<Point_3,Index,int> Intersection;
@@ -192,26 +209,123 @@ public:
   // Kernel_traits compatibility
   typedef IGT R;
 
-private:
-  typedef Mesh_3::Triangle_accessor_primitive<
-    TriangleAccessor, IGT>                              AABB_primitive;
-  typedef class AABB_traits<IGT,AABB_primitive>         AABB_traits;
-  typedef class AABB_tree<AABB_traits>                  AABB_tree_;
-private:
-  typedef typename AABB_tree_::Primitive_id              AABB_primitive_id;
-  typedef typename AABB_tree_::Primitive Primitive;
-  typedef typename AABB_traits::Bounding_box            Bounding_box;
+  BOOST_MPL_HAS_XXX_TRAIT_DEF(HalfedgeDS)
+
+  template <typename P>
+  struct Primitive {
+    typedef typename boost::graph_traits<P>::face_descriptor face_descriptor_t;
+
+    face_descriptor_t face_descriptor;
+    const P* graph;
+
+    typedef Triangle_from_face_descriptor_map<P>   Triangle_pmap;
+    typedef One_point_from_face_descriptor_map<P>  Point_pmap;
+
+    typedef typename Triangle_pmap::value_type              Datum;
+    typedef typename Point_pmap::value_type                 Point;
+    typedef Primitive                                       Id;
+
+    Primitive() : face_descriptor(), graph() {}
+
+    template <typename IndexIterator>
+    Primitive(IndexIterator it, const P& polyhedron)
+      : face_descriptor(*it), graph(&polyhedron)
+    {}
+
+    bool operator==(const Primitive& other) const {
+      return other.face_descriptor == this->face_descriptor
+        && other.graph == this->graph;
+    }
+
+    Datum datum() const {
+      return get(Triangle_pmap(graph), face_descriptor);
+    }
+
+    Point reference_point() const {
+      return get(Point_pmap(graph), face_descriptor);
+    }
+
+    Id id() const {
+      return *this;
+    }
+  }; // Primitive template, for face graphs
+
+  template <typename P, bool is_a_CGAL_Polyhedron_3 = has_HalfedgeDS<P>::value>
+  struct Primitive_type {
+    typedef Primitive<P> type;
+
+    static
+    typename IGT_::Triangle_3 datum(const typename type::Id primitive_id) {
+      CGAL::Triangle_from_face_descriptor_map<P> pmap(primitive_id.graph);
+      return get(pmap, primitive_id.face_descriptor);
+    }
+
+    static Surface_patch_index get_index(const typename type::Id primitive_id) {
+      return get(get(face_patch_id_t<Patch_id>(),
+                     *primitive_id.graph),
+                 primitive_id.face_descriptor);
+    }
+  }; // Primitive_type (for non-Polyhedron_3)
+
+  template <typename P> struct Primitive_type<P, true> {
+    typedef AABB_face_graph_triangle_primitive<P > type;
+
+    static
+    typename IGT_::Triangle_3 datum(const typename type::Id face_handle) {
+      typedef typename IGT_::Point_3 Point;
+      const Point& a = face_handle->halfedge()->vertex()->point();
+      const Point& b = face_handle->halfedge()->next()->vertex()->point();
+      const Point& c = face_handle->halfedge()->next()->next()->vertex()->point();
+      return typename IGT_::Triangle_3(a,b,c);
+    }
+
+    static Surface_patch_index get_index(const typename type::Id face_handle,
+                                         Tag_false)
+    {
+      typename boost::property_map<P, face_patch_id_t<Patch_id> >::type pmap;
+      return get(pmap, face_handle);
+    }
+
+    static Surface_patch_index get_index(const typename type::Id,
+                                         Tag_true)
+    {
+      return Surface_patch_index(0,1);
+    }
+
+    static Surface_patch_index get_index(const typename type::Id face_handle)
+    {
+      namespace m = boost::mpl;
+      return get_index(face_handle,
+                       Boolean_tag<m::or_<boost::is_same<Patch_id, void>,
+                                          boost::is_same<Patch_id, Tag_false>
+                       >::value>());
+    }
+  }; // Primitive_type specialized for CGAL::Polyehdron_3
+
+public:
+  typedef typename Primitive_type<Polyhedron>::type       Ins_fctor_primitive;
+  typedef CGAL::AABB_traits<IGT, Ins_fctor_primitive>     Ins_fctor_traits;
+  typedef CGAL::AABB_tree<Ins_fctor_traits>               Ins_fctor_AABB_tree;
+
+  typedef Side_of_triangle_mesh<Polyhedron,
+                                IGT,
+                                Default,
+                                Ins_fctor_AABB_tree>      Inside_functor;
+  typedef typename Inside_functor::AABB_tree              AABB_tree_;
+  BOOST_STATIC_ASSERT((boost::is_same<AABB_tree_, Ins_fctor_AABB_tree>::value));
+  typedef typename AABB_tree_::AABB_traits                AABB_traits;
+  typedef typename AABB_tree_::Primitive                  AABB_primitive;
+  typedef typename AABB_tree_::Primitive_id               AABB_primitive_id;
+  typedef typename AABB_traits::Bounding_box              Bounding_box;
 
 public:
 
   /// Default constructor
-  Polyhedral_mesh_domain_3()
+  Polyhedral_mesh_domain_3(CGAL::Random* p_rng = NULL)
     : tree_()
     , bounding_tree_(&tree_)
-    , p_rng_(NULL)
-    , delete_rng_(true)
+    , p_rng_(p_rng)
   {
-    p_rng_ = new CGAL::Random(0);
   }
 
   /**
@@ -220,42 +334,33 @@ public:
    */
   Polyhedral_mesh_domain_3(const Polyhedron& p,
                            CGAL::Random* p_rng = NULL)
-    : tree_(TriangleAccessor().triangles_begin(p),
-            TriangleAccessor().triangles_end(p))
+    : tree_()
     , bounding_tree_(&tree_) // the bounding tree is tree_
     , p_rng_(p_rng)
-    , delete_rng_(false)
   {
-    if(!p.is_pure_triangle()) {
+    this->add_primitives(p);
+    if(! is_triangle_mesh(p)) {
       std::cerr << "Your input polyhedron must be triangulated!\n";
       CGAL_error_msg("Your input polyhedron must be triangulated!");
     }
-    if(!p_rng_)
-    {
-      p_rng_ = new CGAL::Random(0);
-      delete_rng_ = true;
-    }
+    this->build();
   }
 
   Polyhedral_mesh_domain_3(const Polyhedron& p,
                            const Polyhedron& bounding_polyhedron,
                            CGAL::Random* p_rng = NULL)
-    : tree_(TriangleAccessor().triangles_begin(p),
-            TriangleAccessor().triangles_end(p))
-    , bounding_tree_(new AABB_tree_(TriangleAccessor().triangles_begin(bounding_polyhedron),
-                                    TriangleAccessor().triangles_end(bounding_polyhedron)))
+    : tree_()
+    , bounding_tree_(new AABB_tree_)
     , p_rng_(p_rng)
-    , delete_rng_(false)
   {
-    tree_.insert(TriangleAccessor().triangles_begin(bounding_polyhedron),
-                 TriangleAccessor().triangles_end(bounding_polyhedron));
-    tree_.build();
-    bounding_tree_->build();
-    if(!p_rng_)
-    {
-      p_rng_ = new CGAL::Random(0);
-      delete_rng_ = true;
+    this->add_primitives(p);
+    this->add_primitives(bounding_polyhedron);
+    if(!bounding_polyhedron.empty()) {
+      this->add_primitives_to_bounding_tree(bounding_polyhedron);
+    } else {
+      this->set_surface_only();
     }
+    this->build();
   }
 
   /**
@@ -279,27 +384,16 @@ public:
   {
     if(begin != end) {
       for(; begin != end; ++begin) {
-        tree_.insert(TriangleAccessor().triangles_begin(**begin),
-                     TriangleAccessor().triangles_end(**begin));
+        this->add_primitives(**begin);
       }
-      tree_.insert(TriangleAccessor().triangles_begin(bounding_polyhedron),
-                   TriangleAccessor().triangles_end(bounding_polyhedron));
-      tree_.build();
-      bounding_tree_ =
-        new AABB_tree_(TriangleAccessor().triangles_begin(bounding_polyhedron),
-                       TriangleAccessor().triangles_end(bounding_polyhedron));
-      bounding_tree_->build();
+      this->add_primitives(bounding_polyhedron);
     }
-    else {
-      tree_.rebuild(TriangleAccessor().triangles_begin(bounding_polyhedron),
-                    TriangleAccessor().triangles_end(bounding_polyhedron));
-      bounding_tree_ = &tree_;
-    }
-    if(!p_rng_)
-    {
-      p_rng_ = new CGAL::Random(0);
-      delete_rng_ = true;
+    if(!bounding_polyhedron.empty()) {
+      this->add_primitives_to_bounding_tree(bounding_polyhedron);
+    } else {
+      this->set_surface_only();
     }
+    this->build();
   }
 
   /**
@@ -317,21 +411,14 @@ public:
                            InputPolyhedraPtrIterator end,
                            CGAL::Random* p_rng = NULL)
     : p_rng_(p_rng)
-    , delete_rng_(false)
   {
     if(begin != end) {
       for(; begin != end; ++begin) {
-        tree_.insert(TriangleAccessor().triangles_begin(**begin),
-                     TriangleAccessor().triangles_end(**begin));
+        this->add_primitives(**begin);
       }
       tree_.build();
     }
     bounding_tree_ = 0;
-    if(!p_rng_)
-    {
-      p_rng_ = new CGAL::Random(0);
-      delete_rng_ = true;
-    }
   }
 
   /// Destructor
@@ -339,8 +426,10 @@ public:
     if(bounding_tree_ != 0 && bounding_tree_ != &tree_) {
       delete bounding_tree_;
     }
-    if(delete_rng_)
-      delete p_rng_;
+  }
+
+  void set_surface_only() {
+    bounding_tree_ = 0;
   }
 
   /**
@@ -366,6 +455,14 @@ public:
   }
 
 
+  /**
+   * Returns a bounding box of the domain
+   */
+  Bbox_3 bbox() const {
+    return tree_.bbox();
+  }
+
+
   /**
    * Returns true if point~\ccc{p} is in the domain. If \ccc{p} is in the
    *  domain, the parameter index is set to the index of the subdomain
@@ -473,7 +570,8 @@ public:
 #endif // not CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
       {
 #ifndef CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
-        CGAL_precondition(r_domain_.do_intersect_surface_object()(q));
+        CGAL_precondition(r_domain_.do_intersect_surface_object()(q)
+                          != boost::none);
 #endif // NOT CGAL_MESH_3_NO_LONGER_CALLS_DO_INTERSECT_3
 
         intersection = r_domain_.tree_.any_intersection(q);
@@ -598,11 +696,7 @@ public:
   Surface_patch_index make_surface_index(
     const AABB_primitive_id& primitive_id = AABB_primitive_id() ) const
   {
-    Mesh_3::details::Surface_patch_index_generator<Subdomain_index,
-                                                   Polyhedron,
-                                                   Use_patch_id_tag> generator;
-
-    return generator(primitive_id);
+    return Primitive_type<Polyhedron>::get_index(primitive_id);
   }
 
   // Undocumented function, used to implement a sizing field that
@@ -613,13 +707,31 @@ public:
     return tree_;
   }
 
+  const AABB_tree* bounding_aabb_tree_ptr() const {
+    return bounding_tree_;
+  }
+
 protected:
   void add_primitives(const Polyhedron& p)
   {
-    tree_.insert(TriangleAccessor().triangles_begin(p),
-                 TriangleAccessor().triangles_end(p));
+    tree_.insert(faces(p).begin(), faces(p).end(), p);
+  }
 
+  void add_primitives_to_bounding_tree(const Polyhedron& p)
+  {
+    if(bounding_tree_ == &tree_ || bounding_tree_ == 0) {
+      bounding_tree_ = new AABB_tree_;
+    }
+    bounding_tree_->insert(faces(p).begin(), faces(p).end(), p);
+  }
+
+  void build() {
     tree_.build();
+    CGAL_assertion(!tree_.empty());
+    if(bounding_tree_ != &tree_ && bounding_tree_ != 0) {
+      bounding_tree_->build();
+      CGAL_assertion(!bounding_tree_->empty());
+    }
   }
 
 private:
@@ -686,16 +798,7 @@ public:
 
   void set_random_generator(CGAL::Random* p_rng)
   {
-    if(delete_rng_) delete p_rng_;
-    if(!p_rng)
-    {
-      p_rng_ = new CGAL::Random(0);
-      delete_rng_ = true;
-    }
-    else {
-      p_rng_ = p_rng;
-      delete_rng_ = false;
-    }
+    p_rng_ = p_rng;
   }
 
 private:
@@ -726,7 +829,9 @@ Construct_initial_points::operator()(OutputIterator pts,
                         FT( (bbox.ymin() + bbox.ymax()) / 2),
                         FT( (bbox.zmin() + bbox.zmax()) / 2) );
 
-  CGAL::Random& rng = *(r_domain_.p_rng_);
+  CGAL::Random& rng = *(r_domain_.p_rng_ != 0 ?
+                        r_domain_.p_rng_ :
+                        new Random(0));
   Random_points_on_sphere_3<Point_3> random_point(1., rng);
 
   int i = n;
@@ -763,6 +868,7 @@ Construct_initial_points::operator()(OutputIterator pts,
 #ifdef CGAL_MESH_3_VERBOSE
   std::cerr << std::endl;
 #endif
+  if(r_domain_.p_rng_ == 0) delete &rng;
   return pts;
 }
 
@@ -775,17 +881,15 @@ Is_in_domain::operator()(const Point_3& p) const
 {
   if(r_domain_.bounding_tree_ == 0) return Subdomain();
 
-  internal::Point_inside_vertical_ray_cast<IGT_, AABB_tree_> inside_functor;
-  Bounded_side side = inside_functor(p, *(r_domain_.bounding_tree_));
+  Inside_functor inside_functor(*(r_domain_.bounding_tree_));
+  Bounded_side side = inside_functor(p);
 
   if(side == CGAL::ON_UNBOUNDED_SIDE) { return Subdomain(); }
   else { return Subdomain(Subdomain_index(1)); } // case ON_BOUNDARY && ON_BOUNDED_SIDE
 }
 
-
-
-
 }  // end namespace CGAL
 
-
+#include <CGAL/enable_warnings.h>
+  
 #endif // POLYHEDRAL_MESH_TRAITS_3_H_
