@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Polynomial/include/CGAL/Polynomial_type_generator.h $
+// $Id: Polynomial_type_generator.h 18acb2e 2018-01-18T15:05:34+00:00 Andreas Fabri
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Michael Hemmer <hemmer@informatik.uni-mainz.de> 
@@ -24,6 +25,8 @@
 #ifndef CGAL_POLYNOMIAL_TYPE_GENERATOR_H
 #define CGAL_POLYNOMIAL_TYPE_GENERATOR_H
 
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/Polynomial_traits_d.h>
 
 namespace CGAL {
@@ -42,4 +45,6 @@ struct Polynomial_type_generator<T,0>{ typedef T Type; };
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_POLYNOMIAL_GENERATOR_H
