@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/QP_solver/include/CGAL/QP_solver/QP_partial_exact_pricing.h $
+// $Id: QP_partial_exact_pricing.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Sven Schoenherr
@@ -24,6 +25,9 @@
 #ifndef CGAL_QP_PARTIAL_EXACT_PRICING_H
 #define CGAL_QP_PARTIAL_EXACT_PRICING_H
 
+#include <CGAL/license/QP_solver.h>
+
+
 // includes
 #include <CGAL/QP_solver/QP__partial_base.h>
 
@@ -55,7 +59,7 @@ class QP_partial_exact_pricing : public QP__partial_base<Q,ET,Tags> {
 
     // creation
     QP_partial_exact_pricing( bool     randomize = false,
-			       Random&  random    = default_random);
+			       Random&  random    = get_default_random());
 
     // operations
     int  pricing(int& direction );
