@@ -13,8 +13,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/GraphicsView/include/CGAL/Qt/GraphicsViewInput.h $
+// $Id: GraphicsViewInput.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Andreas Fabri <Andreas.Fabri@geometryfactory.com>
@@ -22,8 +23,13 @@
 
 #ifndef CGAL_QT_GRAPHICS_VIEW_INPUT_H
 #define CGAL_QT_GRAPHICS_VIEW_INPUT_H
-#include <CGAL/export/Qt4.h>
-#include <CGAL/auto_link/Qt4.h>
+
+#include <CGAL/license/GraphicsView.h>
+
+
+#include <CGAL/export/Qt.h>
+#include <CGAL/auto_link/Qt.h>
+
 #ifndef Q_MOC_RUN
 #  include <CGAL/Object.h>
 #endif
@@ -31,7 +37,7 @@
 
 namespace CGAL {
 namespace Qt {
-class CGAL_QT4_EXPORT GraphicsViewInput  : public QObject
+class CGAL_QT_EXPORT GraphicsViewInput  : public QObject
 {
   Q_OBJECT
 
@@ -40,11 +46,11 @@ public:
     : QObject(parent)
   {}
 
-signals:
+Q_SIGNALS:
   void generate(CGAL::Object o);
   void modelChanged();
 
-public slots:
+public Q_SLOTS:
 
   virtual void processInput(CGAL::Object /*o*/) {}
 
