@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Segment_Delaunay_graph_2/include/CGAL/Segment_Delaunay_graph_2/Segment_Delaunay_graph_2_impl.h $
+// $Id: Segment_Delaunay_graph_2_impl.h 5133321 2019-06-18T13:18:51+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Menelaos Karavelas <mkaravel@iacm.forth.gr>
@@ -139,11 +140,14 @@ insert_third(const Site_2& t, const Storage_site_2& ss)
   Site_2 s2 = f->vertex(1)->site();
   Site_2 s3 = f->vertex(2)->site();
 
-  Orientation o =
-    geom_traits().orientation_2_object()(s1, s2, s3);
+  Sign s12i3 = geom_traits().vertex_conflict_2_object()(s1, s2, s3);
+  Sign s21i3 = geom_traits().vertex_conflict_2_object()(s2, s1, s3);
 
-  if ( o != COLLINEAR ) {
-    if ( o == RIGHT_TURN ) {
+  CGAL_assertion(s12i3 != ZERO);
+  CGAL_assertion(s21i3 != ZERO);
+
+  if ( s12i3 != s21i3 ) {
+    if ( s21i3 == NEGATIVE ) {
       f->reorient();
       for (int i = 0; i < 3; i++) {
 	f->neighbor(i)->reorient();
@@ -335,7 +339,8 @@ insert_point(const Storage_site_2& ss, const Site_2& t,
     if ( at_res == AT2::INTERIOR ) {
       CGAL_assertion( t.is_input() );
 
-      Vertex_triple vt = insert_exact_point_on_segment(ss, t, vnearest);
+      Vertex_triple vt = (this->*insert_exact_point_on_segment_ptr)(
+          ss, t, vnearest);
       return vt.first;
     } else {
       // the point to be inserted does not belong to the interior of a
@@ -460,9 +465,9 @@ insert_point2(const Storage_site_2& ss, const Site_2& t,
   // REGION AND EXPANDS THE CONFLICT REGION.
   initialize_conflict_region(start_f, l);
 #ifdef CGAL_SDG_NO_FACE_MAP
-  expand_conflict_region(start_f, t, ss, l, vcross);
+  expand_conflict_region(start_f, t, l, vcross);
 #else
-  expand_conflict_region(start_f, t, ss, l, fm, sign_map, vcross);
+  expand_conflict_region(start_f, t, l, fm, sign_map, vcross);
 #endif
 
   CGAL_assertion( !vcross.first );
@@ -499,7 +504,7 @@ find_faces_to_split(const Vertex_handle& v, const Site_2& t) const
       if ( is_infinite(fc) ) { n_inf++; }
       fc++;
     } while ( fc != fc_start );
-    CGAL_assertion( n_inf == 0 || n_inf == 2 || n_inf == 4 );
+    CGAL_assertion( n_inf % 2 == 0 );
   }
 #endif
 
@@ -524,13 +529,13 @@ find_faces_to_split(const Vertex_handle& v, const Site_2& t) const
 	CGAL_assertion(  !is_infinite( ff1->vertex(ccw_v) )  );
 	CGAL_assertion( ff1->vertex(ccw_v)->site().is_point() );
 	sv_ep = ff1->vertex(ccw_v)->site();
+        os1 = oriented_side(v->site(), sv_ep, sitev_supp, t);
       } else {
 	CGAL_assertion(  !is_infinite( ff1->vertex( cw_v) )  );
 	CGAL_assertion( ff1->vertex( cw_v)->site().is_point() );
 	sv_ep = ff1->vertex( cw_v)->site();
+        os1 = oriented_side(sv_ep, v->site(), sitev_supp, t);
       }
-
-      os1 = oriented_side(sv_ep, sitev_supp, t);
     } else {
       os1 = oriented_side(fc1->vertex(0)->site(),
 			  fc1->vertex(1)->site(),
@@ -548,13 +553,13 @@ find_faces_to_split(const Vertex_handle& v, const Site_2& t) const
 	CGAL_assertion(  !is_infinite( ff2->vertex(ccw_v) )  );
 	CGAL_assertion( ff2->vertex(ccw_v)->site().is_point() );
 	sv_ep = ff2->vertex(ccw_v)->site();
+        os2 = oriented_side(v->site(), sv_ep, sitev_supp, t);
       } else {
 	CGAL_assertion(  !is_infinite( ff2->vertex( cw_v) )  );
 	CGAL_assertion( ff2->vertex( cw_v)->site().is_point() );
 	sv_ep = ff2->vertex( cw_v)->site();
+        os2 = oriented_side(sv_ep, v->site(), sitev_supp, t);
       }
-
-      os2 = oriented_side(sv_ep, sitev_supp, t);
     } else {
       os2 = oriented_side(fc2->vertex(0)->site(),
 			  fc2->vertex(1)->site(),
@@ -569,7 +574,7 @@ find_faces_to_split(const Vertex_handle& v, const Site_2& t) const
     }
 
     if ( !found_f2 &&
-	 os1 == ON_POSITIVE_SIDE && os2 != ON_POSITIVE_SIDE ) {
+         os1 != ON_NEGATIVE_SIDE && os2 == ON_NEGATIVE_SIDE ) {
       f2 = ff2;
       found_f2 = true;
     }
@@ -841,9 +846,9 @@ insert_segment_interior(const Site_2& t, const Storage_site_2& ss,
   // REGION AND EXPANDS THE CONFLICT REGION.
   initialize_conflict_region(start_f, l);
 #ifdef CGAL_SDG_NO_FACE_MAP
-  expand_conflict_region(start_f, t, ss, l, vcross);
+  expand_conflict_region(start_f, t, l, vcross);
 #else
-  expand_conflict_region(start_f, t, ss, l, fm, sign_map, vcross);
+  expand_conflict_region(start_f, t, l, fm, sign_map, vcross);
 #endif
 
   CGAL_assertion( vcross.third == AT2::DISJOINT ||
@@ -897,17 +902,7 @@ insert_intersecting_segment_with_tag(const Storage_site_2& /* ss */,
 				     const Site_2& /* t */, Vertex_handle /* v */,
 				     Tag_false)
 {
-#if defined(__POWERPC__) && \
-  defined(__GNUC__) && (__GNUC__ == 3) && (__GNUC_MINOR__ == 4)
-  // hack to avoid nasty warning for G++ 3.4 on Darwin
-  static int i;
-#else
-  static int i = 0;
-#endif
-  if ( i == 0 ) {
-    i = 1;
-    print_error_message();
-  }
+  print_error_message();
   return Vertex_handle();
 }
 
@@ -928,7 +923,7 @@ insert_intersecting_segment_with_tag(const Storage_site_2& ss,
     return v;
   }
 
-  Vertex_triple vt = insert_point_on_segment(ss, t, v, tag);
+  Vertex_triple vt = (this->*insert_point_on_segment_ptr)(ss, t, v, tag);
 
   Vertex_handle vsx = vt.first;
   
@@ -963,152 +958,190 @@ initialize_conflict_region(const Face_handle& f, List& l)
 
 
 template<class Gt, class ST, class D_S, class LTag>
-void
+bool
 Segment_Delaunay_graph_2<Gt,ST,D_S,LTag>::
-expand_conflict_region(const Face_handle& f, const Site_2& t,
-		       const Storage_site_2& ss,
-#ifdef CGAL_SDG_NO_FACE_MAP
-		       List& l,
-#else
-		       List& l, Face_map& fm,
-		       std::map<Face_handle,Sign>& sign_map,
+check_unregistered_face(const Face_handle& n,
+                        const Site_2& t,
+                        List& l,
+#ifndef CGAL_SDG_NO_FACE_MAP
+                        Face_map& fm,
 #endif
-		       Triple<bool,Vertex_handle,Arrangement_type>& vcross)
+                        Triple<bool, Vertex_handle, Arrangement_type>& vcross)
 {
+  for (int j = 0; j < 3; ++j)
+  {
+    Vertex_handle vf = n->vertex(j);
+
+    if ( is_infinite(vf) )
+      continue;
+
+    Arrangement_type at_res = arrangement_type(t, vf);
+
+    CGAL_assertion( vcross.third == AT2::DISJOINT ||
+                    vcross.third == AT2::CROSSING ||
+                    vcross.third == AT2::INTERIOR );
+
+    if ( vf->is_segment() )
+    {
+      CGAL_assertion( at_res != AT2::IDENTICAL );
+      CGAL_assertion( at_res != AT2::TOUCH_11_INTERIOR_1 );
+      CGAL_assertion( at_res != AT2::TOUCH_12_INTERIOR_1 );
+
+      if ( at_res == AT2::CROSSING )
+      {
+        vcross.first = true;
+        vcross.second = vf;
+        vcross.third = AT2::CROSSING;
+        l.clear();
 #ifdef CGAL_SDG_NO_FACE_MAP
-  if ( f->tds_data().is_in_conflict() ) { return; }
+        fhc_.clear();
 #else
-  if ( fm.find(f) != fm.end() ) { return; }
+        fm.clear();
 #endif
-
-  // this is done to stop the recursion when intersecting segments
-  // are found
-  if ( vcross.first ) { return; }
-
-  // setting fm[f] to true means that the face has been reached and
-  // that the face is available for recycling. If we do not want the
-  // face to be available for recycling we must set this flag to
-  // false.
-#ifdef CGAL_SDG_NO_FACE_MAP
-  f->tds_data().mark_in_conflict();
-  fhc_.push_back(f);
-#else
-  fm[f] = true;
-#endif
-
-  //  CGAL_assertion( fm.find(f) != fm.end() );
-
-  for (int i = 0; i < 3; i++) {
-    Face_handle n = f->neighbor(i);
-
-#ifdef CGAL_SDG_NO_FACE_MAP
-    bool face_registered = n->tds_data().is_in_conflict();
-#else
-    bool face_registered = (fm.find(n) != fm.end());
-#endif
-
-    if ( !face_registered ) {
-      for (int j = 0; j < 3; j++) {
-	Vertex_handle vf = n->vertex(j);
-
-	if ( is_infinite(vf) ) { continue; }
-
-	Arrangement_type at_res = arrangement_type(t, vf);
-
-	CGAL_assertion( vcross.third == AT2::DISJOINT ||
-			vcross.third == AT2::CROSSING ||
-			vcross.third == AT2::INTERIOR );
-
-	if ( vf->is_segment() ) {
-	  CGAL_assertion( at_res != AT2::IDENTICAL );
-	  CGAL_assertion( at_res != AT2::TOUCH_11_INTERIOR_1 );
-	  CGAL_assertion( at_res != AT2::TOUCH_12_INTERIOR_1 );
-
-	  if ( at_res == AT2::CROSSING ) {
-	    vcross.first = true;
-	    vcross.second = vf;
-	    vcross.third = AT2::CROSSING;
-	    l.clear();
-#ifdef CGAL_SDG_NO_FACE_MAP
-	    fhc_.clear();
-#else
-	    fm.clear();
-#endif
-	    return;
-	  } else {
-	    CGAL_assertion ( at_res == AT2::DISJOINT ||
-			     at_res == AT2::TOUCH_1 ||
-			     at_res == AT2::TOUCH_2 ||
-			     at_res == AT2::TOUCH_11 ||
-			     at_res == AT2::TOUCH_12 ||
-			     at_res == AT2::TOUCH_21 ||
-			     at_res == AT2::TOUCH_22 );
-	    // we do nothing in these cases
-	  }
-	} else {
-	  CGAL_assertion( vf->is_point() );
-	  if ( at_res == AT2::INTERIOR ) {
-	    vcross.first = true;
-	    vcross.second = vf;
-	    vcross.third = AT2::INTERIOR;
-	    l.clear();
-#ifdef CGAL_SDG_NO_FACE_MAP
-	    fhc_.clear();
-#else
-	    fm.clear();
-#endif
-	    return;
-	  }
-	}
+        return true;
+      }
+      else // at_res != AT2::CROSSING
+      {
+        CGAL_assertion ( at_res == AT2::DISJOINT ||
+                         at_res == AT2::TOUCH_1 ||
+                         at_res == AT2::TOUCH_2 ||
+                         at_res == AT2::TOUCH_11 ||
+                         at_res == AT2::TOUCH_12 ||
+                         at_res == AT2::TOUCH_21 ||
+                         at_res == AT2::TOUCH_22 );
+        // we do nothing in these cases
       }
     }
+    else // ! vf->is_segment()
+    {
+      CGAL_assertion( vf->is_point() );
+      if ( at_res == AT2::INTERIOR )
+      {
+        vcross.first = true;
+        vcross.second = vf;
+        vcross.third = AT2::INTERIOR;
+        l.clear();
+#ifdef CGAL_SDG_NO_FACE_MAP
+        fhc_.clear();
+#else
+        fm.clear();
+#endif
+        return true;
+      }
+    }
+  }
 
-    Sign s = incircle(n, t);
+  return false;
+}
+
+template<class Gt, class ST, class D_S, class LTag>
+void
+Segment_Delaunay_graph_2<Gt,ST,D_S,LTag>::
+expand_conflict_region(const Face_handle& in_f,
+                       const Site_2& t,
+                       List& l,
+#ifndef CGAL_SDG_NO_FACE_MAP
+                       Face_map& fm,
+                       std::map<Face_handle, Sign>& sign_map,
+#endif
+                       Triple<bool, Vertex_handle, Arrangement_type>& vcross)
+{
+  std::stack<Face_handle> face_stack;
+  face_stack.push(in_f);
+
+  while(!face_stack.empty())
+  {
+    // Stop as soon as intersecting segments are found
+    if ( vcross.first )
+      break;
+
+    const Face_handle curr_f = face_stack.top();
+    face_stack.pop();
 
 #ifdef CGAL_SDG_NO_FACE_MAP
-    n->tds_data().set_incircle_sign(s);
-
-    Sign s_f = f->tds_data().incircle_sign();
+    if ( curr_f->tds_data().is_in_conflict() )
+      continue;
 #else
-    sign_map[n] = s;
-
-    Sign s_f = sign_map[f];
+    if ( fm.find(curr_f) != fm.end() )
+      continue;
 #endif
 
-    if ( s == POSITIVE ) { continue; }
-    if ( s != s_f ) { continue; }
+    // setting fm[f] to true means that the face has been reached and
+    // that the face is available for recycling. If we do not want the
+    // face to be available for recycling we must set this flag to false.
+#ifdef CGAL_SDG_NO_FACE_MAP
+    curr_f->tds_data().mark_in_conflict();
+    fhc_.push_back(curr_f);
+#else
+    fm[curr_f] = true;
+#endif
 
-    bool interior_in_conflict = edge_interior(f, i, t, s);
+//    CGAL_assertion( fm.find(curr_f) != fm.end() );
 
-    if ( !interior_in_conflict ) { continue; }
-
-    if ( face_registered ) { continue; }
-
-    Edge e = sym_edge(f, i);
-
-    CGAL_assertion( l.is_in_list(e) );
-    int j = this->_tds.mirror_index(f, i);
-    Edge e_before = sym_edge(n, ccw(j));
-    Edge e_after = sym_edge(n, cw(j));
-    if ( !l.is_in_list(e_before) ) {
-      l.insert_before(e, e_before);
-    }
-    if ( !l.is_in_list(e_after) ) {
-      l.insert_after(e, e_after);
-    }
-    l.remove(e);
+    for (int i = 0; i < 3; ++i)
+    {
+      Face_handle n = curr_f->neighbor(i);
 
 #ifdef CGAL_SDG_NO_FACE_MAP
-    expand_conflict_region(n, t, ss, l, vcross);
+      bool face_registered = n->tds_data().is_in_conflict();
 #else
-    expand_conflict_region(n, t, ss, l, fm, sign_map, vcross);
+      bool face_registered = (fm.find(n) != fm.end());
 #endif
 
-    // this is done to stop the recursion when intersecting segments
-    // are found
-    //    if ( fm.size() == 0 && l.size() == 0 ) { return; }
-    if ( vcross.first ) { return; }
-  } // for-loop
+      if ( !face_registered )
+      {
+#ifdef CGAL_SDG_NO_FACE_MAP
+        if(check_unregistered_face(n, t, l, vcross))
+#else
+        if(check_unregistered_face(n, t, l, fm, vcross))
+#endif
+        {
+          CGAL_assertion(vcross.first);
+          break; // intersecting segments were found
+        }
+      }
+
+      Sign s = incircle(n, t);
+
+#ifdef CGAL_SDG_NO_FACE_MAP
+      n->tds_data().set_incircle_sign(s);
+
+      Sign s_f = curr_f->tds_data().incircle_sign();
+#else
+      sign_map[n] = s;
+
+      Sign s_f = sign_map[curr_f];
+#endif
+
+      if ( s == POSITIVE )
+        continue;
+      if ( s != s_f )
+        continue;
+      if ( face_registered )
+        continue;
+
+      bool interior_in_conflict = edge_interior(curr_f, i, t, s);
+      if ( !interior_in_conflict )
+        continue;
+
+      Edge e = sym_edge(curr_f, i);
+      CGAL_assertion( l.is_in_list(e) );
+
+      int j = this->_tds.mirror_index(curr_f, i);
+      Edge e_before = sym_edge(n, ccw(j));
+      Edge e_after = sym_edge(n, cw(j));
+
+      if ( !l.is_in_list(e_before) )
+        l.insert_before(e, e_before);
+
+      if ( !l.is_in_list(e_after) )
+        l.insert_after(e, e_after);
+
+      l.remove(e);
+
+      face_stack.push(n);
+    } // neighbor for-loop
+  }
 }
 
 
@@ -1568,56 +1601,72 @@ equalize_degrees(const Vertex_handle& v, Self& small_d,
 template<class Gt, class ST, class D_S, class LTag>
 void
 Segment_Delaunay_graph_2<Gt,ST,D_S,LTag>::
-expand_conflict_region_remove(const Face_handle& f, const Site_2& t,
-			      const Storage_site_2& ss,
-			      List& l, Face_map& fm, Sign_map& sign_map)
+expand_conflict_region_remove(const Face_handle& in_f,
+                              const Site_2& t,
+                              List& l,
+                              Face_map& fm,
+                              Sign_map& sign_map)
 {
-  if ( fm.find(f) != fm.end() ) { return; }
+  std::stack<Face_handle> face_stack;
+  face_stack.push(in_f);
 
-  // setting fm[f] to true means that the face has been reached and
-  // that the face is available for recycling. If we do not want the
-  // face to be available for recycling we must set this flag to
-  // false.
-  fm[f] = true;
+  while(!face_stack.empty())
+  {
+    const Face_handle curr_f = face_stack.top();
+    face_stack.pop();
 
-  //  CGAL_assertion( fm.find(f) != fm.end() );
+    if ( fm.find(curr_f) != fm.end() )
+      continue;
 
-  for (int i = 0; i < 3; i++) {
-    Face_handle n = f->neighbor(i);
+    // setting fm[curr_f] to true means that the face has been reached and
+    // that the face is available for recycling. If we do not want the
+    // face to be available for recycling we must set this flag to
+    // false.
+    fm[curr_f] = true;
 
-    bool face_registered = (fm.find(n) != fm.end());
+    //  CGAL_assertion( fm.find(f) != fm.end() );
 
-    Sign s = incircle(n, t);
+    for (int i = 0; i < 3; ++i)
+    {
+      Face_handle n = curr_f->neighbor(i);
 
-    sign_map[n] = s;
+      bool face_registered = (fm.find(n) != fm.end());
 
-    Sign s_f = sign_map[f];
+      Sign s = incircle(n, t);
 
-    if ( s == POSITIVE ) { continue; }
-    if ( s != s_f ) { continue; }
+      sign_map[n] = s;
 
-    bool interior_in_conflict = edge_interior(f, i, t, s);
+      Sign s_f = sign_map[curr_f];
 
-    if ( !interior_in_conflict ) { continue; }
+      if ( s == POSITIVE )
+        continue;
+      if ( s != s_f )
+        continue;
+      if ( face_registered )
+        continue;
 
-    if ( face_registered ) { continue; }
+      bool interior_in_conflict = edge_interior(curr_f, i, t, s);
+      if ( !interior_in_conflict )
+        continue;
 
-    Edge e = sym_edge(f, i);
+      Edge e = sym_edge(curr_f, i);
+      CGAL_assertion( l.is_in_list(e) );
 
-    CGAL_assertion( l.is_in_list(e) );
-    int j = this->_tds.mirror_index(f, i);
-    Edge e_before = sym_edge(n, ccw(j));
-    Edge e_after = sym_edge(n, cw(j));
-    if ( !l.is_in_list(e_before) ) {
-      l.insert_before(e, e_before);
-    }
-    if ( !l.is_in_list(e_after) ) {
-      l.insert_after(e, e_after);
-    }
-    l.remove(e);
+      int j = this->_tds.mirror_index(curr_f, i);
+      Edge e_before = sym_edge(n, ccw(j));
+      Edge e_after = sym_edge(n, cw(j));
 
-    expand_conflict_region_remove(n, t, ss, l, fm, sign_map);
-  } // for-loop
+      if ( !l.is_in_list(e_before) )
+        l.insert_before(e, e_before);
+
+      if ( !l.is_in_list(e_after) )
+        l.insert_after(e, e_after);
+
+      l.remove(e);
+
+      face_stack.push(n);
+    } // neighbor for-loop
+  }
 }
 
 
@@ -1695,7 +1744,7 @@ find_conflict_region_remove(const Vertex_handle& v,
   }
 
   initialize_conflict_region(start_f, l);
-  expand_conflict_region_remove(start_f, t, ss,	l, fm, sign_map);
+  expand_conflict_region_remove(start_f, t, l, fm, sign_map);
 }
 
 
@@ -2564,17 +2613,13 @@ Object
 Segment_Delaunay_graph_2<Gt,ST,D_S,LTag>::
 primal(const Edge e) const
 {
-  typedef typename Gt::Line_2   Line_2;
-  typedef typename Gt::Ray_2    Ray_2;
-
   CGAL_precondition( !is_infinite(e) );
 
   if ( this->dimension() == 1 ) {
     Site_2 p = (e.first)->vertex(cw(e.second))->site();
     Site_2 q = (e.first)->vertex(ccw(e.second))->site();
 
-    Line_2 l = construct_sdg_bisector_2_object()(p,q);
-    return make_object(l);
+    return make_object(construct_sdg_bisector_2_object()(p,q));
   }
 
   // dimension == 2
@@ -2593,8 +2638,7 @@ primal(const Edge e) const
        is_infinite(e.first->neighbor(e.second)) )  {
     Site_2 p = (e.first)->vertex(cw(e.second))->site();
     Site_2 q = (e.first)->vertex(ccw(e.second))->site();
-    Line_2 l = construct_sdg_bisector_2_object()(p,q);
-    return make_object(l);
+    return make_object(construct_sdg_bisector_2_object()(p,q));
   }
 
   // only one of the adjacent faces is infinite
@@ -2618,8 +2662,7 @@ primal(const Edge e) const
   Site_2 q = ee.first->vertex(  cw(ee.second) )->site();
   Site_2 r = ee.first->vertex(     ee.second  )->site();
 
-  Ray_2 ray = construct_sdg_bisector_ray_2_object()(p,q,r);
-  return make_object(ray);
+  return make_object(construct_sdg_bisector_ray_2_object()(p,q,r));
 }
 
 //--------------------------------------------------------------------
@@ -2697,15 +2740,19 @@ void
 Segment_Delaunay_graph_2<Gt,ST,D_S,LTag>::
 print_error_message() const
 {
-  std::cerr << std::endl;
-  std::cerr << "ATTENTION:" << std::endl;
-  std::cerr << "A segment-segment intersection was found."
-	    << std::endl;
-  std::cerr << "The Segment_Delaunay_graph_2 class is not configured"
-	    << " to handle this situation." << std::endl;
-  std::cerr << "Please look at the documentation on how to handle"
-	    << " this behavior." << std::endl;
-  std::cerr << std::endl;
+  CGAL_STATIC_THREAD_LOCAL_VARIABLE(int, once, 0);
+  if(once == 0){
+    ++once;
+    std::cerr << std::endl;
+    std::cerr << "ATTENTION:" << std::endl;
+    std::cerr << "A segment-segment intersection was found."
+              << std::endl;
+    std::cerr << "The Segment_Delaunay_graph_2 class is not configured"
+              << " to handle this situation." << std::endl;
+    std::cerr << "Please look at the documentation on how to handle"
+              << " this behavior." << std::endl;
+    std::cerr << std::endl;
+  }
 }
 
 //--------------------------------------------------------------------
@@ -2781,6 +2828,11 @@ void
 Segment_Delaunay_graph_2<Gt,ST,D_S,LTag>::
 copy(Segment_Delaunay_graph_2& other)
 {
+  // copy the insert_on method pointers
+  insert_point_on_segment_ptr = other.insert_point_on_segment_ptr;
+  insert_exact_point_on_segment_ptr =
+    other.insert_exact_point_on_segment_ptr;
+
   // first copy the point container
   pc_ = other.pc_;
 
