@@ -11,16 +11,25 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_2_impl.h $
+// $Id: Straight_skeleton_builder_2_impl.h 599e201 2018-01-18T14:08:08+00:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
 //
 #ifndef CGAL_STRAIGHT_SKELETON_BUILDER_2_IMPL_H
 #define CGAL_STRAIGHT_SKELETON_BUILDER_2_IMPL_H 1
 
+#include <CGAL/license/Straight_skeleton_2.h>
+
+
 #include <boost/bind.hpp>
 #include <boost/utility.hpp>
+#include <boost/version.hpp>
+#if BOOST_VERSION == 106000
+//ice_not is deprecated in boost 1.60 but used within adjacency_matrix.hpp
+#include <boost/type_traits/detail/ice_not.hpp>
+#endif
 #include <boost/graph/adjacency_matrix.hpp>
 #include <CGAL/Unique_hash_map.h>
 
@@ -535,7 +544,7 @@ void Straight_skeleton_builder_2<Gt,Ss,V>::CreateContourBisectors()
 template<class Gt, class Ss, class V>
 void Straight_skeleton_builder_2<Gt,Ss,V>::InitPhase()
 {
-  mVisitor.on_initialization_started(mSSkel->size_of_vertices());
+  mVisitor.on_initialization_started(static_cast<int>(mSSkel->size_of_vertices()));
   CreateContourBisectors();
   CreateInitialEvents();
   mVisitor.on_initialization_finished();
