@@ -11,14 +11,18 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_events_2.h $
+// $Id: Straight_skeleton_builder_events_2.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
 //
 #ifndef CGAL_STRAIGHT_SKELETON_BUILDER_EVENTS_2_H
 #define CGAL_STRAIGHT_SKELETON_BUILDER_EVENTS_2_H 1
 
+#include <CGAL/license/Straight_skeleton_2.h>
+
+
 #include<ostream>
 
 #include <CGAL/Straight_skeleton_2/Straight_skeleton_aux.h>
