@@ -11,14 +11,18 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Straight_skeleton_2/include/CGAL/Straight_skeleton_2/Straight_skeleton_builder_traits_2_aux.h $
+// $Id: Straight_skeleton_builder_traits_2_aux.h 599e201 2018-01-18T14:08:08+00:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Fernando Cacciola <fernando_cacciola@ciudad.com.ar>
 //
 #ifndef CGAL_STRAIGHT_SKELETON_BUILDER_TRAITS_2_AUX_H
 #define CGAL_STRAIGHT_SKELETON_BUILDER_TRAITS_2_AUX_H 1
 
+#include <CGAL/license/Straight_skeleton_2.h>
+
+
 #include <CGAL/tags.h>
 #include <CGAL/Handle.h>
 #include <CGAL/Uncertain.h>
@@ -122,7 +126,7 @@ public:
       if ( fr )
         return From_Filtered(fr);
     }
-    catch (Uncertain_conversion_exception) {}
+    catch (Uncertain_conversion_exception&) {}
 
     Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
     EC_result_type er = Exact_construction(To_Exact(a1)) ;
@@ -140,7 +144,7 @@ public:
       if ( fr )
         return From_Filtered(fr);
     }
-    catch (Uncertain_conversion_exception) {}
+    catch (Uncertain_conversion_exception&) {}
     
     Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
     EC_result_type er = Exact_construction(To_Exact(a1), To_Exact(a2)) ;
@@ -158,7 +162,7 @@ public:
       if ( fr )
         return From_Filtered(fr);
     }
-    catch (Uncertain_conversion_exception) {}
+    catch (Uncertain_conversion_exception&) {}
     
     Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
     EC_result_type er = Exact_construction(To_Exact(a1), To_Exact(a2), To_Exact(a3)) ;
@@ -177,7 +181,7 @@ public:
       if ( fr )
         return From_Filtered(fr);
     }
-    catch (Uncertain_conversion_exception) {}
+    catch (Uncertain_conversion_exception&) {}
     
     Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
     EC_result_type er = Exact_construction(To_Exact(a1), To_Exact(a2), To_Exact(a3), To_Exact(a4)) ;
@@ -196,7 +200,7 @@ public:
       if ( fr )
         return From_Filtered(fr);
     }
-    catch (Uncertain_conversion_exception) {}
+    catch (Uncertain_conversion_exception&) {}
     
     Protect_FPU_rounding<!Protection> P(CGAL_FE_TONEAREST);
     EC_result_type er = Exact_construction(To_Exact(a1), To_Exact(a2), To_Exact(a3), To_Exact(a4), To_Exact(a5)) ;
@@ -296,10 +300,10 @@ public:
         mCSIdx=0; mNCSIdx=1; break ;
         
       case TRISEGMENT_COLLINEARITY_ALL:
-        mCSIdx=-1; mNCSIdx=-1; break ;
+        mCSIdx = mNCSIdx = (std::numeric_limits<unsigned>::max)(); break ;
         
       case TRISEGMENT_COLLINEARITY_NONE:
-        mCSIdx=-1; mNCSIdx=-1; break ;
+        mCSIdx = mNCSIdx = (std::numeric_limits<unsigned>::max)(); break ;
     }
   }
     
