@@ -1,9 +1,10 @@
 // Copyright (c) 2005  INRIA Sophia-Antipolis (France).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Stream_lines_2/include/CGAL/Stream_lines_2.h $
-// $Id: Stream_lines_2.h 58437 2010-09-02 10:19:55Z afabri $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Stream_lines_2/include/CGAL/Stream_lines_2.h $
+// $Id: Stream_lines_2.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Abdelkrim Mebarki <Abdelkrim.Mebarki@sophia.inria.fr>
@@ -20,9 +22,10 @@
 #ifndef CGAL_STREAM_LINES_2_H_ 
 #define CGAL_STREAM_LINES_2_H_
 
+#include <CGAL/license/Stream_lines_2.h>
+
+
 #include <CGAL/basic.h>
-#include <CGAL/Cartesian.h>
-#include <CGAL/Filtered_kernel.h>
 #include <CGAL/Delaunay_triangulation_2.h>
 
 #include <queue>
@@ -35,8 +38,10 @@
 #include <CGAL/streamlines_assertions.h>
 
 #include <boost/tuple/tuple.hpp>
+#include <boost/random/uniform_real.hpp> // undocumented class
 #include <boost/random/linear_congruential.hpp>
 #include <boost/random/uniform_smallint.hpp>
+#include <boost/shared_ptr.hpp>
 
 namespace CGAL {
 
@@ -100,11 +105,22 @@ protected:
   Point_2                                                        seed_point;
   int                                                            samp_step;
   unsigned int _number_of_lines;
-  Vector_field_2 * vf_2;
-  Integrator_2 * int_2;
+  boost::shared_ptr<Vector_field_2> vf_2;
+  boost::shared_ptr<Integrator_2> int_2;
 public:
   void set_separating_distance(FT new_value){separating_distance = new_value;}
   void set_saturation_ratio(FT new_value){ saturation_ratio = new_value;}
+
+  FT get_separating_distance() const
+  {
+    return separating_distance;
+  } 
+
+  FT get_saturation_ratio() const
+  {
+    return saturation_ratio;
+  }
+
   void update()
     {
       ir = il = 0; // initialization
@@ -234,8 +250,8 @@ vector_field_2, const Integrator_2 & m_integrator, const FT & m_separating_dista
       m_DT.insert(pPoint);
     }
   _number_of_lines = 0;
-  vf_2 = new Vector_field_2(vector_field_2);
-  int_2 = new Integrator_2(m_integrator);
+  vf_2 = boost::shared_ptr<Vector_field_2>(new Vector_field_2(vector_field_2));
+  int_2 = boost::shared_ptr<Integrator_2>(new Integrator_2(m_integrator));
   samp_step = sampling_step;
   stl_container.clear();
   place_stream_lines(vector_field_2, m_integrator,
