@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Surface_mesher/include/CGAL/Surface_mesh_complex_2_in_triangulation_3.h $
+// $Id: Surface_mesh_complex_2_in_triangulation_3.h 799260f 2018-01-18T10:05:01+00:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Laurent Rineau
@@ -21,6 +22,10 @@
 #ifndef CGAL_SURFACE_MESH_COMPLEX_2_IN_TRIANGULATION_3_H
 #define CGAL_SURFACE_MESH_COMPLEX_2_IN_TRIANGULATION_3_H
 
+#include <CGAL/license/Surface_mesher.h>
+
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/Complex_2_in_triangulation_3.h>
 
 namespace CGAL {
@@ -43,4 +48,6 @@ public:
 
 } // end namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_SURFACE_MESH_COMPLEX_2_IN_TRIANGULATION_3_H
