@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Surface_mesher/include/CGAL/Surface_mesher/Has_edges.h $
+// $Id: Has_edges.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Laurent RINEAU
@@ -21,6 +22,9 @@
 #ifndef CGAL_SURFACE_MESHER_HAS_EDGES_H
 #define CGAL_SURFACE_MESHER_HAS_EDGES_H
 
+#include <CGAL/license/Surface_mesher.h>
+
+
 namespace CGAL {
   namespace Surface_mesher {
 
