@@ -2,9 +2,10 @@
 // Copyright (c) 2008       GeometryFactory, Sophia Antipolis (France)
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -12,8 +13,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Surface_mesher/include/CGAL/Surface_mesher/Implicit_surface_oracle_3.h $
-// $Id: Implicit_surface_oracle_3.h 56293 2010-05-17 14:19:22Z afabri $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Surface_mesher/include/CGAL/Surface_mesher/Implicit_surface_oracle_3.h $
+// $Id: Implicit_surface_oracle_3.h 26c857a 2018-06-13T15:11:45+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Steve OUDOT, Laurent RINEAU
@@ -21,10 +23,17 @@
 #ifndef CGAL_SURFACE_MESHER_IMPLICIT_SURFACE_ORACLE_3_H
 #define CGAL_SURFACE_MESHER_IMPLICIT_SURFACE_ORACLE_3_H
 
+#include <CGAL/license/Surface_mesher.h>
+
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/Surface_mesher/Null_oracle_visitor.h>
 #include <CGAL/point_generators_3.h>
 #include <CGAL/Surface_mesher/Sphere_oracle_3.h>
 #include <CGAL/Real_embeddable_traits.h>
+#include <CGAL/Surface_mesher/Profile_counter.h>
+#include <CGAL/squared_distance_3.h>
+#include <CGAL/Origin.h>
 
 #include <boost/mpl/has_xxx.hpp>
 
@@ -97,7 +106,7 @@ namespace CGAL {
   namespace {
     
   template <typename T>
-  struct Return_min : std::binary_function<T, T, T>
+  struct Return_min : CGAL::cpp98::binary_function<T, T, T>
   {
     T operator()(const T& a, const T& b) const
     {
@@ -487,14 +496,10 @@ namespace CGAL {
 
   };  // end Implicit_surface_oracle_3
 
-template <typename FT>
-FT approximate_sqrt(const FT x) {
-  return FT (CGAL_NTS sqrt(CGAL_NTS to_double(x)));
-}
-
   }  // namespace Surface_mesher
 
 } // namespace CGAL
 
+#include <CGAL/enable_warnings.h>
 
 #endif  // CGAL_SURFACE_MESHER_IMPLICIT_SURFACE_ORACLE_3_H
