@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/TDS_2/include/CGAL/Triangulation_data_structure_2.h $
+// $Id: Triangulation_data_structure_2.h 7ddef3d 2019-03-05T16:20:49+01:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Mariette Yvinec
@@ -21,7 +22,10 @@
 #ifndef CGAL_TRIANGULATION_DATA_STRUCTURE_2_H
 #define CGAL_TRIANGULATION_DATA_STRUCTURE_2_H
 
-#include <CGAL/basic.h>
+#include <CGAL/license/TDS_2.h>
+
+
+#include <CGAL/config.h>
 #include <iostream>
 #include <list>
 #include <map>
@@ -292,9 +296,11 @@ public:
 // 		  FaceIt face_begin,
 // 		  FaceIt face_end);
   
-  Vertex_handle create_vertex(const Vertex &v = Vertex());
+  Vertex_handle create_vertex();
+  Vertex_handle create_vertex(const Vertex &v);
   Vertex_handle create_vertex(Vertex_handle v); //calls copy constructor 
-  Face_handle create_face(const Face& f = Face());
+  Face_handle create_face();
+  Face_handle create_face(const Face& f);
   Face_handle create_face(Face_handle f); //calls copy constructor 
 
  Face_handle create_face(Face_handle f1, int i1, 
@@ -312,7 +318,9 @@ public:
 			  Face_handle f1, 
 			  Face_handle f2, 
 			  Face_handle f3);
+
   void set_adjacency(Face_handle f0, int i0, Face_handle f1, int i1) const;
+
   void delete_face(Face_handle);
   void delete_vertex(Vertex_handle);
 
@@ -353,10 +361,12 @@ public:
   // HELPING
 private:
   typedef std::pair<Vertex_handle,Vertex_handle> Vh_pair;
+public:
   void  set_adjacency(Face_handle fh, 
 		      int ih, 
 		      std::map< Vh_pair, Edge>& edge_map);
   void reorient_faces();
+private:
   bool dim_down_precondition(Face_handle f, int i);
 
 public:
@@ -424,6 +434,106 @@ public:
 
   // template members definition
 public:
+
+  /************* START OF MODIFICATIONS ***************/
+
+  template< class FaceIt >
+  Vertex_handle insert_in_hole(FaceIt face_begin, FaceIt face_end) 
+  {
+    Vertex_handle newv = create_vertex();
+    insert_in_hole(newv, face_begin, face_end);
+    return newv;
+  }
+
+
+  template< class FaceIt >
+  void insert_in_hole(Vertex_handle v, FaceIt face_begin, FaceIt face_end) 
+  {
+
+    CGAL_triangulation_precondition(dimension() == 2);
+
+    std::vector<Face_handle>  new_faces;
+    std::vector<Edge>         bdry_edges;
+
+    Face_handle fh = *face_begin;
+    int ii = 0;
+    bool found_boundary = false;
+    do {
+      if (std::find(face_begin, face_end, fh->neighbor(ii)) == face_end) {
+        bdry_edges.push_back(Edge(fh, ii));
+        found_boundary = true;
+      } else {
+        int newi = fh->neighbor(ii)->index(fh->vertex(ccw(ii)));
+        fh = fh->neighbor(ii);
+        ii = newi;
+      }
+    } while(!found_boundary);
+    // Now we have found ONE edge on the boundary. 
+    // From that one edge we must walk on the boundary 
+    // of the hole until we've covered the whole thing.
+
+    bool complete_walk = false;
+    do {
+      Face_handle nh = fh->neighbor(ccw(ii));
+      if (std::find(face_begin, face_end, nh) == face_end) {
+        ii = ccw(ii);
+        Edge new_edge(fh, ii);
+        if (std::find(bdry_edges.begin(), bdry_edges.end(), new_edge) == bdry_edges.end()) {
+          bdry_edges.push_back(Edge(fh, ii));
+        } else {
+          complete_walk = true;
+        }
+      } else {
+        int newi = cw(nh->index(fh->vertex(ii)));
+        fh = nh;
+        ii = newi;
+      }
+    } while (!complete_walk);
+    // At this point, bdry_edges contains the edges that define
+    // the boundary of the hole with a specific ordering: for any
+    // two consecutive edges in the vector e1 = (f1, i1), 
+    // e2 = (f2, i2) it holds that 
+    //      f1->vertex(cw(i1)) == f2->vertex(ccw(i2))
+
+    for (unsigned int jj = 0; jj < bdry_edges.size(); jj++) {
+      Face_handle fh = bdry_edges[jj].first;
+      int idx = bdry_edges[jj].second;
+      
+      Vertex_handle v1 = fh->vertex(ccw(idx));
+      Vertex_handle v2 = fh->vertex(cw(idx));
+
+      Face_handle nf = fh->neighbor(idx);
+      int jdx = mirror_index(fh, idx);
+
+      Face_handle new_f = create_face(v, v1, v2);
+      v1->set_face(new_f);
+      set_adjacency(new_f, 0, nf, jdx);
+      new_faces.push_back(new_f);
+    }
+    // At this point we have created all the new faces of the triangulation,
+    // and we have set adjacency relationships with the faces on the border
+    // of the hole.
+
+    for (unsigned int i = 0; i < new_faces.size() - 1; i++) {
+      set_adjacency(new_faces[i], 1, new_faces[i+1], 2);
+    }
+    set_adjacency(new_faces[0], 2, new_faces[new_faces.size()-1], 1);
+    // Now we have also set adjacency relationships between the new faces.
+
+    for (FaceIt it = face_begin; it != face_end; it++) {
+      delete_face(*it);
+    }
+    // The old faces that were in conflict are now deleted.
+
+    v->set_face(new_faces[0]);
+    // Set the pointer of the new vertex to one of the new faces.
+  }
+
+
+
+  /************* END OF MODIFICATIONS ***************/
+
+
   template< class EdgeIt>
   Vertex_handle star_hole(EdgeIt edge_begin, EdgeIt edge_end)
   // creates a new vertex 
@@ -1244,6 +1354,14 @@ make_hole(Vertex_handle v, List_edges& hole)
   return;
 }
 
+template <class Vb, class Fb>
+inline
+typename Triangulation_data_structure_2<Vb,Fb>::Vertex_handle
+Triangulation_data_structure_2<Vb,Fb>::
+create_vertex()
+{
+  return vertices().emplace();
+}
 
 template <class Vb, class Fb>
 inline
@@ -1263,6 +1381,14 @@ create_vertex(Vertex_handle vh)
   return vertices().insert(*vh);
 }
 
+template <class Vb, class Fb>
+typename Triangulation_data_structure_2<Vb,Fb>::Face_handle
+Triangulation_data_structure_2<Vb,Fb>::
+create_face()
+{
+  return faces().emplace();
+}
+
 template <class Vb, class Fb>
 typename Triangulation_data_structure_2<Vb,Fb>::Face_handle
 Triangulation_data_structure_2<Vb,Fb>::
@@ -1527,7 +1653,7 @@ join_vertices(Face_handle f, int i, Vertex_handle v)
     return join_vertices(f->neighbor(i), mirror_index(f,i), v);
   }
 
-  int deg2 = degree(v2);
+  size_type deg2 = degree(v2);
 
   CGAL_triangulation_precondition( deg2 >= 3 );
 
@@ -1605,7 +1731,8 @@ join_vertices(Face_handle f, int i, Vertex_handle v)
     ++fc;
   } while ( fc != fc_start );
 
-  CGAL_triangulation_assertion( int(star_faces_of_v2.size()) == deg2 );
+  CGAL_triangulation_assertion(
+    static_cast<size_type>(star_faces_of_v2.size()) == deg2 );
 
   // from this point and on we modify the values
 
@@ -1630,7 +1757,7 @@ join_vertices(Face_handle f, int i, Vertex_handle v)
   if ( v1->face() == f || v1->face() == g ) v1->set_face(tl);
 
 
-#ifndef CGAL_NO_ASSERTIONS
+#if ! defined(CGAL_TRIANGULATION_NO_ASSERTIONS) && ! defined(CGAL_NO_ASSERTIONS)
   for (Face_iterator fit = faces_begin(); fit != faces_end(); ++fit) {
     int id;
     CGAL_triangulation_assertion( !fit->has_vertex(v2, id) );
@@ -1892,7 +2019,7 @@ namespace internal { namespace TDS_2{
   struct Default_vertex_converter
   {
     Vertex_tgt operator()(const Vertex_src& src) const {
-      return Vertex_src( src.point() );
+      return Vertex_tgt( src.point() );
     }
     
     void operator()(const Vertex_src&,Vertex_tgt&) const {}
@@ -2270,24 +2397,24 @@ reorient_faces()
   std::set<Face_handle> oriented_set;
   std::stack<Face_handle>  st;
   Face_iterator fit = faces_begin();
-  int nf  = std::distance(faces_begin(),faces_end());
+  std::ptrdiff_t nf  = std::distance(faces_begin(),faces_end());
 
-  while (static_cast<int>(oriented_set.size()) != nf) {
-    while ( oriented_set.find(fit) != oriented_set.end()){
+  while (0 != nf) {
+    while ( !oriented_set.insert(fit).second ){
       ++fit; // find a germ for  non oriented components 
     }
     // orient component
-    oriented_set.insert(fit);
+    --nf;
     st.push(fit);
     while ( ! st.empty()) {
       Face_handle fh = st.top();
       st.pop();
       for(int ih = 0 ; ih < 3 ; ++ih){
 	Face_handle fn = fh->neighbor(ih);
-	if (oriented_set.find(fn) == oriented_set.end()){
+	if (oriented_set.insert(fn).second){
 	  int in = fn->index(fh);
 	  if (fn->vertex(cw(in)) != fh->vertex(ccw(ih))) fn->reorient();
-	  oriented_set.insert(fn);
+          --nf;
 	  st.push(fn);
 	}
       }
