@@ -1,9 +1,10 @@
 // Copyright (c) 1998, 2001, 2003  INRIA Sophia-Antipolis (France).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Triangulation_3/include/CGAL/Triangulation_hierarchy_3.h $
-// $Id: Triangulation_hierarchy_3.h 60298 2010-12-10 17:02:10Z lrineau $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Triangulation_3/include/CGAL/Triangulation_hierarchy_3.h $
+// $Id: Triangulation_hierarchy_3.h bfdb89e 2018-05-24T14:50:40+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Olivier Devillers <Olivier.Devillers@sophia.inria.fr>
 //                 Sylvain Pion
@@ -20,16 +22,11 @@
 #ifndef CGAL_TRIANGULATION_HIERARCHY_3_H
 #define CGAL_TRIANGULATION_HIERARCHY_3_H
 
-#include <CGAL/basic.h>
-#include <CGAL/triangulation_assertions.h>
-#include <CGAL/Triangulation_hierarchy_vertex_base_3.h>
-#include <CGAL/Location_policy.h>
+#include <CGAL/license/Triangulation_3.h>
 
-#include <boost/random/linear_congruential.hpp>
-#include <boost/random/geometric_distribution.hpp>
-#include <boost/random/variate_generator.hpp>
-
-namespace CGAL {
+// Commented because the class is actually used by Delaunay_triangulation_hierarchy_3.h
+// #define CGAL_DEPRECATED_HEADER "<CGAL/Triangulation_hierarchy_3.h>"
+// #include <CGAL/internal/deprecation_warning.h>
 
 // This class is deprecated, but must be kept for backward compatibility.
 //
@@ -39,6 +36,34 @@ namespace CGAL {
 //
 // Then, later, maybe merge the Compact/Fast codes in a cleaner factorized way.
 
+#include <CGAL/basic.h>
+#include <CGAL/internal/Has_nested_type_Bare_point.h>
+#include <CGAL/triangulation_assertions.h>
+#include <CGAL/Triangulation_hierarchy_vertex_base_3.h>
+#include <CGAL/Location_policy.h>
+
+#include <CGAL/internal/boost/function_property_map.hpp>
+
+#include <boost/random/linear_congruential.hpp>
+#include <boost/random/geometric_distribution.hpp>
+#include <boost/random/variate_generator.hpp>
+#include <boost/utility/result_of.hpp>
+
+#ifndef CGAL_TRIANGULATION_3_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+#include <CGAL/Spatial_sort_traits_adapter_3.h>
+#include <CGAL/spatial_sort.h>
+#include <CGAL/internal/info_check.h>
+
+#include <boost/tuple/tuple.hpp>
+#include <boost/iterator/zip_iterator.hpp>
+#include <boost/mpl/and.hpp>
+#include <boost/mpl/identity.hpp>
+#include <boost/mpl/if.hpp>
+
+#endif //CGAL_TRIANGULATION_3_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+
+namespace CGAL {
+
 template < class Tr >
 class Triangulation_hierarchy_3
   : public Tr
@@ -53,7 +78,6 @@ public:
   typedef Tr                                   Tr_Base;
   typedef Fast_location                        Location_policy;
   typedef typename Tr_Base::Geom_traits        Geom_traits;
-  typedef typename Tr_Base::Point              Point;
   typedef typename Tr_Base::size_type          size_type;
   typedef typename Tr_Base::Vertex_handle      Vertex_handle;
   typedef typename Tr_Base::Cell_handle        Cell_handle;
@@ -65,6 +89,12 @@ public:
   typedef typename Tr_Base::Finite_facets_iterator    Finite_facets_iterator;
   typedef typename Tr_Base::Finite_edges_iterator     Finite_edges_iterator;
 
+  // this may be weighted or not
+  typedef typename Tr_Base::Point              Point;
+
+  typedef typename Tr_Base::Weighted_tag       Weighted_tag;
+  typedef typename Tr_Base::Periodic_tag       Periodic_tag;
+
   using Tr_Base::number_of_vertices;
   using Tr_Base::geom_traits;
 
@@ -124,13 +154,37 @@ public:
   Vertex_handle insert(const Point &p, Locate_type lt, Cell_handle loc,
                        int li, int lj);
 
+#ifndef CGAL_TRIANGULATION_3_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
   template < class InputIterator >
-  std::ptrdiff_t insert(InputIterator first, InputIterator last)
+  std::ptrdiff_t
+  insert( InputIterator first, InputIterator last,
+          typename boost::enable_if<
+            boost::is_convertible<
+                typename std::iterator_traits<InputIterator>::value_type,
+                Point
+            >
+          >::type* = NULL
+  )
+#else
+  template < class InputIterator >
+  std::ptrdiff_t
+  insert( InputIterator first, InputIterator last)
+#endif //CGAL_TRIANGULATION_3_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
   {
     size_type n = number_of_vertices();
 
       std::vector<Point> points (first, last);
-      spatial_sort (points.begin(), points.end(), geom_traits());
+
+      // Spatial sort can only be used with Geom_traits::Point_3: we need an adapter
+      typedef typename Geom_traits::Construct_point_3 Construct_point_3;
+      typedef typename boost::result_of<const Construct_point_3(const Point&)>::type Ret;
+      typedef CGAL::internal::boost_::function_property_map<Construct_point_3, Point, Ret> fpmap;
+      typedef CGAL::Spatial_sort_traits_adapter_3<Geom_traits, fpmap> Search_traits_3;
+
+      spatial_sort(points.begin(), points.end(),
+                   Search_traits_3(
+                     CGAL::internal::boost_::make_function_property_map<Point, Ret, Construct_point_3>(
+                       geom_traits().construct_point_3_object()), geom_traits()));
 
       // hints[i] is the vertex of the previously inserted point in level i.
       // Thanks to spatial sort, they are better hints than what the hierarchy
@@ -153,6 +207,120 @@ public:
       return number_of_vertices() - n;
   }
 
+#ifndef CGAL_TRIANGULATION_3_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+private:
+  //top stands for tuple-or-pair
+  template <class Info>
+  const Point& top_get_first(const std::pair<Point,Info>& pair) const { return pair.first; }
+  template <class Info>
+  const Info& top_get_second(const std::pair<Point,Info>& pair) const { return pair.second; }
+  template <class Info>
+  const Point& top_get_first(const boost::tuple<Point,Info>& tuple) const { return boost::get<0>(tuple); }
+  template <class Info>
+  const Info& top_get_second(const boost::tuple<Point,Info>& tuple) const { return boost::get<1>(tuple); }
+
+  template<class Construct_bare_point, class Container>
+  struct Index_to_Bare_point
+  {
+    const typename Geom_traits::Point_3& operator()(const std::size_t& i) const
+    {
+      return cp(c[i]);
+    }
+
+    Index_to_Bare_point(const Container& c, const Construct_bare_point& cp)
+      : c(c), cp(cp) { }
+
+    const Container& c;
+    const Construct_bare_point cp;
+  };
+
+  template <class Tuple_or_pair,class InputIterator>
+  std::ptrdiff_t insert_with_info(InputIterator first,InputIterator last)
+  {
+    size_type n = number_of_vertices();
+    std::vector<std::size_t> indices;
+    std::vector<Point> points;
+    std::vector<typename Vertex::Info> infos;
+    std::size_t index=0;
+    for (InputIterator it=first;it!=last;++it){
+      Tuple_or_pair value=*it;
+      points.push_back( top_get_first(value)  );
+      infos.push_back ( top_get_second(value) );
+      indices.push_back(index++);
+    }
+
+    // We need to sort the points and their info at the same time through
+    // the `indices` vector AND spatial sort can only handle Geom_traits::Point_3.
+    typedef typename Geom_traits::Construct_point_3 Construct_point_3;
+    typedef Index_to_Bare_point<Construct_point_3,
+                                std::vector<Point> > Access_bare_point;
+    typedef typename boost::result_of<const Construct_point_3(const Point&)>::type Ret;
+    typedef CGAL::internal::boost_::function_property_map<Access_bare_point, std::size_t, Ret> fpmap;
+    typedef CGAL::Spatial_sort_traits_adapter_3<Geom_traits, fpmap> Search_traits_3;
+
+    Access_bare_point accessor(points, geom_traits().construct_point_3_object());
+    spatial_sort(indices.begin(), indices.end(),
+                 Search_traits_3(
+                   CGAL::internal::boost_::make_function_property_map<
+                     std::size_t, Ret, Access_bare_point>(accessor),
+                   geom_traits()));
+
+
+    // hints[i] is the vertex of the previously inserted point in level i.
+    // Thanks to spatial sort, they are better hints than what the hierarchy
+    // would give us.
+    Vertex_handle hints[maxlevel];
+    for (typename std::vector<std::size_t>::const_iterator
+      it = indices.begin(), end = indices.end();
+      it != end; ++it)
+    {
+        int vertex_level = random_level();
+
+        Vertex_handle v = hints[0] = hierarchy[0]->insert (points[*it], hints[0]);
+        v->info()=infos[*it];
+        Vertex_handle prev = v;
+
+        for (int level = 1; level <= vertex_level; ++level) {
+            v = hints[level] = hierarchy[level]->insert (points[*it], hints[level]);
+            set_up_down(v, prev);
+            prev = v;
+        }
+    }
+    return number_of_vertices() - n;
+  }
+
+public:
+
+  template < class InputIterator >
+  std::ptrdiff_t
+  insert( InputIterator first,
+          InputIterator last,
+          typename boost::enable_if<
+            boost::is_convertible<
+              typename std::iterator_traits<InputIterator>::value_type,
+              std::pair<Point,typename internal::Info_check<Vertex>::type>
+            > >::type* =NULL
+  )
+  {
+    return insert_with_info< std::pair<Point,typename internal::Info_check<Vertex>::type> >(first,last);
+  }
+
+  template <class  InputIterator_1,class InputIterator_2>
+  std::ptrdiff_t
+  insert( boost::zip_iterator< boost::tuple<InputIterator_1,InputIterator_2> > first,
+          boost::zip_iterator< boost::tuple<InputIterator_1,InputIterator_2> > last,
+          typename boost::enable_if<
+            boost::mpl::and_<
+              boost::is_convertible< typename std::iterator_traits<InputIterator_1>::value_type, Point >,
+              boost::is_convertible< typename std::iterator_traits<InputIterator_2>::value_type, typename internal::Info_check<Vertex>::type >
+            >
+          >::type* =NULL
+  )
+  {
+    return insert_with_info< boost::tuple<Point,typename internal::Info_check<Vertex>::type> >(first,last);
+  }
+#endif //CGAL_TRIANGULATION_3_DONT_INSERT_RANGE_OF_POINTS_WITH_INFO
+
   void remove(Vertex_handle v);
 
   template < typename InputIterator >
@@ -175,9 +343,9 @@ public:
     std::vector<Vertex_handle> vo(first, beyond), vc;
     int l=0;
     while(1) {
-      int n = vo.size();
-      if(!n) break;
-      for(int i=0; i<n; i++) {
+      size_type n = vo.size();
+      if(n == 0) break;
+      for(size_type i=0; i<n; i++) {
         if(vo[i]->up() != Vertex_handle()) vc.push_back(vo[i]->up());
       }
       hierarchy[l++]->remove_cluster(vo.begin(), vo.end());
@@ -187,10 +355,6 @@ public:
     return n - this->number_of_vertices();
   }
 
-#ifndef CGAL_NO_DEPRECATED_CODE
-  CGAL_DEPRECATED Vertex_handle move_point(Vertex_handle v, const Point & p);
-#endif
-
   Vertex_handle move_if_no_collision(Vertex_handle v, const Point &p);
   Vertex_handle move(Vertex_handle v, const Point &p);
 
@@ -206,7 +370,7 @@ public: // some internal methods
 		
   template <class OutputItCells>
   Vertex_handle insert_and_give_new_cells(const Point& p,
-                                          OutputItCells fit,
+                                          OutputItCells /* fit */,
                                           Vertex_handle hint)
   {
     return insert_and_give_new_cells(p, hint == Vertex_handle() ? 
@@ -551,34 +715,6 @@ remove_and_give_new_cells(Vertex_handle v, OutputItCells fit)
   }
 }
 
-#ifndef CGAL_NO_DEPRECATED_CODE
-template < class Tr >
-typename Triangulation_hierarchy_3<Tr>::Vertex_handle
-Triangulation_hierarchy_3<Tr>::
-move_point(Vertex_handle v, const Point & p)
-{
-  CGAL_triangulation_precondition(v != Vertex_handle());
-  Vertex_handle old, ret;
-
-  for (std::size_t l = 0; l < maxlevel; ++l) {
-    Vertex_handle u = v->up();
-    Vertex_handle w = hierarchy[l]->move_point(v, p);
-    if (l == 0) {
-	ret = w;
-    }
-    else {
-        set_up_down(w, old);
-    }
-    if (u == Vertex_handle())
-	break;
-    old = w;
-    v = u;
-  }
-
-  return ret;
-}
-#endif
-
 template <class Tr>
 typename Triangulation_hierarchy_3<Tr>::Vertex_handle
 Triangulation_hierarchy_3<Tr>::
