@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Polytope_distance_d/include/CGAL/Width_3.h $
+// $Id: Width_3.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Thomas Herrmann, Lutz Kettner
@@ -21,6 +22,9 @@
 #ifndef CGAL_WIDTH_3_H
 #define CGAL_WIDTH_3_H
 
+#include <CGAL/license/Polytope_distance_d.h>
+
+
 #include <CGAL/basic.h>
 #include <cstdlib>
 #include <iostream>
