@@ -11,8 +11,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Triangulation_2/include/CGAL/boost/graph/graph_traits_Triangulation_2.h $
+// $Id: graph_traits_Triangulation_2.h 560c77f 2019-07-14T22:54:16+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Andreas Fabri, Fernando Cacciola
@@ -20,6 +21,11 @@
 #ifndef CGAL_GRAPH_TRAITS_TRIANGULATION_2_H
 #define CGAL_GRAPH_TRAITS_TRIANGULATION_2_H
 
+#include <functional>
+
+// include this to avoid a VC15 warning
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/graph_traits_Triangulation_data_structure_2.h>
 #include <boost/config.hpp>
 #include <boost/iterator_adaptors.hpp>
 #include <boost/graph/graph_traits.hpp>
@@ -28,6 +34,7 @@
 #include <CGAL/boost/graph/properties.h>
 #include <CGAL/Triangulation_2.h>
 #include <CGAL/Iterator_range.h>
+#include <CGAL/iterator.h>
 
 // The functions and classes in this file allows the user to
 // treat a CGAL Triangulation_2 object as a boost graph "as is". No
@@ -38,214 +45,48 @@ namespace CGAL {
 
   namespace detail {
 
-template < class T, class EdgeBase >
-class Edge : public EdgeBase {
-  typedef typename T::Face_handle Face_handle ;
-  public:
-  
-  Edge()
-  {}
+    template <typename Tr>
+    struct T2_halfedge_descriptor
+    {
+      typedef typename Tr::Face_handle face_descriptor;
+      face_descriptor first;
+      int second;
+      operator std::pair<face_descriptor, int>() { return std::make_pair(first,second); }
+      
+      T2_halfedge_descriptor()
+        : first(), second(0)
+      {}
+      
+      T2_halfedge_descriptor(const typename Tr::Edge& e)
+        : first(e.first), second(e.second)
+      {}
+      
+      T2_halfedge_descriptor(face_descriptor fd, int i)
+        : first(fd), second(i)
+      {}
+      
+      friend std::size_t hash_value(const T2_halfedge_descriptor& h)
+      {
+        return hash_value(h.first);
+      } 
+      
+      bool operator==(const T2_halfedge_descriptor& other) const
+      {
+        return (first == other.first) && (second == other.second);
+      }
+      
+      bool operator!=(const T2_halfedge_descriptor& other) const
+      {
+        return (first != other.first) || (second != other.second);
+      }
 
-  Edge(Face_handle  fh, int i)
-    : EdgeBase(fh,i)
-  {}
-  
-  Edge(const EdgeBase& e)
-    : EdgeBase(e)
-  {}
-
-  Edge(const Edge& e)
-    : EdgeBase(e)
-  {}
-
-  Edge&
-  operator=(const Edge& e)
-  {
-    this->first = e.first;
-    this->second = e.second;
-    return *this;
-}
-
-  bool operator==(const Edge& other) const
-  {
-    if((this->first == other.first)&&(this->second == other.second)) return true;
-    Face_handle fh = this->first->neighbor(this->second);
-    if(other.first != fh) return false;
-    int i = fh->index(this->first);
-    return (other.second == i);
-  }
-
-  bool operator!=(Edge& other) const
-  {
-    return ! (*this == other);
-  }
-};
-
-template <class Circ, class E>
-class Out_edge_circulator : public Circ
-{
-private:
-  mutable E e;
-
-public:
-
-  typedef E value_type;
-  typedef E* pointer;
-  typedef E& reference;
-
-  Out_edge_circulator()
-    : Circ()
-  {}
-
-  Out_edge_circulator(Circ c)
-    : Circ(c)
-  {}
-
-  const E& operator*() const
-  {
-    E ed = static_cast<const Circ*>(this)->operator*();
-    e = E(ed.first->neighbor(ed.second), ed.first->neighbor(ed.second)->index(ed.first));
-    return e;
-  }
-};
- 
-template <class Circ, class E>
-class In_edge_circulator : public Circ
-{
-private:
-  mutable E e;
-
-public:
-
-  typedef E value_type;
-  typedef E* pointer;
-  typedef E& reference;
-
-  In_edge_circulator()
-    : Circ()
-  {}
-
-  In_edge_circulator(Circ c)
-    : Circ(c)
-  {}
-
-  const E& operator*() const
-{
-    typename Circ::value_type ed = static_cast<const Circ*>(this)->operator*();
-    e = E(ed);
-    return e;
-  }
-};
-  
-
-  //  The vertex iterator of the bgl must evaluate to a vertex handle, not to a vertex
-template < class T>
-class boost_all_vertices_iterator {
-protected:
- typename T::All_vertices_iterator nt;
-public:
-  typedef typename T::All_vertices_iterator  Iterator;
-  typedef boost_all_vertices_iterator<T> Self;
-
-  typedef typename std::iterator_traits<Iterator>::iterator_category iterator_category;
-  typedef typename T::Vertex_handle  value_type;
-  typedef typename std::iterator_traits<Iterator>::difference_type           difference_type;
-  typedef value_type      reference;
-  typedef value_type      pointer;
-
-  // CREATION
-  // --------
-
-  boost_all_vertices_iterator()
-  {}
-
-  boost_all_vertices_iterator( Iterator j) : nt(j) {}
-
-  // OPERATIONS Forward Category
-  // ---------------------------
-
-
-  bool operator==( const Self& i) const { return ( nt == i.nt); }
-  bool operator!=( const Self& i) const { return !(nt == i.nt );   }
-  value_type  operator*() const  { return nt; }
-  value_type    operator->()  { return nt; }
-
-  Self& operator++() {
-    ++nt;
-    return *this;
-  }
-
-  Self  operator++(int) {
-    Self tmp = *this;
-    ++*this;
-    return tmp;
-  }
-
-  Self& operator--() {
-    --nt;
-    return *this;
-  }
-
-  Self  operator--(int) {
-    Self tmp = *this;
-    --*this;
-    return tmp;
-  }
-};
-
-template < class T>
-class boost_all_faces_iterator {
-protected:
- typename T::All_faces_iterator nt;
-public:
-  typedef typename T::All_faces_iterator  Iterator;
-  typedef boost_all_faces_iterator<T> Self;
-
-  typedef typename std::iterator_traits<Iterator>::iterator_category iterator_category;
-  typedef typename T::Face_handle  value_type;
-  typedef typename std::iterator_traits<Iterator>::difference_type           difference_type;
-  typedef value_type      reference;
-  typedef value_type      pointer;
-
-  // CREATION
-  // --------
-
-  boost_all_faces_iterator()
-  {}
-
-  boost_all_faces_iterator( Iterator j) : nt(j) {}
-
-  // OPERATIONS Forward Category
-  // ---------------------------
-
-
-  bool operator==( const Self& i) const { return ( nt == i.nt); }
-  bool operator!=( const Self& i) const { return !(nt == i.nt );   }
-  value_type  operator*() const  { return nt; }
-  value_type    operator->()  { return nt; }
-
-  Self& operator++() {
-    ++nt;
-    return *this;
-  }
-
-  Self  operator++(int) {
-    Self tmp = *this;
-    ++*this;
-    return tmp;
-  }
-
-  Self& operator--() {
-    --nt;
-    return *this;
-  }
-
-  Self  operator--(int) {
-    Self tmp = *this;
-    --*this;
-    return tmp;
-  }
-};
+      bool operator<(const T2_halfedge_descriptor& other) const
+      {
+        if(first < other.first) return true;
+        if(first > other.first) return false;
+        return second  < other.second;
+      }
+    };
 
 
   } // namespace detail
@@ -270,46 +111,12 @@ namespace boost {
     typedef typename CGAL::Triangulation_2<GT,TDS>::All_edges_iterator  edge_iterator;
 
 
-    // with just a typedef to Edge VC++ has ambiguities for function `next()`
-    struct halfedge_descriptor
-    {
-      face_descriptor first;
-      int second;
-      operator std::pair<face_descriptor, int>() { return std::make_pair(first,second); }
-      
-      halfedge_descriptor()
-      {}
-      
-      halfedge_descriptor(const typename Triangulation::Edge& e)
-        : first(e.first), second(e.second)
-      {}
-      
-      halfedge_descriptor(face_descriptor fd, int i)
-        : first(fd), second(i)
-      {}
-      
-      bool operator==(const halfedge_descriptor& other) const
-      {
-        return (first == other.first) && (second == other.second);
-      }
-      
-      bool operator!=(const halfedge_descriptor& other) const
-      {
-        return (first != other.first) || (second != other.second);
-      }
+    typedef CGAL::detail::T2_halfedge_descriptor<Triangulation> halfedge_descriptor;
 
-      bool operator<(const halfedge_descriptor& other) const
-      {
-        if(first < other.first) return true;
-        if(first > other.first) return false;
-        return second  < other.second;
-      }
-    };
+    typedef typename Triangulation::All_halfedges_iterator  halfedge_iterator;
 
-    typedef typename CGAL::Triangulation_2<GT,TDS>::All_halfedges_iterator  halfedge_iterator;
-
-    typedef CGAL::detail::boost_all_vertices_iterator<Triangulation> vertex_iterator;
-    typedef CGAL::detail::boost_all_faces_iterator<Triangulation> face_iterator;
+    typedef CGAL::Prevent_deref<typename Triangulation::All_vertices_iterator> vertex_iterator;
+    typedef CGAL::Prevent_deref<typename Triangulation::All_faces_iterator> face_iterator;
     typedef CGAL::Counting_iterator<CGAL::detail::Out_edge_circulator<typename Triangulation::Edge_circulator, edge_descriptor>, edge_descriptor > out_edge_iterator;
     typedef CGAL::Counting_iterator<CGAL::detail::In_edge_circulator<typename Triangulation::Edge_circulator, edge_descriptor>, edge_descriptor > in_edge_iterator;
     typedef CGAL::Counting_iterator<typename Triangulation::Vertex_circulator> Incident_vertices_iterator;
@@ -325,10 +132,10 @@ namespace boost {
     typedef size_type faces_size_type;
     typedef size_type degree_size_type;
 
-  // nulls
-  static vertex_descriptor   null_vertex() { return vertex_descriptor(); }
-  static face_descriptor     null_face()   { return face_descriptor(); }
-  static halfedge_descriptor     null_halfedge()   { return halfedge_descriptor(); }
+    // nulls
+    static vertex_descriptor   null_vertex() { return vertex_descriptor(); }
+    static face_descriptor     null_face()   { return face_descriptor(); }
+    static halfedge_descriptor     null_halfedge()   { return halfedge_descriptor(); }
   };
 
 
@@ -446,6 +253,57 @@ namespace CGAL {
     return edge_descriptor(e.first,e.second);
   }
 
+  template <class Gt, class Tds>
+  std::pair<typename boost::graph_traits< Triangulation_2<Gt,Tds> >::edge_descriptor,
+            bool>
+  edge(typename boost::graph_traits< Triangulation_2<Gt,Tds> >::vertex_descriptor u,
+       typename boost::graph_traits< Triangulation_2<Gt,Tds> >::vertex_descriptor v,
+       const Triangulation_2<Gt,Tds>& g)
+  {
+    typedef typename boost::graph_traits< Triangulation_2<Gt,Tds> >::edge_descriptor edge_descriptor;
+
+    typename Triangulation_2<Gt,Tds>::Edge_circulator c = g.incident_edges(u), done(c);
+    if (c != 0) {
+      do {
+        // find the index of the other vertex of *c
+        int indv = 3 - c->first->index(u) - c->second;
+        if(c->first->vertex(indv) == v)
+          return std::make_pair(edge_descriptor(c->first, c->second), true);
+      } while (++c != done);
+    }
+
+    return std::make_pair(edge_descriptor(), false);
+  }
+
+  template <class Gt, class Tds>
+  std::pair<typename boost::graph_traits<Triangulation_2<Gt,Tds> >::halfedge_descriptor,
+            bool>
+  halfedge(typename boost::graph_traits<Triangulation_2<Gt,Tds> >::vertex_descriptor u,
+           typename boost::graph_traits<Triangulation_2<Gt,Tds> >::vertex_descriptor v,
+           const Triangulation_2<Gt,Tds>& g)
+  {
+    typedef typename boost::graph_traits< Triangulation_2<Gt,Tds> >::halfedge_descriptor halfedge_descriptor;
+    typedef typename boost::graph_traits< Triangulation_2<Gt,Tds> >::edge_descriptor edge_descriptor;
+    typedef typename boost::graph_traits< Triangulation_2<Gt,Tds> >::face_descriptor face_descriptor;
+
+    std::pair<edge_descriptor, bool> eb = edge(u, v, g);
+
+    if(!eb.second)
+      return std::make_pair(halfedge_descriptor(), false);
+
+    const edge_descriptor& e = eb.first;
+
+    if(e.first->vertex(g.ccw(e.first->index(u))) == v)
+    {
+      return std::make_pair(halfedge_descriptor(e.first, e.second), true);
+    }
+    else
+    {
+      face_descriptor nf = e.first->neighbor(e.second);
+      int idx = nf->index(e.first);
+      return std::make_pair(halfedge_descriptor(nf, idx), true);
+    }
+  }
 
   template <class Gt, class Tds>
   inline Iterator_range<typename boost::graph_traits< Triangulation_2<Gt,Tds> >::vertex_iterator>  
@@ -613,7 +471,7 @@ namespace CGAL {
     {}
     
     long operator[](key_type vh) const {
-      return vh->id(); 
+      return vh->id();
     }
   };
 
@@ -628,11 +486,11 @@ namespace CGAL {
 
     friend reference get(T2_vertex_point_map<Gt,Tds>, key_type vh)
     { 
-      return vh->point(); 
+      return vh->point();
     }
     friend void put(T2_vertex_point_map<Gt,Tds>, key_type vh, reference v)
     {
-      vh->point()=v; 
+      vh->point() = v;
     }
     reference operator[](key_type vh) const {
       return vh->point();
@@ -675,7 +533,7 @@ namespace CGAL {
     { }
 
     value_type operator[](key_type e) const {
-      return tr.segment(e).squared_length();
+      return approximate_sqrt(tr.segment(e).squared_length());
     }
   };
 
@@ -720,8 +578,6 @@ namespace CGAL {
     };
   };
 
-
-
   template <>
   struct T2_property_map<boost::vertex_point_t> {
     template <class Gt, class Tds>
@@ -731,7 +587,6 @@ namespace CGAL {
     };
   };
 
-
   template <>
   struct T2_property_map<boost::edge_index_t> {
     template <class Gt, class Tds>
@@ -741,7 +596,6 @@ namespace CGAL {
     };
   };
 
-
   template <>
   struct T2_property_map<boost::edge_weight_t> {
     template <class Gt, class Tds>
@@ -811,6 +665,32 @@ namespace boost {
   };
 } // namespace boost
 
-//#include <CGAL/graph_traits_Delaunay_triangulation_2.h>
+
+namespace std {
+
+
+#if defined(BOOST_MSVC)
+#  pragma warning(push)
+#  pragma warning(disable:4099) // For VC10 it is class hash 
+#endif
+
+#ifndef CGAL_CFG_NO_STD_HASH
+
+  template < class Tr>
+  struct hash<CGAL::detail::T2_halfedge_descriptor<Tr> > {
+    std::size_t operator()(const CGAL::detail::T2_halfedge_descriptor<Tr>& e) const
+    {
+      return hash_value(e);
+    }
+  };
+
+#endif // CGAL_CFG_NO_STD_HASH
+
+#if defined(BOOST_MSVC)
+#  pragma warning(pop)
+#endif
+
+} // namespace std
+
 
 #endif // CGAL_GRAPH_TRAITS_TRIANGULATION_2_H
