@@ -12,31 +12,33 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Point_set_processing_3/include/CGAL/edge_aware_upsample_point_set.h $
+// $Id: edge_aware_upsample_point_set.h 2f81a21 2019-07-28T18:38:59+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s) : Shihao Wu, Clement Jamin, Pierre Alliez 
 
 #ifndef CGAL_UPSAMPLE_POINT_SET_H
 #define CGAL_UPSAMPLE_POINT_SET_H
 
+#include <CGAL/license/Point_set_processing_3.h>
+
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/property_map.h>
 #include <CGAL/point_set_processing_assertions.h>
-#include <CGAL/internal/Rich_grid.h>
-#include <CGAL/Timer.h>
+#include <CGAL/Point_set_processing_3/internal/Rich_grid.h>
+#include <CGAL/Real_timer.h>
 #include <CGAL/Memory_sizer.h>
 #include <CGAL/compute_average_spacing.h>
 
+#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/boost/graph/named_params_helper.h>
+
 #include <iterator>
 #include <set>
 #include <utility>
 
-#include <boost/version.hpp>
-#if BOOST_VERSION >= 104000
-  #include <boost/property_map/property_map.hpp>
-#else
-  #include <boost/property_map.hpp>
-#endif
 
 //#define  CGAL_PSP3_VERBOSE
 
@@ -64,7 +66,7 @@ base_point_selection(
   const rich_grid_internal::Rich_point<Kernel>& query, ///< 3D point to project
   const std::vector<rich_grid_internal::Rich_point<Kernel> >& 
                     neighbor_points,///< neighbor sample points
-  const typename Kernel::FT edge_sensitivity,///< edge senstivity parameter
+  const typename Kernel::FT edge_sensitivity,///< edge sensitivity parameter
   unsigned int& output_base_index ///< base point index
   )
 {
@@ -153,7 +155,7 @@ update_new_point(
   typedef typename Kernel::FT FT;
   typedef typename rich_grid_internal::Rich_point<Kernel> Rich_point;
 
-  CGAL_assertion_code( unsigned int size = rich_point_set.size() );
+  CGAL_assertion_code( unsigned int size = static_cast<unsigned int>(rich_point_set.size()) );
   CGAL_point_set_processing_precondition(father_index >= 0 &&
                                          father_index < size);
   CGAL_point_set_processing_precondition(mother_index >= 0 &&
@@ -183,7 +185,7 @@ update_new_point(
   neighbor_indexes.insert(father_v.index);
   neighbor_indexes.insert(mother_v.index);
 
-  double radius2 = radius * radius;
+  FT radius2 = radius * radius;
 
   new_v.neighbors.clear();
   std::set<int>::iterator set_iter;
@@ -215,7 +217,7 @@ update_new_point(
 
   for (unsigned int i = 0; i < new_v.neighbors.size(); ++i)
   {
-    Rich_point& t = rich_point_set[new_v.neighbors[i]];
+    const Rich_point& t = rich_point_set[new_v.neighbors[i]];
     FT dist2 = CGAL::squared_distance(new_v.pt, t.pt);
     FT theta = std::exp(dist2 * radius16);
 
@@ -233,12 +235,12 @@ update_new_point(
   }
 
   // select best candidate
-  FT min_project_dist = (FT)(std::numeric_limits<double>::max)();
+  FT min_project_dist = (std::numeric_limits<FT>::max)();
   unsigned int best = 0;
 
   for (unsigned int i = 0; i < candidate_num; ++i)
   {
-    FT absolute_dist = abs(project_dist_sum[i] / weight_sum[i]);
+    FT absolute_dist = CGAL::abs(project_dist_sum[i] / weight_sum[i]);
     if (absolute_dist < min_project_dist)
     {
       min_project_dist = absolute_dist;
@@ -278,76 +280,86 @@ update_new_point(
 // Public section
 // ----------------------------------------------------------------------------
 
-/// \ingroup PkgPointSetProcessing
-/// This method progressively upsamples the point set while 
-/// approaching the edge singularities (detected by normal variation), which 
-/// generates a denser point set from an input point set. This has applications 
-/// in point-based rendering, hole filling, and sparse surface reconstruction. 
-/// Normals of points are required as input. For more details, please refer to \cgalCite{ear-2013}.
-/// 
-/// @tparam OutputIterator Type of the output iterator. 
-///         The type of the objects put in it is 
-///         `std::pair<Kernel::Point_3, Kernel::Vector_3>`.
-///         Note that the user may use a 
-///         <A HREF="http://www.boost.org/libs/iterator/doc/function_output_iterator.html">function_output_iterator</A>
-///         to match specific needs.
-/// @tparam ForwardIterator iterator over input points.
-/// @tparam PointPMap is a model of `ReadablePropertyMap` 
-///         with the value type of `ForwardIterator` as key and `Kernel::Point_3` as value type.
-///         It can be omitted if the value type of `ForwardIterator` is convertible to 
-///         `Kernel::Point_3`.
-/// @tparam NormalPMap is a model of `ReadablePropertyMap` with the value type of `ForwardIterator` as key
-///         and `Kernel::Vector_3` as value type.
-/// @tparam Kernel Geometric traits class.
-///      It can be omitted and deduced automatically from the value type of  `PointPMap`
-///      using `Kernel_traits`.
-///
+/**
+   \ingroup PkgPointSetProcessing3Algorithms
+   This method progressively upsamples the point set while 
+   approaching the edge singularities (detected by normal variation), which 
+   generates a denser point set from an input point set. This has applications 
+   in point-based rendering, hole filling, and sparse surface reconstruction. 
+   Normals of points are required as input. For more details, please refer to \cgalCite{ear-2013}.
+ 
+   \tparam ConcurrencyTag enables sequential versus parallel versions
+   of `compute_average_spacing()` (called internally).  Possible
+   values are `Sequential_tag` and `Parallel_tag`.
+   \tparam PointRange is a model of `ConstRange`. The value type of
+   its iterator is the key type of the named parameter `point_map`.
+   \tparam OutputIterator Type of the output iterator. 
+   The type of the objects put in it is 
+   `std::pair<geom_traits::Point_3, geom_traits::Vector_3>`.
+   Note that the user may use a 
+   <A HREF="https://www.boost.org/libs/iterator/doc/function_output_iterator.html">function_output_iterator</A>
+   to match specific needs.
 
-// This variant requires all parameters.
-template <typename OutputIterator,
-          typename ForwardIterator, 
-          typename PointPMap, 
-          typename NormalPMap,
-          typename Kernel>
+   \param points input point range.
+   \param output iterator where output points and normals are put.
+   \param np optional sequence of \ref psp_namedparameters "Named Parameters" among the ones listed below.
+
+   \cgalNamedParamsBegin
+     \cgalParamBegin{point_map} a model of `ReadablePropertyMap` with value type `geom_traits::Point_3`.
+     If this parameter is omitted, `CGAL::Identity_property_map<geom_traits::Point_3>` is used.\cgalParamEnd
+     \cgalParamBegin{normal_map} a model of `ReadablePropertyMap` with value type
+     `geom_traits::Vector_3`.\cgalParamEnd
+     \cgalParamBegin{sharpness_angle} controls the sharpness of the result.\cgalParamEnd
+     \cgalParamBegin{edge_sensitivity} controls the priority of points inserted along sharp features. See
+     section \ref Point_set_processing_3Upsample_Parameter1 for an example.\cgalParamEnd
+     \cgalParamBegin{neighbor_radius} spherical neighborhood radius.\cgalParamEnd
+     \cgalParamBegin{number_of_output_points} is the number of output points to generate.\cgalParamEnd
+     \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `Kernel`\cgalParamEnd
+   \cgalNamedParamsEnd
+
+*/
+template <typename ConcurrencyTag,
+          typename PointRange,
+	  typename OutputIterator,
+          typename NamedParameters>
 OutputIterator
 edge_aware_upsample_point_set(
-  ForwardIterator first,  ///< forward iterator on the first input point.
-  ForwardIterator beyond, ///< past-the-end iterator.
-  OutputIterator output,  ///< output iterator where output points and normals 
-                          ///< are put.
-  PointPMap point_pmap,   ///< point property map.
-  NormalPMap normal_pmap, ///< vector property map.
-  const typename Kernel::FT sharpness_angle,  ///< 
-                    ///< controls the preservation of sharp features. 
-                    ///< The larger the value is,
-                    ///< the smoother the result will be.
-                    ///< The range of possible value is [0, 90].
-                    ///< See section \ref Point_set_processing_3Upsample_Parameter2
-                    ///< for an example.
-  typename Kernel::FT edge_sensitivity,  ///<  
-                    ///< larger values of edge-sensitivity give higher priority 
-                    ///< to inserting points along sharp features.
-                    ///< The range of possible values is `[0, 1]`.
-                    ///< See section \ref Point_set_processing_3Upsample_Parameter1
-                    ///< for an example.
-  typename Kernel::FT neighbor_radius, ///< 
-                    ///< indicates the radius of the largest hole that should be filled.
-                    ///< The default value is set to 3 times the average spacing of the point set.
-                    ///< If the value given by user is smaller than the average spacing, 
-                    ///< the function will use the default value instead.
-  const unsigned int number_of_output_points,///< number of output
-                                             ///< points to generate.
-  const Kernel& /*kernel*/ ///< geometric traits.
-)
+  const PointRange& points,
+  OutputIterator output,
+  const NamedParameters& np)
 {
+  using parameters::choose_parameter;
+  using parameters::get_parameter;
+  
   // basic geometric types
+  typedef typename Point_set_processing_3::GetPointMap<PointRange, NamedParameters>::type PointMap;
+  typedef typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::type NormalMap;
+  typedef typename Point_set_processing_3::GetK<PointRange, NamedParameters>::Kernel Kernel;
+
+  CGAL_static_assertion_msg(!(boost::is_same<NormalMap,
+                              typename Point_set_processing_3::GetNormalMap<PointRange, NamedParameters>::NoMap>::value),
+                            "Error: no normal map");
+  
   typedef typename Kernel::Point_3 Point;
   typedef typename Kernel::Vector_3 Vector;
   typedef typename Kernel::FT FT;
   typedef typename rich_grid_internal::Rich_point<Kernel> Rich_point;
 
+  PointMap point_map = choose_parameter(get_parameter(np, internal_np::point_map), PointMap());
+  NormalMap normal_map = choose_parameter(get_parameter(np, internal_np::normal_map), NormalMap());
+  double sharpness_angle = choose_parameter(get_parameter(np, internal_np::sharpness_angle), 30.);
+  double edge_sensitivity = choose_parameter(get_parameter(np, internal_np::edge_sensitivity), 1);
+  double neighbor_radius = choose_parameter(get_parameter(np, internal_np::neighbor_radius), -1);
+  std::size_t number_of_output_points = choose_parameter(get_parameter(np, internal_np::number_of_output_points), 1000);
+
+  std::cerr << sharpness_angle << " " << edge_sensitivity << " " << neighbor_radius
+            << " " << number_of_output_points << std::endl;
+  // trick in case the output iterator add points to the input container
+  typename PointRange::const_iterator begin = points.begin();
+  typename PointRange::const_iterator end = points.end();
+  
   // preconditions
-  CGAL_point_set_processing_precondition(first != beyond);
+  CGAL_point_set_processing_precondition(begin != end);
   CGAL_point_set_processing_precondition(sharpness_angle >= 0 
                                        &&sharpness_angle <= 90);
   CGAL_point_set_processing_precondition(edge_sensitivity >= 0 
@@ -356,40 +368,33 @@ edge_aware_upsample_point_set(
 
   edge_sensitivity *= 10;  // just project [0, 1] to [0, 10].
 
-  std::size_t number_of_input = std::distance(first, beyond);
+  std::size_t number_of_input = std::distance(begin, end);
   CGAL_point_set_processing_precondition(number_of_output_points > number_of_input);
 
 
   const unsigned int nb_neighbors = 6; // 1 ring
-  FT average_spacing = CGAL::compute_average_spacing(
-                   first, beyond,
-                   point_pmap,
-                   nb_neighbors);
+  FT average_spacing = CGAL::compute_average_spacing<ConcurrencyTag>(
+    points, nb_neighbors, np);
 
-  if (neighbor_radius < average_spacing * 1.0)
+  if (neighbor_radius < average_spacing)
   {
-    neighbor_radius = average_spacing * 3.0;
+    neighbor_radius = average_spacing * 3.0f;
 #ifdef CGAL_PSP3_VERBOSE
     std::cout << "neighbor radius: " << neighbor_radius << std::endl;
 #endif
   }
   
-  Timer task_timer;
+  Real_timer task_timer;
 
   // copy rich point set
   std::vector<Rich_point> rich_point_set(number_of_input);
   CGAL::Bbox_3 bbox(0., 0., 0., 0., 0., 0.);
   
-  ForwardIterator it = first; // point iterator
-  for(unsigned int i = 0; it != beyond; ++it, ++i)
+  typename PointRange::const_iterator it = begin; // point iterator
+  for(unsigned int i = 0; it != end; ++it, ++i)
   {
-#ifdef CGAL_USE_PROPERTY_MAPS_API_V1
-    rich_point_set[i].pt = get(point_pmap, it);
-    rich_point_set[i].normal = get(normal_pmap, it);
-#else
-    rich_point_set[i].pt = get(point_pmap, *it);
-    rich_point_set[i].normal = get(normal_pmap, *it);
-#endif
+    rich_point_set[i].pt = get(point_map, *it);
+    rich_point_set[i].normal = get(normal_map, *it);
 
     rich_point_set[i].index = i;
     bbox += rich_point_set[i].pt.bbox();
@@ -399,16 +404,16 @@ edge_aware_upsample_point_set(
   // compute neighborhood
   rich_grid_internal::compute_ball_neighbors_one_self(rich_point_set,
                                                       bbox,
-                                                      neighbor_radius);
+                                                      FT(neighbor_radius));
 
   //
-  FT cos_sigma = std::cos(sharpness_angle / 180.0 * 3.1415926);
+  FT cos_sigma = static_cast<FT>(std::cos(CGAL::to_double(sharpness_angle) / 180.0 * CGAL_PI));
   FT sharpness_bandwidth = std::pow((CGAL::max)((FT)1e-8, (FT)1.0 - cos_sigma), 2);
 
   FT sum_density = 0.0;
   unsigned int count_density = 1;
   double max_iter_time = 20;
-  FT current_radius = neighbor_radius;
+  FT current_radius = FT(neighbor_radius);
   FT density_pass_threshold = 0.0;
 
   for (unsigned int iter_time = 0; iter_time < max_iter_time; ++iter_time)
@@ -431,7 +436,7 @@ edge_aware_upsample_point_set(
     std::cout << "current radius: " << current_radius << std::endl; 
  #endif
 
-    unsigned int current_size = rich_point_set.size();
+    std::size_t current_size = rich_point_set.size();
     std::vector<bool> is_pass_threshold(current_size, false);
 
     if (iter_time == 0)
@@ -439,7 +444,7 @@ edge_aware_upsample_point_set(
       //estimate density threshold for the first time
       for (unsigned int i = 0; i < rich_point_set.size() * 0.05; ++i)
       {
-        Rich_point& v = rich_point_set[i];
+        const Rich_point& v = rich_point_set[i];
 
         if (v.neighbors.empty())
           continue;
@@ -452,10 +457,10 @@ edge_aware_upsample_point_set(
         }
 
         unsigned int base_index = 0;
-        double density2 = upsample_internal::
+        FT density2 = upsample_internal::
                               base_point_selection(v,
                                                    neighbor_rich_points,
-                                                   edge_sensitivity,
+                                                   FT(edge_sensitivity),
                                                    base_index);
 
         if (density2 < 0)
@@ -468,7 +473,7 @@ edge_aware_upsample_point_set(
       }
     }
 
-    density_pass_threshold = sqrt(sum_density / count_density) * 0.65;
+    density_pass_threshold = static_cast<FT>(sqrt(sum_density / count_density) * FT(0.65));
     sum_density = 0.;
     count_density = 1;
 
@@ -494,7 +499,7 @@ edge_aware_upsample_point_set(
           continue;
         }
 
-        Rich_point& v = rich_point_set[i];
+        const Rich_point& v = rich_point_set[i];
 
         if (v.neighbors.empty())
           continue;
@@ -511,7 +516,7 @@ edge_aware_upsample_point_set(
         FT density2 = upsample_internal::
                               base_point_selection(v,
                                                    neighbor_rich_points,
-                                                   edge_sensitivity,
+                                                   FT(edge_sensitivity),
                                                    base_index);
 
         // test if it pass the density threshold
@@ -533,7 +538,7 @@ edge_aware_upsample_point_set(
         Rich_point& base = rich_point_set[mother_index];
         Vector diff_v_base = base.pt - v.pt;
         new_v.pt = v.pt + (diff_v_base * FT(0.5));
-        new_v.index = rich_point_set.size();
+        new_v.index = static_cast<unsigned int>(rich_point_set.size());
 
         unsigned int new_point_index = new_v.index;
         rich_point_set.push_back(new_v);
@@ -570,9 +575,9 @@ edge_aware_upsample_point_set(
     }
   }
 
-  for (unsigned int i = number_of_input; i < rich_point_set.size(); ++i)
+  for (std::size_t i = number_of_input; i < rich_point_set.size(); ++i)
   {
-    Rich_point& v = rich_point_set[i];
+    const Rich_point& v = rich_point_set[i];
     Point point = v.pt;
     Vector normal = v.normal;
     *output++ = std::make_pair(point, normal);
@@ -582,70 +587,134 @@ edge_aware_upsample_point_set(
 }
 
 
-/// @cond SKIP_IN_MANUAL
-// This variant deduces the kernel from the point property map.
-template <typename OutputIterator,
+/// \cond SKIP_IN_MANUAL
+// variant with default NP
+template <typename ConcurrencyTag,
+          typename PointRange,
+	  typename OutputIterator>
+OutputIterator
+edge_aware_upsample_point_set(
+  const PointRange& points,
+  OutputIterator output)
+{
+  return edge_aware_upsample_point_set<ConcurrencyTag>
+    (points, output, CGAL::Point_set_processing_3::parameters::all_default(points));
+}
+
+
+#ifndef CGAL_NO_DEPRECATED_CODE  
+// deprecated API
+template <typename ConcurrencyTag,
+	  typename OutputIterator,
+          typename ForwardIterator, 
+          typename PointMap, 
+          typename NormalMap,
+          typename Kernel>
+CGAL_DEPRECATED_MSG("you are using the deprecated V1 API of CGAL::edge_aware_upsample_point_set(), please update your code")
+OutputIterator
+edge_aware_upsample_point_set(
+  ForwardIterator first,  ///< forward iterator on the first input point.
+  ForwardIterator beyond, ///< past-the-end iterator.
+  OutputIterator output,  ///< output iterator where output points and normals 
+                          ///< are put.
+  PointMap point_map,   ///< point property map.
+  NormalMap normal_map, ///< vector property map.
+  const typename Kernel::FT sharpness_angle,  ///< 
+                    ///< controls the preservation of sharp features. 
+                    ///< The larger the value is,
+                    ///< the smoother the result will be.
+                    ///< The range of possible values is `[0, 90]`.
+                    ///< See section \ref Point_set_processing_3Upsample_Parameter2
+                    ///< for an example.
+  typename Kernel::FT edge_sensitivity,  ///<  
+                    ///< larger values of edge-sensitivity give higher priority 
+                    ///< to inserting points along sharp features.
+                    ///< The range of possible values is `[0, 1]`.
+                    ///< See section \ref Point_set_processing_3Upsample_Parameter1
+                    ///< for an example.
+  typename Kernel::FT neighbor_radius, ///< 
+                    ///< indicates the radius of the largest hole that should be filled.
+                    ///< The default value is set to 3 times the average spacing of the point set.
+                    ///< If the value given by user is smaller than the average spacing, 
+                    ///< the function will use the default value instead.
+  const std::size_t number_of_output_points,///< number of output
+                                             ///< points to generate.
+  const Kernel& /*kernel*/ ///< geometric traits.
+)
+{
+  return edge_aware_upsample_point_set<ConcurrencyTag>
+    (CGAL::make_range (first, beyond), output,
+     CGAL::parameters::point_map (point_map).
+     normal_map (normal_map).
+     sharpness_angle (sharpness_angle).
+     edge_sensitivity (edge_sensitivity).
+     neighbor_radius (neighbor_radius).
+     number_of_output_points (number_of_output_points).
+     geom_traits (Kernel()));
+}
+  
+
+// deprecated API
+template <typename ConcurrencyTag,
+	  typename OutputIterator,
           typename ForwardIterator,
-          typename PointPMap,
-          typename NormalPMap>
+          typename PointMap,
+          typename NormalMap>
+CGAL_DEPRECATED_MSG("you are using the deprecated V1 API of CGAL::edge_aware_upsample_point_set(), please update your code")
 OutputIterator
 edge_aware_upsample_point_set(
   ForwardIterator first,    ///< forward iterator to the first input point.
   ForwardIterator beyond,   ///< past-the-end iterator.
   OutputIterator output, ///< output iterator over points.
-  PointPMap point_pmap, ///< property map: `ForwardIterator` -> Point_3.
-  NormalPMap normal_pmap, ///< property map: `ForwardIterator` -> Vector_3.
+  PointMap point_map, ///< property map: `ForwardIterator` -> Point_3.
+  NormalMap normal_map, ///< property map: `ForwardIterator` -> Vector_3.
   double sharpness_angle,  ///< control sharpness(0-90)
-  double edge_sensitivity,  ///< edge senstivity(0-5)
+  double edge_sensitivity,  ///< edge sensitivity(0-5)
   double neighbor_radius, ///< initial size of neighbors.
-  const unsigned int number_of_output_points///< number of iterations.   
+  const std::size_t number_of_output_points///< number of iterations.   
   )
 {
-  typedef typename boost::property_traits<PointPMap>::value_type Point;
-  typedef typename Kernel_traits<Point>::Kernel Kernel;
-  return edge_aware_upsample_point_set(
-    first, beyond,
-    output,
-    point_pmap,
-    normal_pmap,
-    sharpness_angle, 
-    edge_sensitivity,
-    neighbor_radius, 
-    number_of_output_points,
-    Kernel());
+  return edge_aware_upsample_point_set<ConcurrencyTag>
+    (CGAL::make_range (first, beyond), output,
+     CGAL::parameters::point_map (point_map).
+     normal_map (normal_map).
+     sharpness_angle (sharpness_angle).
+     edge_sensitivity (edge_sensitivity).
+     neighbor_radius (neighbor_radius).
+     number_of_output_points (number_of_output_points));
 }
-/// @endcond
 
-
-/// @cond SKIP_IN_MANUAL
-template <typename OutputIterator,
+// deprecated API
+template <typename ConcurrencyTag,
+	  typename OutputIterator,
           typename ForwardIterator,
-          typename NormalPMap>
+          typename NormalMap>
+CGAL_DEPRECATED_MSG("you are using the deprecated V1 API of CGAL::edge_aware_upsample_point_set(), please update your code")
 OutputIterator
 edge_aware_upsample_point_set(
   ForwardIterator first, ///< iterator over the first input point
   ForwardIterator beyond, ///< past-the-end iterator
   OutputIterator output, ///< output iterator over points.
-  NormalPMap normal_pmap, ///< property map:  OutputIterator -> Vector_3.
+  NormalMap normal_map, ///< property map:  OutputIterator -> Vector_3.
   double sharpness_angle = 30,  ///< control sharpness(0-90)
-  double edge_sensitivity = 1,  ///< edge senstivity(0-5)
+  double edge_sensitivity = 1,  ///< edge sensitivity(0-5)
   double neighbor_radius = -1, ///< initial size of neighbors.
-  const unsigned int number_of_output_points = 1000///< number of output points.     
+  const std::size_t number_of_output_points = 1000///< number of output points.     
   )
 {
-  // just deduce value_type of OutputIterator
-  return edge_aware_upsample_point_set
-    <typename value_type_traits<OutputIterator>::type>(
-    first, beyond,
-    output,
-    normal_pmap,
-    sharpness_angle, 
-    edge_sensitivity,
-    neighbor_radius, 
-    number_of_output_points);
+  return edge_aware_upsample_point_set<ConcurrencyTag>
+    (CGAL::make_range (first, beyond), output,
+     CGAL::parameters::normal_map (normal_map).
+     sharpness_angle (sharpness_angle).
+     edge_sensitivity (edge_sensitivity).
+     neighbor_radius (neighbor_radius).
+     number_of_output_points (number_of_output_points));
 }
-/// @endcond
+#endif // CGAL_NO_DEPRECATED_CODE
+/// \endcond
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_UPSAMPLE_POINT_SET_H
