@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/AABB_tree/include/CGAL/internal/AABB_tree/AABB_drawing_traits.h $
+// $Id: AABB_drawing_traits.h bc2410d 2018-02-28T11:40:49+01:00 Maxime Gimeno
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     :  Camille Wormser, Pierre Alliez
@@ -21,14 +22,20 @@
 #ifndef CGAL_AABB_DRAWING_TRAITS_H
 #define CGAL_AABB_DRAWING_TRAITS_H
 
+#include <CGAL/license/AABB_tree.h>
+
+
 #include <CGAL/Bbox_3.h>
-#include <CGAL/gl.h>
+#include <vector>
 
 namespace CGAL {
 
 template<typename Primitive, typename Node>
 struct AABB_drawing_traits
 {
+  std::vector<float> *v_edges;
+  double offset[3];
+
   typedef CGAL::Bbox_3 Bbox;
   bool go_further() { return true; }
 
@@ -45,9 +52,8 @@ struct AABB_drawing_traits
   }
 
   // draw bbox
-  static void gl_draw(const Bbox& bb)
+  void gl_draw(const Bbox& bb)
   {
-    ::glBegin(GL_LINES);
     gl_draw_edge(bb.xmin(), bb.ymin(), bb.zmin(),
                  bb.xmax(), bb.ymin(), bb.zmin());
     gl_draw_edge(bb.xmin(), bb.ymin(), bb.zmin(),
@@ -76,14 +82,18 @@ struct AABB_drawing_traits
                  bb.xmax(), bb.ymin(), bb.zmax());
     gl_draw_edge(bb.xmax(), bb.ymax(), bb.zmax(),
                  bb.xmax(), bb.ymax(), bb.zmin());
-    ::glEnd();
   }
 
-  static void gl_draw_edge(double px, double py, double pz,
+  void gl_draw_edge(double px, double py, double pz,
                            double qx, double qy, double qz)
   {
-    ::glVertex3d(px,py,pz);
-    ::glVertex3d(qx,qy,qz);
+      v_edges->push_back((float)px+offset[0]);
+      v_edges->push_back((float)py+offset[1]);
+      v_edges->push_back((float)pz+offset[2]);
+
+      v_edges->push_back((float)qx+offset[0]);
+      v_edges->push_back((float)qy+offset[1]);
+      v_edges->push_back((float)qz+offset[2]);
   }
 }; // AABB_drawing_traits
 
