@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Combinatorial_map/include/CGAL/internal/Combinatorial_map_copy_functors.h $
+// $Id: Combinatorial_map_copy_functors.h 7b9b04f 2018-02-16T11:32:07+01:00 Guillaume Damiand
+// SPDX-License-Identifier: LGPL-3.0+
 //
 // Author(s)     : Guillaume Damiand <guillaume.damiand@liris.cnrs.fr>
 //
@@ -21,10 +22,11 @@
 #define CGAL_COMBINATORIAL_MAP_COPY_FUNCTORS_H
 
 #include <CGAL/internal/Combinatorial_map_utility.h>
-#include <CGAL/internal/Combinatorial_map_internal_functors.h>
+#include <CGAL/Combinatorial_map_functors.h>
 #include <CGAL/Dimension.h>
 #include <CGAL/Kernel_traits.h>
 #include <CGAL/Cartesian_converter.h>
+#include <CGAL/use.h>
 
 /* Definition of functors used internally to copy combinatorial maps attributes
  * (we need functors as attributes are stored in tuple, thus all the access
@@ -36,6 +38,12 @@ template< typename Map1, typename Map2, unsigned int i>
 struct Default_converter_cmap_attributes;
 template< typename Map1, typename Map2>
 struct Default_converter_cmap_0attributes_with_point;
+template< typename Map1, typename Map2>
+struct Cast_converter_dart_info;
+template< typename Map1, typename Map2,
+          typename Info1=typename Map1::Dart_info,
+          typename Info2=typename Map2::Dart_info>
+struct Default_converter_dart_info;
 // ****************************************************************************
 namespace internal
 {
@@ -63,7 +71,7 @@ struct Create_attribute_if_same_info_cmap<Map1, Map2, i, Info, Info>
   run(Map2& map2, const Info& info)
   { typename Map2::template Attribute_handle<i>::type
         res = map2.template create_attribute<i>();
-    map2.template get_attribute<i>(res).info()=info;
+    map2.template info_of_attribute<i>(res)=info;
     return res;
   }
 };
@@ -139,12 +147,12 @@ template<typename Map1, typename Map2, unsigned int i, typename Converters,
 struct Get_convert_attribute_functor
 {
   static typename Map2::template Attribute_handle<i>::type
-  run( const Map1* cmap1, Map2* cmap2, typename Map1::Dart_const_handle dh1,
+  run( const Map1& cmap1, Map2& cmap2, typename Map1::Dart_const_handle dh1,
        typename Map2::Dart_handle dh2, const Converters& /*converters*/)
   {
     return
         CGAL::Default_converter_cmap_attributes<Map1, Map2, i>()
-        (*cmap1, *cmap2, dh1, dh2);
+        (cmap1, cmap2, dh1, dh2);
   }
 };
 
@@ -152,27 +160,28 @@ template<typename Map1, typename Map2, unsigned int i, typename Converters>
 struct Get_convert_attribute_functor<Map1,Map2,i,Converters,false>
 {
   static typename Map2::template Attribute_handle<i>::type
-  run( const Map1* cmap1, Map2* cmap2, typename Map1::Dart_const_handle dh1,
+  run( const Map1& cmap1, Map2& cmap2, typename Map1::Dart_const_handle dh1,
        typename Map2::Dart_handle dh2, const Converters& converters)
   {
-    return CGAL::cpp11::get<i>(converters) (*cmap1, *cmap2, dh1, dh2);
+    return CGAL::cpp11::get<i>(converters) (cmap1, cmap2, dh1, dh2);
   }
 };
 // ****************************************************************************
-// Call a given functor if both i-attribute have an non void info
+// Call a given functor if both i-attribute have an non void info, or both
+// have no info.
 template< typename Map1, typename Map2, unsigned int i,
           typename Converters,
-          bool Withinfo1=CGAL::internal::template
+          bool Withinfo1=CGAL::template
           Is_attribute_has_non_void_info
           <typename Map1::template Attribute_type<i>::type>::value,
-          bool Withinfo2=CGAL::internal::template
+          bool Withinfo2=CGAL::template
           Is_attribute_has_non_void_info
           <typename Map2::template Attribute_type<i>::type>::value >
 struct Call_functor_if_both_attributes_have_info
 {
   static typename Map2::template Attribute_handle<i>::type
-  run( const Map1*,
-       Map2*,
+  run( const Map1&,
+       Map2&,
        typename Map1::Dart_const_handle,
        typename Map2::Dart_handle,
        const Converters&)
@@ -184,8 +193,8 @@ struct Call_functor_if_both_attributes_have_info<Map1, Map2, i,
     Converters, true, true>
 {
   static typename Map2::template Attribute_handle<i>::type
-  run( const Map1* cmap1,
-       Map2* cmap2,
+  run( const Map1& cmap1,
+       Map2& cmap2,
        typename Map1::Dart_const_handle dh1,
        typename Map2::Dart_handle dh2,
        const Converters& converters )
@@ -194,20 +203,35 @@ struct Call_functor_if_both_attributes_have_info<Map1, Map2, i,
         run(cmap1, cmap2, dh1, dh2, converters);
   }
 };
+
+template< typename Map1, typename Map2, unsigned int i, typename Converters >
+struct Call_functor_if_both_attributes_have_info<Map1, Map2, i,
+    Converters, false, false>
+{
+  static typename Map2::template Attribute_handle<i>::type
+  run( const Map1& /*cmap1*/,
+       Map2& cmap2,
+       typename Map1::Dart_const_handle /*dh1*/,
+       typename Map2::Dart_handle /*dh2*/,
+       const Converters& /*converters*/ )
+  {
+    return cmap2.template create_attribute<i>();
+  }
+};
 // ****************************************************************************
 // Call a given functor only if both 0-attribute have a point.
 // general case i!=0 or one attribute without point.
 template< typename Map1, typename Map2, unsigned int i,
           typename Pointconverter,
-          bool Withpoint1=CGAL::internal::template Is_attribute_has_point
+          bool Withpoint1=CGAL::template Is_attribute_has_point
           <typename Map1::template Attribute_type<i>::type>::value,
-          bool Withpoint2=CGAL::internal::template Is_attribute_has_point
+          bool Withpoint2=CGAL::template Is_attribute_has_point
           <typename Map2::template Attribute_type<i>::type>::value >
 struct Call_functor_if_both_attributes_have_point
 {
   static typename Map2::template Attribute_handle<i>::type
-  run( const Map1*,
-       Map2*,
+  run( const Map1&,
+       Map2&,
        typename Map1::Dart_const_handle,
        typename Map2::Dart_handle,
        const Pointconverter&)
@@ -219,12 +243,12 @@ struct Call_functor_if_both_attributes_have_point<Map1, Map2, 0,
     Pointconverter, true, true>
 {
   static typename Map2::template Attribute_handle<0>::type
-  run( const Map1* cmap1,
-       Map2* cmap2,
+  run( const Map1& cmap1,
+       Map2& cmap2,
        typename Map1::Dart_const_handle dh1,
        typename Map2::Dart_handle dh2,
        const Pointconverter& pointconverter )
-  { return pointconverter(*cmap1, *cmap2, dh1, dh2); }
+  { return pointconverter(cmap1, cmap2, dh1, dh2); }
 };
 // ****************************************************************************
 // Copy attribute when if both i-attributes are non void.
@@ -237,27 +261,28 @@ template<typename Map1, typename Map2, typename Converters,
          typename Attr2=typename Map2::template Attribute_type<i>::type >
 struct Copy_attribute_functor_if_nonvoid
 {
-  static void run( const Map1* cmap1,
-                   Map2* cmap2,
+  static void run( const Map1& cmap1,
+                   Map2& cmap2,
                    typename Map1::Dart_const_handle dh1,
                    typename Map2::Dart_handle dh2,
                    const Converters& converters,
                    const Pointconverter& pointconverter)
   {
     // If dh1 has no i-attribute, nothing to copy.
-    if ( cmap1->template attribute<i>(dh1)==Map1::null_handle ) return;
+    if ( cmap1.template attribute<i>(dh1)==Map1::null_handle ) return;
 
     // If dh2 has already an i-attribute, it was already copied.
-    if ( cmap2->template attribute<i>(dh2)!=Map2::null_handle ) return;
+    if ( cmap2.template attribute<i>(dh2)!=Map2::null_handle ) return;
 
-    // Otherwise we copy the info if both attribute have non void info.
+    // Otherwise we copy the attribute if both attributes have non void info,
+    // or if they both have no info.
     typename Map2::template Attribute_handle<i>::type
         res=Call_functor_if_both_attributes_have_info
         <Map1, Map2, i, Converters>::
         run(cmap1, cmap2, dh1, dh2, converters);
 
     if ( res!=Map2::null_handle )
-      cmap2->template set_attribute<i>(dh2, res);
+      cmap2.template set_attribute<i>(dh2, res);
 
     // And the point if both attributes have points (and only for 0-attributes)
     res=Call_functor_if_both_attributes_have_point
@@ -265,8 +290,8 @@ struct Copy_attribute_functor_if_nonvoid
         run(cmap1, cmap2, dh1, dh2, pointconverter);
 
     if ( res!=Map2::null_handle &&
-         cmap2->template attribute<i>(dh2)==Map2::null_handle )
-      cmap2->template set_attribute<i>(dh2, res);
+         cmap2.template attribute<i>(dh2)==Map2::null_handle )
+      cmap2.template set_attribute<i>(dh2, res);
   }
 };
 // Specialisation when attr1 is void, and attr2 is non void i==0. Nothing to
@@ -277,21 +302,21 @@ template<typename Map1, typename Map2, typename Converters,
 struct Copy_attribute_functor_if_nonvoid<Map1, Map2, Converters,
     Pointconverter, 0, CGAL::Void, Attr2>
 {
-  static void run( const Map1*,
-                   Map2* cmap2,
+  static void run( const Map1&,
+                   Map2& cmap2,
                    typename Map1::Dart_const_handle,
                    typename Map2::Dart_handle dh2,
                    const Converters&,
                    const Pointconverter&)
   {
     // If dh2 has already an 0-attribute, it was already created.
-    if ( cmap2->template attribute<0>(dh2)!=Map2::null_handle ) return;
+    if ( cmap2.template attribute<0>(dh2)!=Map2::null_handle ) return;
 
     // Create the point if 0-attributes has Point.
-    if ( CGAL::internal::template Is_attribute_has_point
+    if ( CGAL::template Is_attribute_has_point
          <typename Map2::template Attribute_type<0>::type>::value )
-      cmap2->template
-          set_attribute<0>(dh2, cmap2->template create_attribute<0>());
+      cmap2.template
+          set_attribute<0>(dh2, cmap2.template create_attribute<0>());
   }
 };
 // Specialisation when attr1 is void, and attr2 is non void i!=0.
@@ -301,8 +326,8 @@ template<typename Map1, typename Map2, typename Converters, unsigned int i,
 struct Copy_attribute_functor_if_nonvoid<Map1, Map2, Converters,
     Pointconverter, i, CGAL::Void, Attr2>
 {
-  static void run( const Map1*,
-                   Map2*,
+  static void run( const Map1&,
+                   Map2&,
                    typename Map1::Dart_const_handle,
                    typename Map2::Dart_handle,
                    const Converters&,
@@ -318,8 +343,8 @@ template<typename Map1, typename Map2, typename Converters,
 struct Copy_attributes_functor
 {
   template<unsigned int i>
-  static void run( const Map1* cmap1,
-                   Map2* cmap2,
+  static void run( const Map1& cmap1,
+                   Map2& cmap2,
                    typename Map1::Dart_const_handle dh1,
                    typename Map2::Dart_handle dh2,
                    const Converters& converters,
@@ -330,6 +355,74 @@ struct Copy_attributes_functor
    }
 };
 // ****************************************************************************
+// Copy dart info if they have both the same type.
+// General case with both info different and non void (cf specialization below).
+template<typename Map1, typename Map2, typename DartInfoConverter,
+         typename Info1=typename Map1::Dart_info,
+         typename Info2=typename Map2::Dart_info>
+struct Copy_dart_info_functor_if_nonvoid
+{
+  static void run( const Map1& map1,
+                   Map2& map2,
+                   typename Map1::Dart_const_handle dh1,
+                   typename Map2::Dart_handle dh2,
+                   const DartInfoConverter& converter)
+  { converter(map1, map2, dh1, dh2); }
+};
+// Specialisation when Info1 is void.
+template<typename Map1, typename Map2, typename DartInfoConverter,
+         typename Info2>
+struct Copy_dart_info_functor_if_nonvoid<Map1, Map2, DartInfoConverter,
+    CGAL::Void, Info2>
+{
+  static void run( const Map1&,
+                   Map2&,
+                   typename Map1::Dart_const_handle,
+                   typename Map2::Dart_handle,
+                   const DartInfoConverter&)
+  {}
+};
+// Specialisation when Info2 is void.
+template<typename Map1, typename Map2, typename DartInfoConverter,
+         typename Info1>
+struct Copy_dart_info_functor_if_nonvoid<Map1, Map2, DartInfoConverter,
+    Info1, CGAL::Void>
+{
+  static void run( const Map1&,
+                   Map2&,
+                   typename Map1::Dart_const_handle,
+                   typename Map2::Dart_handle,
+                   const DartInfoConverter&)
+  {}
+};
+// Specialisation when both Info1 and Info2 are void.
+template<typename Map1, typename Map2, typename DartInfoConverter>
+struct Copy_dart_info_functor_if_nonvoid<Map1, Map2, DartInfoConverter,
+    CGAL::Void, CGAL::Void>
+{
+  static void run( const Map1&,
+                   Map2&,
+                   typename Map1::Dart_const_handle,
+                   typename Map2::Dart_handle,
+                   const DartInfoConverter&)
+  {}
+};
+// ****************************************************************************
+/// Copy non void information of dart from one cmap to other.
+/// Map1 is the existing map, to copy into map2.
+template<typename Map1, typename Map2,
+         typename DartInfoConverter=CGAL::Default_converter_dart_info<Map1, Map2> >
+struct Copy_dart_info_functor
+{
+  static void run( const Map1& cmap1, Map2& cmap2,
+                   typename Map1::Dart_const_handle dh1,
+                   typename Map2::Dart_handle dh2,
+                   const DartInfoConverter& converter=DartInfoConverter())
+  { Copy_dart_info_functor_if_nonvoid<Map1, Map2, DartInfoConverter>::
+        run(cmap1, cmap2, dh1, dh2, converter);
+  }
+};
+// ****************************************************************************
 } // namespace internal
 // ****************************************************************************
 // "Converters" called during the copy of attributes, to copy Info.
@@ -346,6 +439,7 @@ struct Default_converter_cmap_attributes
   (const Map1& map1, Map2& map2, typename Map1::Dart_const_handle dh1,
    typename Map2::Dart_handle dh2) const
   {
+    CGAL_USE(dh2);
     CGAL_assertion( map1.template attribute<i>(dh1)!=Map1::null_handle );
     CGAL_assertion( map2.template attribute<i>(dh2)==Map2::null_handle );
     return internal::Create_attribute_if_same_info_cmap
@@ -363,17 +457,48 @@ struct Cast_converter_cmap_attributes
   (const Map1& map1, Map2& map2, typename Map1::Dart_const_handle dh1,
    typename Map2::Dart_handle dh2) const
   {
+    CGAL_USE(dh2);
     CGAL_assertion( map1.template attribute<i>(dh1)!=Map1::null_handle );
     CGAL_assertion( map2.template attribute<i>(dh2)==Map2::null_handle );
     typename Map2::template Attribute_handle<i>::type
       res = map2.template create_attribute<i>();
-    map2.template get_attribute<i>(res).info() =
+    map2.template info_of_attribute<i>(res) =
       (typename Map2::template Attribute_type<i>::type::Info)
         map1.template info<i>(dh1);
     return res;
   }
 };
 // ****************************************************************************
+// Default converter copy only attributes if they have same info types.
+template< typename Map1, typename Map2,
+          typename Info1, typename Info2>
+struct Default_converter_dart_info
+{
+  void operator() (const Map1&, Map2&,
+                   typename Map1::Dart_const_handle,
+                   typename Map2::Dart_handle) const
+  {}
+};
+template< typename Map1, typename Map2, typename Info>
+struct Default_converter_dart_info<Map1, Map2, Info, Info>
+{
+  void operator() (const Map1& map1, Map2& map2,
+                   typename Map1::Dart_const_handle dh1,
+                   typename Map2::Dart_handle dh2) const
+  { map2.info(dh2)=map1.info(dh1); }
+};
+// ****************************************************************************
+// Cast converter of dart info. This can work only if both types are
+// convertible and this is user responsability to use it only in this case.
+template< typename Map1, typename Map2>
+struct Cast_converter_dart_info
+{
+  void operator() (const Map1& map1, Map2& map2,
+                   typename Map1::Dart_const_handle dh1,
+                   typename Map2::Dart_handle dh2) const
+  { map2.info(dh2)=(typename Map2::Dart_info)map1.info(dh1); }
+};
+// ****************************************************************************
 // "Converters" called during the copy of attributes, to copy Point (for
 // attributes having such type defined).
 // Users can replace them by their own converters.
@@ -403,7 +528,7 @@ struct Default_converter_cmap_0attributes_with_point
     internal::Set_point_if_possible_cmap
         <typename Map1::template Attribute_type<0>::type::Point,
         typename Map2::template Attribute_type<0>::type::Point>::
-      run(map1.point(dh1), map2.template get_attribute<0>(res).point());
+      run(map1.point(dh1), map2.point_of_vertex_attribute(res));
     return res;
   }
 };
