@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/TDS_3/include/CGAL/internal/Dummy_tds_3.h $
+// $Id: Dummy_tds_3.h 3c70f8b 2018-03-23T14:17:24+01:00 Laurent Rineau
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Sylvain Pion
@@ -21,6 +22,9 @@
 #ifndef CGAL_INTERNAL_TRIANGULATION_DUMMY_TDS_3_H
 #define CGAL_INTERNAL_TRIANGULATION_DUMMY_TDS_3_H
 
+#include <CGAL/license/TDS_3.h>
+
+
 namespace CGAL { namespace internal {
 
 // Dummy TDS which provides all types that a vertex_base or cell_base can use.
