@@ -12,10 +12,18 @@
 //
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-
+//
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Surface_mesh_segmentation/include/CGAL/internal/Surface_mesh_segmentation/Alpha_expansion_graph_cut.h $
+// $Id: Alpha_expansion_graph_cut.h 560c77f 2019-07-14T22:54:16+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
+//
+// Author(s)     : Ilker O. Yaz
 
 #define CGAL_SURFACE_MESH_SEGMENTATION_ALPHA_EXPANSION_GRAPH_CUT_H
 
+#include <CGAL/license/Surface_mesh_segmentation.h>
+
+
 /// @cond CGAL_DOCUMENT_INTERNAL
 
 /**
@@ -39,6 +47,8 @@
 #endif
 #include <CGAL/trace.h>
 
+#include <CGAL/boost/graph/Named_function_parameters.h>
+
 #include <boost/version.hpp>
 #ifdef CGAL_DO_NOT_USE_BOYKOV_KOLMOGOROV_MAXFLOW_SOFTWARE
 #include <boost/graph/adjacency_list.hpp>
@@ -286,7 +296,7 @@ public:
         cut_time += timer.time();
         #endif
 
-        if(min_cut - flow < flow * tolerance) {
+        if(min_cut - flow <= flow * tolerance) {
           continue;
         }
         min_cut = flow;
@@ -524,7 +534,7 @@ public:
         cut_time += timer.time();
         #endif
 
-        if(min_cut - flow < flow * tolerance) {
+        if(min_cut - flow <= flow * tolerance) {
           continue;
         }
         min_cut = flow;
@@ -654,7 +664,7 @@ public:
         cut_time += timer.time();
         #endif
 
-        if(min_cut - flow < flow * tolerance) {
+        if(min_cut - flow <= flow * tolerance) {
           continue;
         }
 
