@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Partition_2/include/CGAL/is_y_monotone_2.h $
+// $Id: is_y_monotone_2.h ee57fc2 2017-10-21T01:03:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Susan Hert <hert@mpi-sb.mpg.de>
@@ -21,6 +22,9 @@
 #ifndef CGAL_IS_Y_MONOTONE_2_H
 #define CGAL_IS_Y_MONOTONE_2_H
 
+#include <CGAL/license/Partition_2.h>
+
+
 #include <CGAL/circulator.h>
 #include <CGAL/ch_selected_extreme_points_2.h>
 #include <iterator>
