@@ -1,9 +1,10 @@
 // Copyright (c) 1999-2003  ETH Zurich (Switzerland).
 // All rights reserved.
 //
-// This file is part of CGAL (www.cgal.org); you may redistribute it under
-// the terms of the Q Public License version 1.0.
-// See the file LICENSE.QPL distributed with CGAL.
+// This file is part of CGAL (www.cgal.org).
+// You can redistribute it and/or modify it under the terms of the GNU
+// General Public License as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
 // Licensees holding a valid commercial license may use this file in
 // accordance with the commercial license agreement provided with the software.
@@ -11,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/trunk/Min_quadrilateral_2/include/CGAL/min_quadrilateral_2.h $
-// $Id: min_quadrilateral_2.h 59654 2010-11-12 08:30:41Z afabri $
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Bounding_volumes/include/CGAL/min_quadrilateral_2.h $
+// $Id: min_quadrilateral_2.h 26c857a 2018-06-13T15:11:45+02:00 Mael Rouxel-Labbé
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Michael Hoffmann <hoffmann@inf.ethz.ch> and
@@ -21,6 +23,9 @@
 #ifndef CGAL_MIN_QUADRILATERAL_2_H
 #define CGAL_MIN_QUADRILATERAL_2_H 1
 
+#include <CGAL/license/Bounding_volumes.h>
+
+
 #include <CGAL/basic.h>
 #include <CGAL/Optimisation/assertions.h>
 #include <iterator>
@@ -291,7 +296,7 @@ namespace Optimisation {
     
     template < class Kernel >
     class Rdbmop
-    : public std::binary_function< Direction_2, int, Direction_2 >
+    : public CGAL::cpp98::binary_function< Direction_2, int, Direction_2 >
     {
       typename Kernel::Construct_perpendicular_vector_2   cperpvec;
       typename Kernel::Construct_vector_from_direction_2  cvec;
