@@ -16,8 +16,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.2/Number_types/include/CGAL/sse2.h $
+// $Id: sse2.h 0698f79 2017-10-20T23:34:14+02:00 Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     :  Andreas Fabri
@@ -27,9 +28,9 @@
 
 #include <emmintrin.h>
 
-#if defined ( _WIN32 )
+#if defined ( _MSC_VER )
 #define CGAL_ALIGN_16  __declspec(align(16))
-#elif defined( __GNU__ ) || defined(__MINGW64__)
+#elif defined( __GNUC__ )
 #define  CGAL_ALIGN_16 __attribute__((aligned(16))) 
 #endif
 
